#ifndef __FUSION_P_H__
#define __FUSION_P_H__
#include "monty.h"
#include "mosektask_p.h"
#include "list"
#include "vector"
#include "unordered_map"
#include "fusion.h"
namespace mosek
{
namespace fusion
{
// mosek.fusion.BaseModel from file 'src/fusion/cxx/BaseModel_p.h'
// namespace mosek::fusion
struct p_BaseModel
{
  p_BaseModel(BaseModel * _pubthis);

  void _initialize( monty::rc_ptr<BaseModel> m);
  void _initialize( const std::string & name,
                    const std::string & licpath);

  virtual ~p_BaseModel() { /* std::cout << "~p_BaseModel()" << std::endl;*/  }

  static p_BaseModel * _get_impl(Model * _inst) { return _inst->_impl; }

  //----------------------

  bool synched;
  std::string taskname;

  monty::rc_ptr<SolutionStruct> sol_itr;
  monty::rc_ptr<SolutionStruct> sol_itg;
  monty::rc_ptr<SolutionStruct> sol_bas;

  //---------------------

  std::unique_ptr<Task> task;

  //---------------------
  void task_setLogHandler (const msghandler_t & handler);
  void task_setCallbackHandler (const datacbhandler_t & handler);

  int alloc_rangedvar(const std::string & name, double lb, double ub);
  int alloc_linearvar(const std::string & name, mosek::fusion::RelationKey relkey, double bound);
  int task_append_barvar(int size, int num);

  void task_var_name   (int index, const std::string & name);
  void task_con_name   (int index, const std::string & name);
  void task_cone_name  (int index, const std::string & name);
  void task_barvar_name(int index, const std::string & name);
  void task_objectivename(         const std::string & name);

  void task_break_solve();

  //--------------------------

  int task_numvar();
  int task_numcon();
  int task_numcone();
  int task_numbarvar();

  //--------------------------

  void task_put_param(const std::string & name, const std::string & value);
  void task_put_param(const std::string & name, int    value);
  void task_put_param(const std::string & name, double value);
  
  double    task_get_dinf (const std::string & name);
  int       task_get_iinf (const std::string & name);
  long long task_get_liinf(const std::string & name);
  
  //--------------------------
  void task_con_putboundslice_fr(int first, int last); 
  void task_con_putboundslice_lo(int first, int last, const std::shared_ptr<monty::ndarray<double,1>> & rhs);
  void task_con_putboundslice_up(int first, int last, const std::shared_ptr<monty::ndarray<double,1>> & rhs);
  void task_con_putboundslice_ra(int first, int last, const std::shared_ptr<monty::ndarray<double,1>> & lb , 
                                                      const std::shared_ptr<monty::ndarray<double,1>> & ub );
  void task_con_putboundslice_fx(int first, int last, const std::shared_ptr<monty::ndarray<double,1>> & rhs);

  void task_con_putboundlist_lo(const std::shared_ptr<monty::ndarray<int,1>> idxs, const std::shared_ptr<monty::ndarray<double,1>> & rhs);
  void task_con_putboundlist_up(const std::shared_ptr<monty::ndarray<int,1>> idxs, const std::shared_ptr<monty::ndarray<double,1>> & rhs);
  void task_con_putboundlist_fx(const std::shared_ptr<monty::ndarray<int,1>> idxs, const std::shared_ptr<monty::ndarray<double,1>> & rhs);
  void task_con_putboundlist_ra(const std::shared_ptr<monty::ndarray<int,1>> idxs, const std::shared_ptr<monty::ndarray<double,1>> & lb , 
                                                            const std::shared_ptr<monty::ndarray<double,1>> & ub );
  void task_var_putboundslice_fr(int first, int last);
  void task_var_putboundslice_lo(int first, int last, const std::shared_ptr<monty::ndarray<double,1>> & rhs);
  void task_var_putboundslice_up(int first, int last, const std::shared_ptr<monty::ndarray<double,1>> & rhs);
  void task_var_putboundslice_ra(int first, int last, const std::shared_ptr<monty::ndarray<double,1>> & lb, 
                                                      const std::shared_ptr<monty::ndarray<double,1>> & ub);
  void task_var_putboundslice_fx(int first, int last, const std::shared_ptr<monty::ndarray<double,1>> & rhs);
  void task_var_putintlist(const std::shared_ptr<monty::ndarray<int,1>> & idxs);
  void task_var_putcontlist(const std::shared_ptr<monty::ndarray<int,1>> & idxs); 
 
  //--------------------------

  int  task_append_barmatrix
    ( int dim, 
      const std::shared_ptr<monty::ndarray<int,1>>    & subi, 
      const std::shared_ptr<monty::ndarray<int,1>>    & subj, 
      const std::shared_ptr<monty::ndarray<double,1>> & cof);
  int  task_barvar_dim(int j);
  void task_putbaraij (int i, int j, int k);
  void task_putbaraij (int i, int j, const std::shared_ptr<monty::ndarray<int,1>> & k);
  void task_putbarcj  (int j, int k);
  void task_putbarcj  (int j,        const std::shared_ptr<monty::ndarray<int,1>> & k);
  int  task_barvardim (int index);

  int task_append_var(int num);
  int task_append_con(int num);
  int task_append_quadcone (int conesize, int first, int num, int d0, int  d1);
  int task_append_rquadcone(int conesize, int first, int num, int d0, int  d1);

  void task_putarowslice(
    int first, 
    int last, 
    const std::shared_ptr<monty::ndarray<long long,1>> & ptrb, 
    const std::shared_ptr<monty::ndarray<int,1>>       & subj, 
    const std::shared_ptr<monty::ndarray<double,1>>    & cof);
  void task_putaijlist(
    const std::shared_ptr<monty::ndarray<int,1>>       & subi, 
    const std::shared_ptr<monty::ndarray<int,1>>       & subj, 
    const std::shared_ptr<monty::ndarray<double,1>>    & cof, 
    long long                           num);

  void task_setnumvar(int num);
  void task_cleanup(int oldnum, int oldnumcon, int oldnumcone, int oldnumbarvar);
  void task_solve();

  void task_putobjective( 
    bool                             maximize,
    const std::shared_ptr<monty::ndarray<int,1>>    & subj    ,
    const std::shared_ptr<monty::ndarray<double,1>> & cof     ,
    double                           cfix    );

  void task_putobjectivename(const std::string & name);

  void task_write(const std::string & filename);
  void task_dump (const std::string & filename);

  MSKtask_t task_get();
  void dispose();

  void task_putxx_slice(SolutionType which, int first, int last, std::shared_ptr<monty::ndarray<double,1>> & xx);

  static void env_syeig (int n, std::shared_ptr<monty::ndarray<double,1>> & a, std::shared_ptr<monty::ndarray<double,1>> & w);
  static void env_potrf (int n, std::shared_ptr<monty::ndarray<double,1>> & a);                        
  static void env_syevd (int n, std::shared_ptr<monty::ndarray<double,1>> & a, std::shared_ptr<monty::ndarray<double,1>> & w);

  static void env_putlicensecode(std::shared_ptr<monty::ndarray<int,1>> code);
  static void env_putlicensepath(const std::string &licfile);
  static void env_putlicensewait(int wait);

  void convertSolutionStatus(MSKsoltypee soltype, p_SolutionStruct * sol, MSKsolstae status, MSKprostae prosta);


};


// End of file 'src/fusion/cxx/BaseModel_p.h'
struct p_Model : public ::mosek::fusion::p_BaseModel
{
Model * _pubthis;
static mosek::fusion::p_Model* _get_impl(mosek::fusion::Model * _inst){ return static_cast< mosek::fusion::p_Model* >(mosek::fusion::p_BaseModel::_get_impl(_inst)); }
static mosek::fusion::p_Model * _get_impl(mosek::fusion::Model::t _inst) { return _get_impl(_inst.get()); }
p_Model(Model * _pubthis);
virtual ~p_Model() { /* std::cout << "~p_Model" << std::endl;*/ };
int task_vars_used{};int task_vars_allocated{};monty::rc_ptr< ::mosek::fusion::Utils::StringIntMap > con_map{};int cons_used{};std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::ModelConstraint >,1 > > cons{};int vars_used{};std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::ModelVariable >,1 > > vars{};std::shared_ptr< monty::ndarray< bool,1 > > initsol_xx_flag{};std::shared_ptr< monty::ndarray< double,1 > > initsol_xx{};int natbarvarmap_num{};std::shared_ptr< monty::ndarray< int,1 > > natbarvarmap_Var{};monty::rc_ptr< ::mosek::fusion::Utils::StringIntMap > var_map{};int natvarmap_num{};std::shared_ptr< monty::ndarray< long long,1 > > natvarmap_idx{};std::shared_ptr< monty::ndarray< int,1 > > natvarmap_Var{};mosek::fusion::SolutionType solutionptr{};mosek::fusion::AccSolutionStatus acceptable_sol{};std::string model_name{};virtual void destroy();
static Model::t _new_Model(monty::rc_ptr< ::mosek::fusion::Model > _435);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _435);
static Model::t _new_Model();
void _initialize();
static Model::t _new_Model(const std::string &  _442);
void _initialize(const std::string &  _442);
static  void putlicensewait(bool _443);
static  void putlicensepath(const std::string &  _444);
static  void putlicensecode(std::shared_ptr< monty::ndarray< int,1 > > _445);
static  void inst(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _446,int _447,int _448,std::shared_ptr< monty::ndarray< long long,1 > > _449,int _450,std::shared_ptr< monty::ndarray< int,1 > > _451,std::shared_ptr< monty::ndarray< int,1 > > _452,std::shared_ptr< monty::ndarray< int,1 > > _453);
static  void inst(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _463,std::shared_ptr< monty::ndarray< long long,1 > > _464,std::shared_ptr< monty::ndarray< int,1 > > _465,std::shared_ptr< monty::ndarray< int,1 > > _466,std::shared_ptr< monty::ndarray< int,1 > > _467);
virtual void dispose();
virtual void varname(int _470,const std::string &  _471);
virtual void nativeVarToStr(int _472,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _473);
virtual int append_linearvar(monty::rc_ptr< ::mosek::fusion::ModelVariable > _474,long long _475,mosek::fusion::RelationKey _476,double _477);
virtual int append_rangedvar(monty::rc_ptr< ::mosek::fusion::ModelVariable > _479,long long _480,double _481,double _482);
virtual MSKtask_t getTask();
virtual void flushNames();
virtual void writeTask(const std::string &  _486);
virtual long long getSolverLIntInfo(const std::string &  _487);
virtual int getSolverIntInfo(const std::string &  _488);
virtual double getSolverDoubleInfo(const std::string &  _489);
virtual void setCallbackHandler(mosek::datacbhandler_t  _490);
virtual void setLogHandler(mosek::msghandler_t  _491);
virtual void setSolverParam(const std::string &  _492,double _493);
virtual void setSolverParam(const std::string &  _494,int _495);
virtual void setSolverParam(const std::string &  _496,const std::string &  _497);
virtual void breakSolver();
virtual void solve();
virtual void flushSolutions();
virtual void flush_initsol(mosek::fusion::SolutionType _498);
virtual mosek::fusion::SolutionStatus getDualSolutionStatus();
virtual mosek::fusion::SolutionStatus getPrimalSolutionStatus();
virtual double dualObjValue();
virtual double primalObjValue();
virtual monty::rc_ptr< ::mosek::fusion::SolutionStruct > get_sol_cache(mosek::fusion::SolutionType _506,bool _507);
virtual void setSolution_xx(std::shared_ptr< monty::ndarray< int,1 > > _512,std::shared_ptr< monty::ndarray< double,1 > > _513);
virtual void ensure_initsol_xx();
virtual std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< double,1 > >,1 > > getSolution_bars(mosek::fusion::SolutionType _519);
virtual std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< double,1 > >,1 > > getSolution_barx(mosek::fusion::SolutionType _520);
virtual std::shared_ptr< monty::ndarray< double,1 > > getSolution_y(mosek::fusion::SolutionType _521);
virtual std::shared_ptr< monty::ndarray< double,1 > > getSolution_xc(mosek::fusion::SolutionType _522);
virtual std::shared_ptr< monty::ndarray< double,1 > > getSolution_snx(mosek::fusion::SolutionType _523);
virtual std::shared_ptr< monty::ndarray< double,1 > > getSolution_suc(mosek::fusion::SolutionType _524);
virtual std::shared_ptr< monty::ndarray< double,1 > > getSolution_slc(mosek::fusion::SolutionType _525);
virtual std::shared_ptr< monty::ndarray< double,1 > > getSolution_sux(mosek::fusion::SolutionType _526);
virtual std::shared_ptr< monty::ndarray< double,1 > > getSolution_slx(mosek::fusion::SolutionType _527);
virtual std::shared_ptr< monty::ndarray< double,1 > > getSolution_xx(mosek::fusion::SolutionType _528);
virtual void selectedSolution(mosek::fusion::SolutionType _529);
virtual mosek::fusion::AccSolutionStatus getAcceptedSolutionStatus();
virtual void acceptedSolutionStatus(mosek::fusion::AccSolutionStatus _530);
virtual mosek::fusion::ProblemStatus getProblemStatus(mosek::fusion::SolutionType _531);
virtual mosek::fusion::SolutionStatus getDualSolutionStatus(mosek::fusion::SolutionType _533);
virtual mosek::fusion::SolutionStatus getPrimalSolutionStatus(mosek::fusion::SolutionType _534);
virtual mosek::fusion::SolutionStatus getSolutionStatus(mosek::fusion::SolutionType _535,bool _536);
virtual void objective_(const std::string &  _539,mosek::fusion::ObjectiveSense _540,monty::rc_ptr< ::mosek::fusion::Expression > _541);
virtual void objective(double _578);
virtual void objective(mosek::fusion::ObjectiveSense _579,double _580);
virtual void objective(mosek::fusion::ObjectiveSense _581,monty::rc_ptr< ::mosek::fusion::Variable > _582);
virtual void objective(mosek::fusion::ObjectiveSense _583,monty::rc_ptr< ::mosek::fusion::Expression > _584);
virtual void objective(const std::string &  _585,double _586);
virtual void objective(const std::string &  _587,mosek::fusion::ObjectiveSense _588,double _589);
virtual void objective(const std::string &  _590,mosek::fusion::ObjectiveSense _591,monty::rc_ptr< ::mosek::fusion::Variable > _592);
virtual void objective(const std::string &  _593,mosek::fusion::ObjectiveSense _594,monty::rc_ptr< ::mosek::fusion::Expression > _595);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Variable > _596,monty::rc_ptr< ::mosek::fusion::QConeDomain > _597);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _598,monty::rc_ptr< ::mosek::fusion::Variable > _599,monty::rc_ptr< ::mosek::fusion::QConeDomain > _600);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Set > _601,monty::rc_ptr< ::mosek::fusion::Variable > _602,monty::rc_ptr< ::mosek::fusion::QConeDomain > _603);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _604,monty::rc_ptr< ::mosek::fusion::Set > _605,monty::rc_ptr< ::mosek::fusion::Variable > _606,monty::rc_ptr< ::mosek::fusion::QConeDomain > _607);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Variable > _608,monty::rc_ptr< ::mosek::fusion::RangeDomain > _609);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _610,monty::rc_ptr< ::mosek::fusion::Variable > _611,monty::rc_ptr< ::mosek::fusion::RangeDomain > _612);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Set > _613,monty::rc_ptr< ::mosek::fusion::Variable > _614,monty::rc_ptr< ::mosek::fusion::RangeDomain > _615);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _616,monty::rc_ptr< ::mosek::fusion::Set > _617,monty::rc_ptr< ::mosek::fusion::Variable > _618,monty::rc_ptr< ::mosek::fusion::RangeDomain > _619);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Variable > _620,monty::rc_ptr< ::mosek::fusion::LinearDomain > _621);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _622,monty::rc_ptr< ::mosek::fusion::Variable > _623,monty::rc_ptr< ::mosek::fusion::LinearDomain > _624);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Set > _625,monty::rc_ptr< ::mosek::fusion::Variable > _626,monty::rc_ptr< ::mosek::fusion::LinearDomain > _627);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _628,monty::rc_ptr< ::mosek::fusion::Set > _629,monty::rc_ptr< ::mosek::fusion::Variable > _630,monty::rc_ptr< ::mosek::fusion::LinearDomain > _631);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Variable > _632,monty::rc_ptr< ::mosek::fusion::LinPSDDomain > _633);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _634,monty::rc_ptr< ::mosek::fusion::Variable > _635,monty::rc_ptr< ::mosek::fusion::LinPSDDomain > _636);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Variable > _637,monty::rc_ptr< ::mosek::fusion::PSDDomain > _638);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _639,monty::rc_ptr< ::mosek::fusion::Variable > _640,monty::rc_ptr< ::mosek::fusion::PSDDomain > _641);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Expression > _642,monty::rc_ptr< ::mosek::fusion::QConeDomain > _643);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _644,monty::rc_ptr< ::mosek::fusion::Expression > _645,monty::rc_ptr< ::mosek::fusion::QConeDomain > _646);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Set > _647,monty::rc_ptr< ::mosek::fusion::Expression > _648,monty::rc_ptr< ::mosek::fusion::QConeDomain > _649);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _650,monty::rc_ptr< ::mosek::fusion::Set > _651,monty::rc_ptr< ::mosek::fusion::Expression > _652,monty::rc_ptr< ::mosek::fusion::QConeDomain > _653);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Expression > _654,monty::rc_ptr< ::mosek::fusion::RangeDomain > _655);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _656,monty::rc_ptr< ::mosek::fusion::Expression > _657,monty::rc_ptr< ::mosek::fusion::RangeDomain > _658);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Set > _659,monty::rc_ptr< ::mosek::fusion::Expression > _660,monty::rc_ptr< ::mosek::fusion::RangeDomain > _661);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _662,monty::rc_ptr< ::mosek::fusion::Set > _663,monty::rc_ptr< ::mosek::fusion::Expression > _664,monty::rc_ptr< ::mosek::fusion::RangeDomain > _665);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Expression > _666,monty::rc_ptr< ::mosek::fusion::LinearDomain > _667);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _668,monty::rc_ptr< ::mosek::fusion::Expression > _669,monty::rc_ptr< ::mosek::fusion::LinearDomain > _670);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Set > _671,monty::rc_ptr< ::mosek::fusion::Expression > _672,monty::rc_ptr< ::mosek::fusion::LinearDomain > _673);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _674,monty::rc_ptr< ::mosek::fusion::Set > _675,monty::rc_ptr< ::mosek::fusion::Expression > _676,monty::rc_ptr< ::mosek::fusion::LinearDomain > _677);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Expression > _678,monty::rc_ptr< ::mosek::fusion::LinPSDDomain > _679);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _680,monty::rc_ptr< ::mosek::fusion::Expression > _681,monty::rc_ptr< ::mosek::fusion::LinPSDDomain > _682);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(monty::rc_ptr< ::mosek::fusion::Expression > _683,monty::rc_ptr< ::mosek::fusion::PSDDomain > _684);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint(const std::string &  _685,monty::rc_ptr< ::mosek::fusion::Expression > _686,monty::rc_ptr< ::mosek::fusion::PSDDomain > _687);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(monty::rc_ptr< ::mosek::fusion::LinPSDDomain > _688);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(int _689,int _690,monty::rc_ptr< ::mosek::fusion::LinPSDDomain > _691);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(int _692,monty::rc_ptr< ::mosek::fusion::LinPSDDomain > _693);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _694,monty::rc_ptr< ::mosek::fusion::LinPSDDomain > _695);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _696,int _697,int _698,monty::rc_ptr< ::mosek::fusion::LinPSDDomain > _699);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _700,int _701,monty::rc_ptr< ::mosek::fusion::LinPSDDomain > _702);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _703,monty::rc_ptr< ::mosek::fusion::Set > _704,monty::rc_ptr< ::mosek::fusion::LinPSDDomain > _705);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(monty::rc_ptr< ::mosek::fusion::PSDDomain > _706);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(int _707,int _708,monty::rc_ptr< ::mosek::fusion::PSDDomain > _709);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(int _710,monty::rc_ptr< ::mosek::fusion::PSDDomain > _711);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _712,monty::rc_ptr< ::mosek::fusion::PSDDomain > _713);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _714,int _715,int _716,monty::rc_ptr< ::mosek::fusion::PSDDomain > _717);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _718,int _719,monty::rc_ptr< ::mosek::fusion::PSDDomain > _720);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _721,monty::rc_ptr< ::mosek::fusion::Set > _722,monty::rc_ptr< ::mosek::fusion::PSDDomain > _723);
virtual monty::rc_ptr< ::mosek::fusion::SymmetricVariable > variable(int _724,monty::rc_ptr< ::mosek::fusion::SymmetricLinearDomain > _725);
virtual monty::rc_ptr< ::mosek::fusion::SymmetricVariable > variable(const std::string &  _726,int _727,monty::rc_ptr< ::mosek::fusion::SymmetricLinearDomain > _728);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(monty::rc_ptr< ::mosek::fusion::QConeDomain > _729);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(monty::rc_ptr< ::mosek::fusion::RangeDomain > _730);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(monty::rc_ptr< ::mosek::fusion::LinearDomain > _731);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(std::shared_ptr< monty::ndarray< int,1 > > _732,monty::rc_ptr< ::mosek::fusion::RangeDomain > _733);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(std::shared_ptr< monty::ndarray< int,1 > > _734,monty::rc_ptr< ::mosek::fusion::LinearDomain > _735);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(monty::rc_ptr< ::mosek::fusion::Set > _736,monty::rc_ptr< ::mosek::fusion::QConeDomain > _737);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(monty::rc_ptr< ::mosek::fusion::Set > _738,monty::rc_ptr< ::mosek::fusion::RangeDomain > _739);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(monty::rc_ptr< ::mosek::fusion::Set > _740,monty::rc_ptr< ::mosek::fusion::LinearDomain > _741);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(int _742,monty::rc_ptr< ::mosek::fusion::QConeDomain > _743);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(int _744,monty::rc_ptr< ::mosek::fusion::RangeDomain > _745);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(int _746,monty::rc_ptr< ::mosek::fusion::LinearDomain > _747);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(std::shared_ptr< monty::ndarray< int,1 > > _748);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(int _749);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable();
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _750,monty::rc_ptr< ::mosek::fusion::QConeDomain > _751);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _752,monty::rc_ptr< ::mosek::fusion::RangeDomain > _753);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _754,monty::rc_ptr< ::mosek::fusion::LinearDomain > _755);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _756,std::shared_ptr< monty::ndarray< int,1 > > _757,monty::rc_ptr< ::mosek::fusion::RangeDomain > _758);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _759,std::shared_ptr< monty::ndarray< int,1 > > _760,monty::rc_ptr< ::mosek::fusion::LinearDomain > _761);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _762,monty::rc_ptr< ::mosek::fusion::Set > _763,monty::rc_ptr< ::mosek::fusion::QConeDomain > _764);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _765,monty::rc_ptr< ::mosek::fusion::Set > _766,monty::rc_ptr< ::mosek::fusion::RangeDomain > _767);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _768,monty::rc_ptr< ::mosek::fusion::Set > _769,monty::rc_ptr< ::mosek::fusion::LinearDomain > _770);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _771,int _772,monty::rc_ptr< ::mosek::fusion::QConeDomain > _773);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _774,int _775,monty::rc_ptr< ::mosek::fusion::RangeDomain > _776);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _777,int _778,monty::rc_ptr< ::mosek::fusion::LinearDomain > _779);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _780,std::shared_ptr< monty::ndarray< int,1 > > _781);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _782,int _783);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable(const std::string &  _784);
virtual monty::rc_ptr< ::mosek::fusion::Variable > ranged_variable(const std::string &  _785,int _786,monty::rc_ptr< ::mosek::fusion::SymmetricRangeDomain > _787);
virtual monty::rc_ptr< ::mosek::fusion::Variable > ranged_variable(const std::string &  _806,monty::rc_ptr< ::mosek::fusion::Set > _807,monty::rc_ptr< ::mosek::fusion::RangeDomain > _808);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable_(const std::string &  _825,monty::rc_ptr< ::mosek::fusion::Set > _826,monty::rc_ptr< ::mosek::fusion::QConeDomain > _827);
virtual monty::rc_ptr< ::mosek::fusion::SymmetricVariable > variable_(const std::string &  _851,int _852,monty::rc_ptr< ::mosek::fusion::SymmetricLinearDomain > _853);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable_(const std::string &  _871,monty::rc_ptr< ::mosek::fusion::Set > _872,monty::rc_ptr< ::mosek::fusion::LinearDomain > _873);
virtual monty::rc_ptr< ::mosek::fusion::Variable > variable_(const std::string &  _890,monty::rc_ptr< ::mosek::fusion::Set > _891,monty::rc_ptr< ::mosek::fusion::LinPSDDomain > _892);
virtual monty::rc_ptr< ::mosek::fusion::SymmetricVariable > variable_(const std::string &  _900,monty::rc_ptr< ::mosek::fusion::Set > _901,monty::rc_ptr< ::mosek::fusion::PSDDomain > _902);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint_(const std::string &  _907,monty::rc_ptr< ::mosek::fusion::Set > _908,monty::rc_ptr< ::mosek::fusion::Expression > _909,monty::rc_ptr< ::mosek::fusion::RangeDomain > _910);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint_(const std::string &  _933,monty::rc_ptr< ::mosek::fusion::Set > _934,monty::rc_ptr< ::mosek::fusion::Expression > _935,monty::rc_ptr< ::mosek::fusion::QConeDomain > _936);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint_(const std::string &  _971,monty::rc_ptr< ::mosek::fusion::Set > _972,monty::rc_ptr< ::mosek::fusion::Expression > _973,monty::rc_ptr< ::mosek::fusion::LinearDomain > _974);
virtual monty::rc_ptr< ::mosek::fusion::ConNZStruct > build_conA(std::shared_ptr< monty::ndarray< long long,1 > > _995,long long _996,std::shared_ptr< monty::ndarray< long long,1 > > _997,std::shared_ptr< monty::ndarray< long long,1 > > _998,std::shared_ptr< monty::ndarray< double,1 > > _999,std::shared_ptr< monty::ndarray< double,1 > > _1000,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _1001);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint_(const std::string &  _1058,monty::rc_ptr< ::mosek::fusion::Expression > _1059,monty::rc_ptr< ::mosek::fusion::LinPSDDomain > _1060);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > constraint_(const std::string &  _1095,monty::rc_ptr< ::mosek::fusion::Expression > _1096,monty::rc_ptr< ::mosek::fusion::PSDDomain > _1097);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > nonsym_psdconstraint(const std::string &  _1110,monty::rc_ptr< ::mosek::fusion::Expression > _1111,monty::rc_ptr< ::mosek::fusion::PSDDomain > _1112);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > sdptrilcon(const std::string &  _1173,int _1174,int _1175,std::shared_ptr< monty::ndarray< long long,1 > > _1176,std::shared_ptr< monty::ndarray< long long,1 > > _1177,std::shared_ptr< monty::ndarray< long long,1 > > _1178,std::shared_ptr< monty::ndarray< long long,1 > > _1179,std::shared_ptr< monty::ndarray< double,1 > > _1180,std::shared_ptr< monty::ndarray< double,1 > > _1181,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _1182);
virtual void addConstraint(const std::string &  _1253,monty::rc_ptr< ::mosek::fusion::ModelConstraint > _1254);
virtual void addVariable(const std::string &  _1258,monty::rc_ptr< ::mosek::fusion::ModelVariable > _1259);
virtual long long numConstraints();
virtual long long numVariables();
virtual bool hasConstraint(const std::string &  _1263);
virtual bool hasVariable(const std::string &  _1264);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > getConstraint(int _1265);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > getConstraint(const std::string &  _1266);
virtual monty::rc_ptr< ::mosek::fusion::Variable > getVariable(int _1267);
virtual monty::rc_ptr< ::mosek::fusion::Variable > getVariable(const std::string &  _1268);
virtual std::string getName();
virtual monty::rc_ptr< ::mosek::fusion::Model > clone();
virtual void natbarvarmap_ensure(int _1269);
virtual void natvarmap_ensure(int _1274);
virtual int task_alloc_vars(int _1279);
}; // struct Model;

// mosek.fusion.Debug from file 'src/fusion/cxx/Debug_p.h'
// namespace mosek::fusion
struct p_Debug
{
  Debug * _pubthis;

  p_Debug(Debug * _pubthis) : _pubthis(_pubthis) {}

  static Debug::t o ()                 { return monty::rc_ptr<Debug>(new Debug()); }
  Debug::t p (const std::string & val) { std::cout << val; return Debug::t(_pubthis); }
  Debug::t p (      int val)           { std::cout << val; return Debug::t(_pubthis); }
  Debug::t p (long long val)           { std::cout << val; return Debug::t(_pubthis); }
  Debug::t p (   double val)           { std::cout << val; return Debug::t(_pubthis); }
  Debug::t p (     bool val)           { std::cout << val; return Debug::t(_pubthis); }
  Debug::t lf()                        { std::cout << std::endl; return Debug::t(_pubthis); }

  static p_Debug * _get_impl(Debug * _inst) { return _inst->ptr.get(); }

  template<typename T>
  Debug::t p(const std::shared_ptr<monty::ndarray<T,1>> & val)
  {
    if (val->size() > 0)
    {
      std::cout << (*val)[0];
      for (int i = 1; i < val->size(); ++i)
        std::cout << "," << (*val)[i];
    }
    return Debug::t(_pubthis);
  }
};
// End of file 'src/fusion/cxx/Debug_p.h'
struct p_Sort
{
Sort * _pubthis;
static mosek::fusion::p_Sort* _get_impl(mosek::fusion::Sort * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_Sort * _get_impl(mosek::fusion::Sort::t _inst) { return _get_impl(_inst.get()); }
p_Sort(Sort * _pubthis);
virtual ~p_Sort() { /* std::cout << "~p_Sort" << std::endl;*/ };
virtual void destroy();
static  void argTransposeSort(std::shared_ptr< monty::ndarray< long long,1 > > _142,std::shared_ptr< monty::ndarray< long long,1 > > _143,int _144,int _145,int _146,std::shared_ptr< monty::ndarray< long long,1 > > _147);
static  void argsort(std::shared_ptr< monty::ndarray< long long,1 > > _155,std::shared_ptr< monty::ndarray< long long,1 > > _156);
static  void argsort(std::shared_ptr< monty::ndarray< long long,1 > > _157,std::shared_ptr< monty::ndarray< int,1 > > _158);
static  void argsort(std::shared_ptr< monty::ndarray< long long,1 > > _159,std::shared_ptr< monty::ndarray< long long,1 > > _160,std::shared_ptr< monty::ndarray< long long,1 > > _161);
static  void argsort(std::shared_ptr< monty::ndarray< long long,1 > > _162,std::shared_ptr< monty::ndarray< int,1 > > _163,std::shared_ptr< monty::ndarray< int,1 > > _164);
static  void argsort(std::shared_ptr< monty::ndarray< long long,1 > > _165,std::shared_ptr< monty::ndarray< long long,1 > > _166,long long _167,long long _168);
static  void argsort(std::shared_ptr< monty::ndarray< long long,1 > > _169,std::shared_ptr< monty::ndarray< int,1 > > _170,long long _171,long long _172);
static  void argsort(std::shared_ptr< monty::ndarray< long long,1 > > _173,std::shared_ptr< monty::ndarray< long long,1 > > _174,std::shared_ptr< monty::ndarray< long long,1 > > _175,long long _176,long long _177);
static  void argsort(std::shared_ptr< monty::ndarray< long long,1 > > _178,std::shared_ptr< monty::ndarray< int,1 > > _179,std::shared_ptr< monty::ndarray< int,1 > > _180,long long _181,long long _182);
static  void argsort(std::shared_ptr< monty::ndarray< long long,1 > > _183,std::shared_ptr< monty::ndarray< long long,1 > > _184,long long _185,long long _186,bool _187);
static  void argsort(std::shared_ptr< monty::ndarray< long long,1 > > _190,std::shared_ptr< monty::ndarray< int,1 > > _191,long long _192,long long _193,bool _194);
static  void argsort(std::shared_ptr< monty::ndarray< long long,1 > > _197,std::shared_ptr< monty::ndarray< long long,1 > > _198,std::shared_ptr< monty::ndarray< long long,1 > > _199,long long _200,long long _201,bool _202);
static  void argsort(std::shared_ptr< monty::ndarray< long long,1 > > _205,std::shared_ptr< monty::ndarray< int,1 > > _206,std::shared_ptr< monty::ndarray< int,1 > > _207,long long _208,long long _209,bool _210);
static  void argbucketsort(std::shared_ptr< monty::ndarray< long long,1 > > _213,std::shared_ptr< monty::ndarray< long long,1 > > _214,long long _215,long long _216,long long _217,long long _218);
static  void argbucketsort(std::shared_ptr< monty::ndarray< long long,1 > > _219,std::shared_ptr< monty::ndarray< int,1 > > _220,long long _221,long long _222,int _223,int _224);
static  void getminmax(std::shared_ptr< monty::ndarray< long long,1 > > _225,std::shared_ptr< monty::ndarray< long long,1 > > _226,std::shared_ptr< monty::ndarray< long long,1 > > _227,long long _228,long long _229,std::shared_ptr< monty::ndarray< long long,1 > > _230);
static  void getminmax(std::shared_ptr< monty::ndarray< long long,1 > > _233,std::shared_ptr< monty::ndarray< int,1 > > _234,std::shared_ptr< monty::ndarray< int,1 > > _235,long long _236,long long _237,std::shared_ptr< monty::ndarray< int,1 > > _238);
static  bool issorted(std::shared_ptr< monty::ndarray< long long,1 > > _241,std::shared_ptr< monty::ndarray< long long,1 > > _242,long long _243,long long _244,bool _245);
static  bool issorted(std::shared_ptr< monty::ndarray< long long,1 > > _247,std::shared_ptr< monty::ndarray< int,1 > > _248,long long _249,long long _250,bool _251);
static  bool issorted(std::shared_ptr< monty::ndarray< long long,1 > > _253,std::shared_ptr< monty::ndarray< long long,1 > > _254,std::shared_ptr< monty::ndarray< long long,1 > > _255,long long _256,long long _257,bool _258);
static  bool issorted(std::shared_ptr< monty::ndarray< long long,1 > > _260,std::shared_ptr< monty::ndarray< int,1 > > _261,std::shared_ptr< monty::ndarray< int,1 > > _262,long long _263,long long _264,bool _265);
}; // struct Sort;

struct p_IndexCounter
{
IndexCounter * _pubthis;
static mosek::fusion::p_IndexCounter* _get_impl(mosek::fusion::IndexCounter * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_IndexCounter * _get_impl(mosek::fusion::IndexCounter::t _inst) { return _get_impl(_inst.get()); }
p_IndexCounter(IndexCounter * _pubthis);
virtual ~p_IndexCounter() { /* std::cout << "~p_IndexCounter" << std::endl;*/ };
long long start{};std::shared_ptr< monty::ndarray< int,1 > > dims{};std::shared_ptr< monty::ndarray< long long,1 > > strides{};std::shared_ptr< monty::ndarray< long long,1 > > st{};std::shared_ptr< monty::ndarray< int,1 > > ii{};int n{};virtual void destroy();
static IndexCounter::t _new_IndexCounter(monty::rc_ptr< ::mosek::fusion::Set > _267);
void _initialize(monty::rc_ptr< ::mosek::fusion::Set > _267);
static IndexCounter::t _new_IndexCounter(long long _270,std::shared_ptr< monty::ndarray< int,1 > > _271,monty::rc_ptr< ::mosek::fusion::Set > _272);
void _initialize(long long _270,std::shared_ptr< monty::ndarray< int,1 > > _271,monty::rc_ptr< ::mosek::fusion::Set > _272);
static IndexCounter::t _new_IndexCounter(long long _275,std::shared_ptr< monty::ndarray< int,1 > > _276,std::shared_ptr< monty::ndarray< long long,1 > > _277);
void _initialize(long long _275,std::shared_ptr< monty::ndarray< int,1 > > _276,std::shared_ptr< monty::ndarray< long long,1 > > _277);
virtual bool atEnd();
virtual std::shared_ptr< monty::ndarray< int,1 > > getIndex();
virtual long long next();
virtual long long get();
virtual void inc();
virtual void reset();
}; // struct IndexCounter;

struct p_CommonTools
{
CommonTools * _pubthis;
static mosek::fusion::p_CommonTools* _get_impl(mosek::fusion::CommonTools * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_CommonTools * _get_impl(mosek::fusion::CommonTools::t _inst) { return _get_impl(_inst.get()); }
p_CommonTools(CommonTools * _pubthis);
virtual ~p_CommonTools() { /* std::cout << "~p_CommonTools" << std::endl;*/ };
virtual void destroy();
static  void ndIncr(std::shared_ptr< monty::ndarray< int,1 > > _283,std::shared_ptr< monty::ndarray< int,1 > > _284,std::shared_ptr< monty::ndarray< int,1 > > _285);
static  void transposeTriplets(std::shared_ptr< monty::ndarray< int,1 > > _287,std::shared_ptr< monty::ndarray< int,1 > > _288,std::shared_ptr< monty::ndarray< double,1 > > _289,std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< long long,1 > >,1 > > _290,std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< long long,1 > >,1 > > _291,std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< double,1 > >,1 > > _292,long long _293,int _294,int _295);
static  void transposeTriplets(std::shared_ptr< monty::ndarray< int,1 > > _308,std::shared_ptr< monty::ndarray< int,1 > > _309,std::shared_ptr< monty::ndarray< double,1 > > _310,std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< int,1 > >,1 > > _311,std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< int,1 > >,1 > > _312,std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< double,1 > >,1 > > _313,long long _314,int _315,int _316);
static  void tripletSort(std::shared_ptr< monty::ndarray< int,1 > > _329,std::shared_ptr< monty::ndarray< int,1 > > _330,std::shared_ptr< monty::ndarray< double,1 > > _331,std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< int,1 > >,1 > > _332,std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< int,1 > >,1 > > _333,std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< double,1 > >,1 > > _334,long long _335,int _336,int _337);
static  void argMSort(std::shared_ptr< monty::ndarray< int,1 > > _363,std::shared_ptr< monty::ndarray< int,1 > > _364);
static  void mergeInto(std::shared_ptr< monty::ndarray< int,1 > > _369,std::shared_ptr< monty::ndarray< int,1 > > _370,std::shared_ptr< monty::ndarray< int,1 > > _371,int _372,int _373,int _374);
static  void argQsort(std::shared_ptr< monty::ndarray< long long,1 > > _380,std::shared_ptr< monty::ndarray< long long,1 > > _381,std::shared_ptr< monty::ndarray< long long,1 > > _382,long long _383,long long _384);
static  void argQsort(std::shared_ptr< monty::ndarray< long long,1 > > _385,std::shared_ptr< monty::ndarray< int,1 > > _386,std::shared_ptr< monty::ndarray< int,1 > > _387,long long _388,long long _389);
}; // struct CommonTools;

struct p_SolutionStruct
{
SolutionStruct * _pubthis;
static mosek::fusion::p_SolutionStruct* _get_impl(mosek::fusion::SolutionStruct * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_SolutionStruct * _get_impl(mosek::fusion::SolutionStruct::t _inst) { return _get_impl(_inst.get()); }
p_SolutionStruct(SolutionStruct * _pubthis);
virtual ~p_SolutionStruct() { /* std::cout << "~p_SolutionStruct" << std::endl;*/ };
std::shared_ptr< monty::ndarray< double,1 > > snx{};std::shared_ptr< monty::ndarray< double,1 > > sux{};std::shared_ptr< monty::ndarray< double,1 > > slx{};std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< double,1 > >,1 > > bars{};std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< double,1 > >,1 > > barx{};std::shared_ptr< monty::ndarray< double,1 > > y{};std::shared_ptr< monty::ndarray< double,1 > > suc{};std::shared_ptr< monty::ndarray< double,1 > > slc{};std::shared_ptr< monty::ndarray< double,1 > > xx{};std::shared_ptr< monty::ndarray< double,1 > > xc{};double dobj{};double pobj{};mosek::fusion::ProblemStatus probstatus{};mosek::fusion::SolutionStatus dstatus{};mosek::fusion::SolutionStatus pstatus{};int sol_numbarvar{};int sol_numcone{};int sol_numvar{};int sol_numcon{};virtual void destroy();
static SolutionStruct::t _new_SolutionStruct(int _390,int _391,int _392,int _393);
void _initialize(int _390,int _391,int _392,int _393);
static SolutionStruct::t _new_SolutionStruct(monty::rc_ptr< ::mosek::fusion::SolutionStruct > _394);
void _initialize(monty::rc_ptr< ::mosek::fusion::SolutionStruct > _394);
virtual monty::rc_ptr< ::mosek::fusion::SolutionStruct > clone();
virtual void resize(int _397,int _398,int _399,int _400);
virtual bool isDualAcceptable(mosek::fusion::AccSolutionStatus _424);
virtual bool isPrimalAcceptable(mosek::fusion::AccSolutionStatus _425);
virtual bool isAcceptable(mosek::fusion::SolutionStatus _426,mosek::fusion::AccSolutionStatus _427);
}; // struct SolutionStruct;

struct p_ConNZStruct
{
ConNZStruct * _pubthis;
static mosek::fusion::p_ConNZStruct* _get_impl(mosek::fusion::ConNZStruct * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_ConNZStruct * _get_impl(mosek::fusion::ConNZStruct::t _inst) { return _get_impl(_inst.get()); }
p_ConNZStruct(ConNZStruct * _pubthis);
virtual ~p_ConNZStruct() { /* std::cout << "~p_ConNZStruct" << std::endl;*/ };
std::shared_ptr< monty::ndarray< int,1 > > barmidx{};std::shared_ptr< monty::ndarray< int,1 > > barsubj{};std::shared_ptr< monty::ndarray< int,1 > > barsubi{};std::shared_ptr< monty::ndarray< double,1 > > bfix{};std::shared_ptr< monty::ndarray< double,1 > > cof{};std::shared_ptr< monty::ndarray< int,1 > > subj{};std::shared_ptr< monty::ndarray< long long,1 > > ptrb{};virtual void destroy();
static ConNZStruct::t _new_ConNZStruct(std::shared_ptr< monty::ndarray< long long,1 > > _428,std::shared_ptr< monty::ndarray< int,1 > > _429,std::shared_ptr< monty::ndarray< double,1 > > _430,std::shared_ptr< monty::ndarray< double,1 > > _431,std::shared_ptr< monty::ndarray< int,1 > > _432,std::shared_ptr< monty::ndarray< int,1 > > _433,std::shared_ptr< monty::ndarray< int,1 > > _434);
void _initialize(std::shared_ptr< monty::ndarray< long long,1 > > _428,std::shared_ptr< monty::ndarray< int,1 > > _429,std::shared_ptr< monty::ndarray< double,1 > > _430,std::shared_ptr< monty::ndarray< double,1 > > _431,std::shared_ptr< monty::ndarray< int,1 > > _432,std::shared_ptr< monty::ndarray< int,1 > > _433,std::shared_ptr< monty::ndarray< int,1 > > _434);
}; // struct ConNZStruct;

struct p_BaseVariable : public /*implements*/ ::mosek::fusion::Variable
{
BaseVariable * _pubthis;
static mosek::fusion::p_BaseVariable* _get_impl(mosek::fusion::BaseVariable * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_BaseVariable * _get_impl(mosek::fusion::BaseVariable::t _inst) { return _get_impl(_inst.get()); }
p_BaseVariable(BaseVariable * _pubthis);
virtual ~p_BaseVariable() { /* std::cout << "~p_BaseVariable" << std::endl;*/ };
monty::rc_ptr< ::mosek::fusion::Model > model{};monty::rc_ptr< ::mosek::fusion::Set > shape_p{};virtual void destroy();
static BaseVariable::t _new_BaseVariable(monty::rc_ptr< ::mosek::fusion::BaseVariable > _2523,monty::rc_ptr< ::mosek::fusion::Model > _2524);
void _initialize(monty::rc_ptr< ::mosek::fusion::BaseVariable > _2523,monty::rc_ptr< ::mosek::fusion::Model > _2524);
static BaseVariable::t _new_BaseVariable(monty::rc_ptr< ::mosek::fusion::Model > _2525,monty::rc_ptr< ::mosek::fusion::Set > _2526);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _2525,monty::rc_ptr< ::mosek::fusion::Set > _2526);
virtual monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > elementDesc(long long _2527,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2528);
virtual void elementName(long long _2529,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2530) { throw monty::AbstractClassError("Call to abstract method"); }
virtual std::string toString();
virtual void inst(std::shared_ptr< monty::ndarray< long long,1 > > _2536,int _2537,int _2538,long long _2539,long long _2540,std::shared_ptr< monty::ndarray< int,1 > > _2541,std::shared_ptr< monty::ndarray< int,1 > > _2542,std::shared_ptr< monty::ndarray< int,1 > > _2543);
virtual void inst(long long _2545,long long _2546,std::shared_ptr< monty::ndarray< int,1 > > _2547,std::shared_ptr< monty::ndarray< int,1 > > _2548,std::shared_ptr< monty::ndarray< int,1 > > _2549) { throw monty::AbstractClassError("Call to abstract method"); }
virtual void set_values(std::shared_ptr< monty::ndarray< long long,1 > > _2550,std::shared_ptr< monty::ndarray< double,1 > > _2551,bool _2552) { throw monty::AbstractClassError("Call to abstract method"); }
virtual void set_values(long long _2553,std::shared_ptr< monty::ndarray< int,1 > > _2554,std::shared_ptr< monty::ndarray< long long,1 > > _2555,int _2556,std::shared_ptr< monty::ndarray< double,1 > > _2557,bool _2558) { throw monty::AbstractClassError("Call to abstract method"); }
virtual void values(int _2559,std::shared_ptr< monty::ndarray< double,1 > > _2560,bool _2561);
virtual void values(std::shared_ptr< monty::ndarray< long long,1 > > _2566,int _2567,std::shared_ptr< monty::ndarray< double,1 > > _2568,bool _2569) { throw monty::AbstractClassError("Call to abstract method"); }
virtual void values(long long _2570,std::shared_ptr< monty::ndarray< int,1 > > _2571,std::shared_ptr< monty::ndarray< long long,1 > > _2572,int _2573,std::shared_ptr< monty::ndarray< double,1 > > _2574,bool _2575) { throw monty::AbstractClassError("Call to abstract method"); }
virtual void setLevel(std::shared_ptr< monty::ndarray< double,1 > > _2576);
virtual monty::rc_ptr< ::mosek::fusion::Model > getModel();
virtual monty::rc_ptr< ::mosek::fusion::Set > shape();
virtual monty::rc_ptr< ::mosek::fusion::Set > getShape();
virtual long long size();
virtual std::shared_ptr< monty::ndarray< double,1 > > dual();
virtual std::shared_ptr< monty::ndarray< double,1 > > level();
virtual void make_continuous(std::shared_ptr< monty::ndarray< long long,1 > > _2581) { throw monty::AbstractClassError("Call to abstract method"); }
virtual void make_integer(std::shared_ptr< monty::ndarray< long long,1 > > _2582) { throw monty::AbstractClassError("Call to abstract method"); }
virtual void makeContinuous();
virtual void makeInteger();
virtual monty::rc_ptr< ::mosek::fusion::Variable > transpose();
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2585,int _2586,int _2587);
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2588,int _2589);
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(std::shared_ptr< monty::ndarray< int,1 > > _2590);
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2591);
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2592,std::shared_ptr< monty::ndarray< int,1 > > _2593,std::shared_ptr< monty::ndarray< int,1 > > _2594);
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2596,std::shared_ptr< monty::ndarray< int,1 > > _2597);
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,2 > > _2599);
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2602);
virtual monty::rc_ptr< ::mosek::fusion::Variable > antidiag(int _2604);
virtual monty::rc_ptr< ::mosek::fusion::Variable > antidiag();
virtual monty::rc_ptr< ::mosek::fusion::Variable > diag(int _2605);
virtual monty::rc_ptr< ::mosek::fusion::Variable > diag();
virtual monty::rc_ptr< ::mosek::fusion::Variable > general_diag(std::shared_ptr< monty::ndarray< int,1 > > _2606,std::shared_ptr< monty::ndarray< int,1 > > _2607);
virtual monty::rc_ptr< ::mosek::fusion::Expression > asExpr();
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(std::shared_ptr< monty::ndarray< int,1 > > _2620,std::shared_ptr< monty::ndarray< int,1 > > _2621);
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(int _2624,int _2625);
}; // struct BaseVariable;

struct p_CompoundVariable : public ::mosek::fusion::p_BaseVariable
{
CompoundVariable * _pubthis;
static mosek::fusion::p_CompoundVariable* _get_impl(mosek::fusion::CompoundVariable * _inst){ return static_cast< mosek::fusion::p_CompoundVariable* >(mosek::fusion::p_BaseVariable::_get_impl(_inst)); }
static mosek::fusion::p_CompoundVariable * _get_impl(mosek::fusion::CompoundVariable::t _inst) { return _get_impl(_inst.get()); }
p_CompoundVariable(CompoundVariable * _pubthis);
virtual ~p_CompoundVariable() { /* std::cout << "~p_CompoundVariable" << std::endl;*/ };
int stackdim{};std::shared_ptr< monty::ndarray< int,1 > > varsb{};std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > vars{};virtual void destroy();
static CompoundVariable::t _new_CompoundVariable(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _1282,int _1283);
void _initialize(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _1282,int _1283);
virtual monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > elementDesc(long long _1289,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _1290);
virtual void elementName(long long _1293,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _1294);
virtual void inst(long long _1297,long long _1298,std::shared_ptr< monty::ndarray< int,1 > > _1299,std::shared_ptr< monty::ndarray< int,1 > > _1300,std::shared_ptr< monty::ndarray< int,1 > > _1301);
virtual void set_values(long long _1304,std::shared_ptr< monty::ndarray< int,1 > > _1305,std::shared_ptr< monty::ndarray< long long,1 > > _1306,int _1307,std::shared_ptr< monty::ndarray< double,1 > > _1308,bool _1309);
virtual void set_values(std::shared_ptr< monty::ndarray< long long,1 > > _1327,std::shared_ptr< monty::ndarray< double,1 > > _1328,bool _1329);
virtual void values(long long _1337,std::shared_ptr< monty::ndarray< int,1 > > _1338,std::shared_ptr< monty::ndarray< long long,1 > > _1339,int _1340,std::shared_ptr< monty::ndarray< double,1 > > _1341,bool _1342);
virtual void values(std::shared_ptr< monty::ndarray< long long,1 > > _1357,int _1358,std::shared_ptr< monty::ndarray< double,1 > > _1359,bool _1360);
virtual void make_continuous(std::shared_ptr< monty::ndarray< long long,1 > > _1367);
virtual void make_integer(std::shared_ptr< monty::ndarray< long long,1 > > _1374);
virtual monty::rc_ptr< ::mosek::fusion::Expression > asExpr();
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(std::shared_ptr< monty::ndarray< int,1 > > _1401,std::shared_ptr< monty::ndarray< int,1 > > _1402);
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(int _1420,int _1421);
static  monty::rc_ptr< ::mosek::fusion::Set > compute_shape(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _1429,int _1430);
static  monty::rc_ptr< ::mosek::fusion::Model > model_from_var(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _1438);
}; // struct CompoundVariable;

struct p_PickVariable : public ::mosek::fusion::p_BaseVariable
{
PickVariable * _pubthis;
static mosek::fusion::p_PickVariable* _get_impl(mosek::fusion::PickVariable * _inst){ return static_cast< mosek::fusion::p_PickVariable* >(mosek::fusion::p_BaseVariable::_get_impl(_inst)); }
static mosek::fusion::p_PickVariable * _get_impl(mosek::fusion::PickVariable::t _inst) { return _get_impl(_inst.get()); }
p_PickVariable(PickVariable * _pubthis);
virtual ~p_PickVariable() { /* std::cout << "~p_PickVariable" << std::endl;*/ };
std::shared_ptr< monty::ndarray< long long,1 > > indexes{};monty::rc_ptr< ::mosek::fusion::Variable > origin{};virtual void destroy();
static PickVariable::t _new_PickVariable(monty::rc_ptr< ::mosek::fusion::Variable > _1439,std::shared_ptr< monty::ndarray< long long,1 > > _1440);
void _initialize(monty::rc_ptr< ::mosek::fusion::Variable > _1439,std::shared_ptr< monty::ndarray< long long,1 > > _1440);
virtual void inst(long long _1443,long long _1444,std::shared_ptr< monty::ndarray< int,1 > > _1445,std::shared_ptr< monty::ndarray< int,1 > > _1446,std::shared_ptr< monty::ndarray< int,1 > > _1447);
virtual void set_values(long long _1448,std::shared_ptr< monty::ndarray< int,1 > > _1449,std::shared_ptr< monty::ndarray< long long,1 > > _1450,int _1451,std::shared_ptr< monty::ndarray< double,1 > > _1452,bool _1453);
virtual void set_values(std::shared_ptr< monty::ndarray< long long,1 > > _1456,std::shared_ptr< monty::ndarray< double,1 > > _1457,bool _1458);
virtual monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > elementDesc(long long _1460,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _1461);
virtual void elementName(long long _1462,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _1463);
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(int _1464,int _1465);
virtual void values(long long _1467,std::shared_ptr< monty::ndarray< int,1 > > _1468,std::shared_ptr< monty::ndarray< long long,1 > > _1469,int _1470,std::shared_ptr< monty::ndarray< double,1 > > _1471,bool _1472);
virtual void values(std::shared_ptr< monty::ndarray< long long,1 > > _1475,int _1476,std::shared_ptr< monty::ndarray< double,1 > > _1477,bool _1478);
virtual void make_continuous(std::shared_ptr< monty::ndarray< long long,1 > > _1481);
virtual void make_integer(std::shared_ptr< monty::ndarray< long long,1 > > _1483);
}; // struct PickVariable;

struct p_SliceVariable : public ::mosek::fusion::p_BaseVariable
{
SliceVariable * _pubthis;
static mosek::fusion::p_SliceVariable* _get_impl(mosek::fusion::SliceVariable * _inst){ return static_cast< mosek::fusion::p_SliceVariable* >(mosek::fusion::p_BaseVariable::_get_impl(_inst)); }
static mosek::fusion::p_SliceVariable * _get_impl(mosek::fusion::SliceVariable::t _inst) { return _get_impl(_inst.get()); }
p_SliceVariable(SliceVariable * _pubthis);
virtual ~p_SliceVariable() { /* std::cout << "~p_SliceVariable" << std::endl;*/ };
std::shared_ptr< monty::ndarray< long long,1 > > strides{};long long first{};monty::rc_ptr< ::mosek::fusion::Variable > origin{};virtual void destroy();
static SliceVariable::t _new_SliceVariable(monty::rc_ptr< ::mosek::fusion::Variable > _1485,monty::rc_ptr< ::mosek::fusion::Set > _1486,long long _1487,std::shared_ptr< monty::ndarray< long long,1 > > _1488);
void _initialize(monty::rc_ptr< ::mosek::fusion::Variable > _1485,monty::rc_ptr< ::mosek::fusion::Set > _1486,long long _1487,std::shared_ptr< monty::ndarray< long long,1 > > _1488);
virtual void inst(long long _1489,long long _1490,std::shared_ptr< monty::ndarray< int,1 > > _1491,std::shared_ptr< monty::ndarray< int,1 > > _1492,std::shared_ptr< monty::ndarray< int,1 > > _1493);
virtual void set_values(long long _1498,std::shared_ptr< monty::ndarray< int,1 > > _1499,std::shared_ptr< monty::ndarray< long long,1 > > _1500,int _1501,std::shared_ptr< monty::ndarray< double,1 > > _1502,bool _1503);
virtual void set_values(std::shared_ptr< monty::ndarray< long long,1 > > _1522,std::shared_ptr< monty::ndarray< double,1 > > _1523,bool _1524);
virtual monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > elementDesc(long long _1530,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _1531);
virtual void elementName(long long _1536,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _1537);
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(std::shared_ptr< monty::ndarray< int,1 > > _1542,std::shared_ptr< monty::ndarray< int,1 > > _1543);
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(int _1547,int _1548);
virtual void values(long long _1549,std::shared_ptr< monty::ndarray< int,1 > > _1550,std::shared_ptr< monty::ndarray< long long,1 > > _1551,int _1552,std::shared_ptr< monty::ndarray< double,1 > > _1553,bool _1554);
virtual void values(std::shared_ptr< monty::ndarray< long long,1 > > _1572,int _1573,std::shared_ptr< monty::ndarray< double,1 > > _1574,bool _1575);
virtual void make_continuous(std::shared_ptr< monty::ndarray< long long,1 > > _1581);
virtual void make_integer(std::shared_ptr< monty::ndarray< long long,1 > > _1587);
}; // struct SliceVariable;

struct p_BoundInterfaceVariable : public ::mosek::fusion::p_SliceVariable
{
BoundInterfaceVariable * _pubthis;
static mosek::fusion::p_BoundInterfaceVariable* _get_impl(mosek::fusion::BoundInterfaceVariable * _inst){ return static_cast< mosek::fusion::p_BoundInterfaceVariable* >(mosek::fusion::p_SliceVariable::_get_impl(_inst)); }
static mosek::fusion::p_BoundInterfaceVariable * _get_impl(mosek::fusion::BoundInterfaceVariable::t _inst) { return _get_impl(_inst.get()); }
p_BoundInterfaceVariable(BoundInterfaceVariable * _pubthis);
virtual ~p_BoundInterfaceVariable() { /* std::cout << "~p_BoundInterfaceVariable" << std::endl;*/ };
monty::rc_ptr< ::mosek::fusion::RangedVariable > originvar{};bool islower{};virtual void destroy();
static BoundInterfaceVariable::t _new_BoundInterfaceVariable(monty::rc_ptr< ::mosek::fusion::RangedVariable > _2361,monty::rc_ptr< ::mosek::fusion::Set > _2362,long long _2363,std::shared_ptr< monty::ndarray< long long,1 > > _2364,bool _2365);
void _initialize(monty::rc_ptr< ::mosek::fusion::RangedVariable > _2361,monty::rc_ptr< ::mosek::fusion::Set > _2362,long long _2363,std::shared_ptr< monty::ndarray< long long,1 > > _2364,bool _2365);
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice_(monty::rc_ptr< ::mosek::fusion::Set > _2366,long long _2367,std::shared_ptr< monty::ndarray< long long,1 > > _2368);
virtual void dual_values(long long _2369,std::shared_ptr< monty::ndarray< int,1 > > _2370,std::shared_ptr< monty::ndarray< long long,1 > > _2371,int _2372,std::shared_ptr< monty::ndarray< double,1 > > _2373);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _2374,int _2375,std::shared_ptr< monty::ndarray< double,1 > > _2376);
}; // struct BoundInterfaceVariable;

struct p_ModelVariable : public ::mosek::fusion::p_BaseVariable
{
ModelVariable * _pubthis;
static mosek::fusion::p_ModelVariable* _get_impl(mosek::fusion::ModelVariable * _inst){ return static_cast< mosek::fusion::p_ModelVariable* >(mosek::fusion::p_BaseVariable::_get_impl(_inst)); }
static mosek::fusion::p_ModelVariable * _get_impl(mosek::fusion::ModelVariable::t _inst) { return _get_impl(_inst.get()); }
p_ModelVariable(ModelVariable * _pubthis);
virtual ~p_ModelVariable() { /* std::cout << "~p_ModelVariable" << std::endl;*/ };
long long varid{};std::string name{};virtual void destroy();
static ModelVariable::t _new_ModelVariable(monty::rc_ptr< ::mosek::fusion::ModelVariable > _2457,monty::rc_ptr< ::mosek::fusion::Model > _2458);
void _initialize(monty::rc_ptr< ::mosek::fusion::ModelVariable > _2457,monty::rc_ptr< ::mosek::fusion::Model > _2458);
static ModelVariable::t _new_ModelVariable(monty::rc_ptr< ::mosek::fusion::Model > _2459,const std::string &  _2460,monty::rc_ptr< ::mosek::fusion::Set > _2461,long long _2462);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _2459,const std::string &  _2460,monty::rc_ptr< ::mosek::fusion::Set > _2461,long long _2462);
virtual void flushNames() { throw monty::AbstractClassError("Call to abstract method"); }
virtual void elementName(long long _2463,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2464);
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(std::shared_ptr< monty::ndarray< int,1 > > _2465,std::shared_ptr< monty::ndarray< int,1 > > _2466);
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(int _2472,int _2473);
virtual monty::rc_ptr< ::mosek::fusion::ModelVariable > clone(monty::rc_ptr< ::mosek::fusion::Model > _2475) { throw monty::AbstractClassError("Call to abstract method"); }
}; // struct ModelVariable;

struct p_SymRangedVariable : public ::mosek::fusion::p_ModelVariable, public /*implements*/ ::mosek::fusion::SymmetricVariable
{
SymRangedVariable * _pubthis;
static mosek::fusion::p_SymRangedVariable* _get_impl(mosek::fusion::SymRangedVariable * _inst){ return static_cast< mosek::fusion::p_SymRangedVariable* >(mosek::fusion::p_ModelVariable::_get_impl(_inst)); }
static mosek::fusion::p_SymRangedVariable * _get_impl(mosek::fusion::SymRangedVariable::t _inst) { return _get_impl(_inst.get()); }
p_SymRangedVariable(SymRangedVariable * _pubthis);
virtual ~p_SymRangedVariable() { /* std::cout << "~p_SymRangedVariable" << std::endl;*/ };
int dim{};bool names_flushed{};std::shared_ptr< monty::ndarray< int,1 > > nativeidxs{};monty::rc_ptr< ::mosek::fusion::RangeDomain > dom{};monty::rc_ptr< ::mosek::fusion::Utils::IntMap > idxmap{};virtual void destroy();
static SymRangedVariable::t _new_SymRangedVariable(monty::rc_ptr< ::mosek::fusion::SymRangedVariable > _1593,monty::rc_ptr< ::mosek::fusion::Model > _1594);
void _initialize(monty::rc_ptr< ::mosek::fusion::SymRangedVariable > _1593,monty::rc_ptr< ::mosek::fusion::Model > _1594);
static SymRangedVariable::t _new_SymRangedVariable(monty::rc_ptr< ::mosek::fusion::Model > _1596,const std::string &  _1597,monty::rc_ptr< ::mosek::fusion::RangeDomain > _1598,int _1599,std::shared_ptr< monty::ndarray< int,1 > > _1600,long long _1601);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _1596,const std::string &  _1597,monty::rc_ptr< ::mosek::fusion::RangeDomain > _1598,int _1599,std::shared_ptr< monty::ndarray< int,1 > > _1600,long long _1601);
virtual std::string toString();
virtual void flushNames();
virtual void inst(std::shared_ptr< monty::ndarray< long long,1 > > _1608,int _1609,int _1610,long long _1611,long long _1612,std::shared_ptr< monty::ndarray< int,1 > > _1613,std::shared_ptr< monty::ndarray< int,1 > > _1614,std::shared_ptr< monty::ndarray< int,1 > > _1615);
virtual void inst(long long _1621,long long _1622,std::shared_ptr< monty::ndarray< int,1 > > _1623,std::shared_ptr< monty::ndarray< int,1 > > _1624,std::shared_ptr< monty::ndarray< int,1 > > _1625);
virtual void dual_u(long long _1630,std::shared_ptr< monty::ndarray< int,1 > > _1631,std::shared_ptr< monty::ndarray< long long,1 > > _1632,int _1633,std::shared_ptr< monty::ndarray< double,1 > > _1634);
virtual void dual_u(std::shared_ptr< monty::ndarray< long long,1 > > _1646,int _1647,std::shared_ptr< monty::ndarray< double,1 > > _1648);
virtual void dual_l(long long _1656,std::shared_ptr< monty::ndarray< int,1 > > _1657,std::shared_ptr< monty::ndarray< long long,1 > > _1658,int _1659,std::shared_ptr< monty::ndarray< double,1 > > _1660);
virtual void dual_l(std::shared_ptr< monty::ndarray< long long,1 > > _1672,int _1673,std::shared_ptr< monty::ndarray< double,1 > > _1674);
virtual void dual_values(long long _1681,std::shared_ptr< monty::ndarray< int,1 > > _1682,std::shared_ptr< monty::ndarray< long long,1 > > _1683,int _1684,std::shared_ptr< monty::ndarray< double,1 > > _1685);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _1697,int _1698,std::shared_ptr< monty::ndarray< double,1 > > _1699);
virtual void set_values(long long _1707,std::shared_ptr< monty::ndarray< int,1 > > _1708,std::shared_ptr< monty::ndarray< long long,1 > > _1709,int _1710,std::shared_ptr< monty::ndarray< double,1 > > _1711,bool _1712);
virtual void set_values(std::shared_ptr< monty::ndarray< long long,1 > > _1726,std::shared_ptr< monty::ndarray< double,1 > > _1727,bool _1728);
virtual void values(long long _1738,std::shared_ptr< monty::ndarray< int,1 > > _1739,std::shared_ptr< monty::ndarray< long long,1 > > _1740,int _1741,std::shared_ptr< monty::ndarray< double,1 > > _1742,bool _1743);
virtual void values(std::shared_ptr< monty::ndarray< long long,1 > > _1752,int _1753,std::shared_ptr< monty::ndarray< double,1 > > _1754,bool _1755);
virtual long long tril_idx(long long _1762);
virtual long long tril_lin_idx(long long _1765);
virtual void make_continuous(std::shared_ptr< monty::ndarray< long long,1 > > _1768);
virtual void make_integer(std::shared_ptr< monty::ndarray< long long,1 > > _1771);
virtual monty::rc_ptr< ::mosek::fusion::ModelVariable > clone(monty::rc_ptr< ::mosek::fusion::Model > _1774);
virtual monty::rc_ptr< ::mosek::fusion::Expression > asExpr() /*override*/
{ return mosek::fusion::p_BaseVariable::asExpr(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(int _2472,int _2473) /*override*/
{ return mosek::fusion::p_ModelVariable::slice(_2472,_2473); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,2 > > _2599) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2599); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2602) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2602); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > antidiag() /*override*/
{ return mosek::fusion::p_BaseVariable::antidiag(); }
virtual void makeContinuous() /*override*/
{ mosek::fusion::p_BaseVariable::makeContinuous(); }
virtual monty::rc_ptr< ::mosek::fusion::Set > shape() /*override*/
{ return mosek::fusion::p_BaseVariable::shape(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2592,std::shared_ptr< monty::ndarray< int,1 > > _2593,std::shared_ptr< monty::ndarray< int,1 > > _2594) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2592,_2593,_2594); }
virtual void elementName(long long _2463,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2464) /*override*/
{ mosek::fusion::p_ModelVariable::elementName(_2463,_2464); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2591) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2591); }
virtual void makeInteger() /*override*/
{ mosek::fusion::p_BaseVariable::makeInteger(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2585,int _2586,int _2587) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2585,_2586,_2587); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > diag() /*override*/
{ return mosek::fusion::p_BaseVariable::diag(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2588,int _2589) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2588,_2589); }
virtual monty::rc_ptr< ::mosek::fusion::Set > getShape() /*override*/
{ return mosek::fusion::p_BaseVariable::getShape(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > transpose() /*override*/
{ return mosek::fusion::p_BaseVariable::transpose(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(std::shared_ptr< monty::ndarray< int,1 > > _2590) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2590); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2596,std::shared_ptr< monty::ndarray< int,1 > > _2597) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2596,_2597); }
virtual std::shared_ptr< monty::ndarray< double,1 > > level() /*override*/
{ return mosek::fusion::p_BaseVariable::level(); }
virtual monty::rc_ptr< ::mosek::fusion::Model > getModel() /*override*/
{ return mosek::fusion::p_BaseVariable::getModel(); }
virtual void setLevel(std::shared_ptr< monty::ndarray< double,1 > > _2576) /*override*/
{ mosek::fusion::p_BaseVariable::setLevel(_2576); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > diag(int _2605) /*override*/
{ return mosek::fusion::p_BaseVariable::diag(_2605); }
virtual std::shared_ptr< monty::ndarray< double,1 > > dual() /*override*/
{ return mosek::fusion::p_BaseVariable::dual(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(std::shared_ptr< monty::ndarray< int,1 > > _2465,std::shared_ptr< monty::ndarray< int,1 > > _2466) /*override*/
{ return mosek::fusion::p_ModelVariable::slice(_2465,_2466); }
virtual monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > elementDesc(long long _2527,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2528) /*override*/
{ return mosek::fusion::p_BaseVariable::elementDesc(_2527,_2528); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > antidiag(int _2604) /*override*/
{ return mosek::fusion::p_BaseVariable::antidiag(_2604); }
virtual long long size() /*override*/
{ return mosek::fusion::p_BaseVariable::size(); }
virtual void values(int _2559,std::shared_ptr< monty::ndarray< double,1 > > _2560,bool _2561) /*override*/
{ mosek::fusion::p_BaseVariable::values(_2559,_2560,_2561); }
}; // struct SymRangedVariable;

struct p_RangedVariable : public ::mosek::fusion::p_ModelVariable
{
RangedVariable * _pubthis;
static mosek::fusion::p_RangedVariable* _get_impl(mosek::fusion::RangedVariable * _inst){ return static_cast< mosek::fusion::p_RangedVariable* >(mosek::fusion::p_ModelVariable::_get_impl(_inst)); }
static mosek::fusion::p_RangedVariable * _get_impl(mosek::fusion::RangedVariable::t _inst) { return _get_impl(_inst.get()); }
p_RangedVariable(RangedVariable * _pubthis);
virtual ~p_RangedVariable() { /* std::cout << "~p_RangedVariable" << std::endl;*/ };
bool names_flushed{};std::shared_ptr< monty::ndarray< int,1 > > nativeidxs{};monty::rc_ptr< ::mosek::fusion::RangeDomain > dom{};monty::rc_ptr< ::mosek::fusion::Utils::IntMap > idxmap{};virtual void destroy();
static RangedVariable::t _new_RangedVariable(monty::rc_ptr< ::mosek::fusion::RangedVariable > _1775,monty::rc_ptr< ::mosek::fusion::Model > _1776);
void _initialize(monty::rc_ptr< ::mosek::fusion::RangedVariable > _1775,monty::rc_ptr< ::mosek::fusion::Model > _1776);
static RangedVariable::t _new_RangedVariable(monty::rc_ptr< ::mosek::fusion::Model > _1778,const std::string &  _1779,monty::rc_ptr< ::mosek::fusion::Set > _1780,monty::rc_ptr< ::mosek::fusion::RangeDomain > _1781,std::shared_ptr< monty::ndarray< int,1 > > _1782,long long _1783);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _1778,const std::string &  _1779,monty::rc_ptr< ::mosek::fusion::Set > _1780,monty::rc_ptr< ::mosek::fusion::RangeDomain > _1781,std::shared_ptr< monty::ndarray< int,1 > > _1782,long long _1783);
virtual monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > elementDesc(long long _1784,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _1785);
virtual void flushNames();
virtual void inst(std::shared_ptr< monty::ndarray< long long,1 > > _1790,int _1791,int _1792,long long _1793,long long _1794,std::shared_ptr< monty::ndarray< int,1 > > _1795,std::shared_ptr< monty::ndarray< int,1 > > _1796,std::shared_ptr< monty::ndarray< int,1 > > _1797);
virtual void inst(long long _1801,long long _1802,std::shared_ptr< monty::ndarray< int,1 > > _1803,std::shared_ptr< monty::ndarray< int,1 > > _1804,std::shared_ptr< monty::ndarray< int,1 > > _1805);
virtual monty::rc_ptr< ::mosek::fusion::Variable > upperBoundVar();
virtual monty::rc_ptr< ::mosek::fusion::Variable > lowerBoundVar();
virtual void dual_u(long long _1812,std::shared_ptr< monty::ndarray< int,1 > > _1813,std::shared_ptr< monty::ndarray< long long,1 > > _1814,int _1815,std::shared_ptr< monty::ndarray< double,1 > > _1816);
virtual void dual_u(std::shared_ptr< monty::ndarray< long long,1 > > _1827,int _1828,std::shared_ptr< monty::ndarray< double,1 > > _1829);
virtual void dual_l(long long _1836,std::shared_ptr< monty::ndarray< int,1 > > _1837,std::shared_ptr< monty::ndarray< long long,1 > > _1838,int _1839,std::shared_ptr< monty::ndarray< double,1 > > _1840);
virtual void dual_l(std::shared_ptr< monty::ndarray< long long,1 > > _1851,int _1852,std::shared_ptr< monty::ndarray< double,1 > > _1853);
virtual void dual_values(long long _1860,std::shared_ptr< monty::ndarray< int,1 > > _1861,std::shared_ptr< monty::ndarray< long long,1 > > _1862,int _1863,std::shared_ptr< monty::ndarray< double,1 > > _1864);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _1876,int _1877,std::shared_ptr< monty::ndarray< double,1 > > _1878);
virtual void set_values(long long _1886,std::shared_ptr< monty::ndarray< int,1 > > _1887,std::shared_ptr< monty::ndarray< long long,1 > > _1888,int _1889,std::shared_ptr< monty::ndarray< double,1 > > _1890,bool _1891);
virtual void set_values(std::shared_ptr< monty::ndarray< long long,1 > > _1905,std::shared_ptr< monty::ndarray< double,1 > > _1906,bool _1907);
virtual void values(long long _1917,std::shared_ptr< monty::ndarray< int,1 > > _1918,std::shared_ptr< monty::ndarray< long long,1 > > _1919,int _1920,std::shared_ptr< monty::ndarray< double,1 > > _1921,bool _1922);
virtual void values(std::shared_ptr< monty::ndarray< long long,1 > > _1931,int _1932,std::shared_ptr< monty::ndarray< double,1 > > _1933,bool _1934);
virtual void make_continuous(std::shared_ptr< monty::ndarray< long long,1 > > _1939);
virtual void make_integer(std::shared_ptr< monty::ndarray< long long,1 > > _1942);
virtual monty::rc_ptr< ::mosek::fusion::ModelVariable > clone(monty::rc_ptr< ::mosek::fusion::Model > _1945);
}; // struct RangedVariable;

struct p_LinearPSDVariable : public ::mosek::fusion::p_ModelVariable
{
LinearPSDVariable * _pubthis;
static mosek::fusion::p_LinearPSDVariable* _get_impl(mosek::fusion::LinearPSDVariable * _inst){ return static_cast< mosek::fusion::p_LinearPSDVariable* >(mosek::fusion::p_ModelVariable::_get_impl(_inst)); }
static mosek::fusion::p_LinearPSDVariable * _get_impl(mosek::fusion::LinearPSDVariable::t _inst) { return _get_impl(_inst.get()); }
p_LinearPSDVariable(LinearPSDVariable * _pubthis);
virtual ~p_LinearPSDVariable() { /* std::cout << "~p_LinearPSDVariable" << std::endl;*/ };
int numcones{};int coneidx{};int conesize{};int sdpvardim{};int blocksize{};virtual void destroy();
static LinearPSDVariable::t _new_LinearPSDVariable(monty::rc_ptr< ::mosek::fusion::LinearPSDVariable > _1946,monty::rc_ptr< ::mosek::fusion::Model > _1947);
void _initialize(monty::rc_ptr< ::mosek::fusion::LinearPSDVariable > _1946,monty::rc_ptr< ::mosek::fusion::Model > _1947);
static LinearPSDVariable::t _new_LinearPSDVariable(monty::rc_ptr< ::mosek::fusion::Model > _1948,const std::string &  _1949,int _1950,monty::rc_ptr< ::mosek::fusion::Set > _1951,int _1952,long long _1953);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _1948,const std::string &  _1949,int _1950,monty::rc_ptr< ::mosek::fusion::Set > _1951,int _1952,long long _1953);
virtual void flushNames();
virtual std::string toString();
virtual void inst(std::shared_ptr< monty::ndarray< long long,1 > > _1957,int _1958,int _1959,long long _1960,long long _1961,std::shared_ptr< monty::ndarray< int,1 > > _1962,std::shared_ptr< monty::ndarray< int,1 > > _1963,std::shared_ptr< monty::ndarray< int,1 > > _1964);
virtual void inst(long long _1974,long long _1975,std::shared_ptr< monty::ndarray< int,1 > > _1976,std::shared_ptr< monty::ndarray< int,1 > > _1977,std::shared_ptr< monty::ndarray< int,1 > > _1978);
virtual void set_values(long long _1984,std::shared_ptr< monty::ndarray< int,1 > > _1985,std::shared_ptr< monty::ndarray< long long,1 > > _1986,int _1987,std::shared_ptr< monty::ndarray< double,1 > > _1988,bool _1989);
virtual void set_values(std::shared_ptr< monty::ndarray< long long,1 > > _1990,std::shared_ptr< monty::ndarray< double,1 > > _1991,bool _1992);
virtual void values(long long _1993,std::shared_ptr< monty::ndarray< int,1 > > _1994,std::shared_ptr< monty::ndarray< long long,1 > > _1995,int _1996,std::shared_ptr< monty::ndarray< double,1 > > _1997,bool _1998);
virtual void values(std::shared_ptr< monty::ndarray< long long,1 > > _2007,int _2008,std::shared_ptr< monty::ndarray< double,1 > > _2009,bool _2010);
virtual void make_continuous(std::shared_ptr< monty::ndarray< long long,1 > > _2016);
virtual void make_integer(std::shared_ptr< monty::ndarray< long long,1 > > _2017);
virtual monty::rc_ptr< ::mosek::fusion::ModelVariable > clone(monty::rc_ptr< ::mosek::fusion::Model > _2018);
}; // struct LinearPSDVariable;

struct p_PSDVariable : public ::mosek::fusion::p_ModelVariable, public /*implements*/ ::mosek::fusion::SymmetricVariable
{
PSDVariable * _pubthis;
static mosek::fusion::p_PSDVariable* _get_impl(mosek::fusion::PSDVariable * _inst){ return static_cast< mosek::fusion::p_PSDVariable* >(mosek::fusion::p_ModelVariable::_get_impl(_inst)); }
static mosek::fusion::p_PSDVariable * _get_impl(mosek::fusion::PSDVariable::t _inst) { return _get_impl(_inst.get()); }
p_PSDVariable(PSDVariable * _pubthis);
virtual ~p_PSDVariable() { /* std::cout << "~p_PSDVariable" << std::endl;*/ };
int numcones{};int coneidx{};int conesize{};virtual void destroy();
static PSDVariable::t _new_PSDVariable(monty::rc_ptr< ::mosek::fusion::PSDVariable > _2019,monty::rc_ptr< ::mosek::fusion::Model > _2020);
void _initialize(monty::rc_ptr< ::mosek::fusion::PSDVariable > _2019,monty::rc_ptr< ::mosek::fusion::Model > _2020);
static PSDVariable::t _new_PSDVariable(monty::rc_ptr< ::mosek::fusion::Model > _2021,const std::string &  _2022,int _2023,int _2024,int _2025,long long _2026);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _2021,const std::string &  _2022,int _2023,int _2024,int _2025,long long _2026);
virtual void flushNames();
virtual std::string toString();
virtual monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > elementDesc(long long _2029,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2030);
virtual void elementName(long long _2036,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2037);
virtual void inst(std::shared_ptr< monty::ndarray< long long,1 > > _2043,int _2044,int _2045,long long _2046,long long _2047,std::shared_ptr< monty::ndarray< int,1 > > _2048,std::shared_ptr< monty::ndarray< int,1 > > _2049,std::shared_ptr< monty::ndarray< int,1 > > _2050);
virtual void inst(long long _2059,long long _2060,std::shared_ptr< monty::ndarray< int,1 > > _2061,std::shared_ptr< monty::ndarray< int,1 > > _2062,std::shared_ptr< monty::ndarray< int,1 > > _2063);
virtual void set_values(long long _2068,std::shared_ptr< monty::ndarray< int,1 > > _2069,std::shared_ptr< monty::ndarray< long long,1 > > _2070,int _2071,std::shared_ptr< monty::ndarray< double,1 > > _2072,bool _2073);
virtual void set_values(std::shared_ptr< monty::ndarray< long long,1 > > _2074,std::shared_ptr< monty::ndarray< double,1 > > _2075,bool _2076);
virtual void values(long long _2077,std::shared_ptr< monty::ndarray< int,1 > > _2078,std::shared_ptr< monty::ndarray< long long,1 > > _2079,int _2080,std::shared_ptr< monty::ndarray< double,1 > > _2081,bool _2082);
virtual void values(std::shared_ptr< monty::ndarray< long long,1 > > _2094,int _2095,std::shared_ptr< monty::ndarray< double,1 > > _2096,bool _2097);
virtual void make_continuous(std::shared_ptr< monty::ndarray< long long,1 > > _2105);
virtual void make_integer(std::shared_ptr< monty::ndarray< long long,1 > > _2106);
virtual monty::rc_ptr< ::mosek::fusion::ModelVariable > clone(monty::rc_ptr< ::mosek::fusion::Model > _2107);
virtual monty::rc_ptr< ::mosek::fusion::Expression > asExpr() /*override*/
{ return mosek::fusion::p_BaseVariable::asExpr(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(int _2472,int _2473) /*override*/
{ return mosek::fusion::p_ModelVariable::slice(_2472,_2473); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,2 > > _2599) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2599); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2602) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2602); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > antidiag() /*override*/
{ return mosek::fusion::p_BaseVariable::antidiag(); }
virtual void makeContinuous() /*override*/
{ mosek::fusion::p_BaseVariable::makeContinuous(); }
virtual monty::rc_ptr< ::mosek::fusion::Set > shape() /*override*/
{ return mosek::fusion::p_BaseVariable::shape(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2592,std::shared_ptr< monty::ndarray< int,1 > > _2593,std::shared_ptr< monty::ndarray< int,1 > > _2594) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2592,_2593,_2594); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2591) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2591); }
virtual void makeInteger() /*override*/
{ mosek::fusion::p_BaseVariable::makeInteger(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2585,int _2586,int _2587) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2585,_2586,_2587); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > diag() /*override*/
{ return mosek::fusion::p_BaseVariable::diag(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2588,int _2589) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2588,_2589); }
virtual monty::rc_ptr< ::mosek::fusion::Set > getShape() /*override*/
{ return mosek::fusion::p_BaseVariable::getShape(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > transpose() /*override*/
{ return mosek::fusion::p_BaseVariable::transpose(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(std::shared_ptr< monty::ndarray< int,1 > > _2590) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2590); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2596,std::shared_ptr< monty::ndarray< int,1 > > _2597) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2596,_2597); }
virtual std::shared_ptr< monty::ndarray< double,1 > > level() /*override*/
{ return mosek::fusion::p_BaseVariable::level(); }
virtual monty::rc_ptr< ::mosek::fusion::Model > getModel() /*override*/
{ return mosek::fusion::p_BaseVariable::getModel(); }
virtual void setLevel(std::shared_ptr< monty::ndarray< double,1 > > _2576) /*override*/
{ mosek::fusion::p_BaseVariable::setLevel(_2576); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > diag(int _2605) /*override*/
{ return mosek::fusion::p_BaseVariable::diag(_2605); }
virtual std::shared_ptr< monty::ndarray< double,1 > > dual() /*override*/
{ return mosek::fusion::p_BaseVariable::dual(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(std::shared_ptr< monty::ndarray< int,1 > > _2465,std::shared_ptr< monty::ndarray< int,1 > > _2466) /*override*/
{ return mosek::fusion::p_ModelVariable::slice(_2465,_2466); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > antidiag(int _2604) /*override*/
{ return mosek::fusion::p_BaseVariable::antidiag(_2604); }
virtual long long size() /*override*/
{ return mosek::fusion::p_BaseVariable::size(); }
virtual void values(int _2559,std::shared_ptr< monty::ndarray< double,1 > > _2560,bool _2561) /*override*/
{ mosek::fusion::p_BaseVariable::values(_2559,_2560,_2561); }
}; // struct PSDVariable;

struct p_SymLinearVariable : public ::mosek::fusion::p_ModelVariable, public /*implements*/ ::mosek::fusion::SymmetricVariable
{
SymLinearVariable * _pubthis;
static mosek::fusion::p_SymLinearVariable* _get_impl(mosek::fusion::SymLinearVariable * _inst){ return static_cast< mosek::fusion::p_SymLinearVariable* >(mosek::fusion::p_ModelVariable::_get_impl(_inst)); }
static mosek::fusion::p_SymLinearVariable * _get_impl(mosek::fusion::SymLinearVariable::t _inst) { return _get_impl(_inst.get()); }
p_SymLinearVariable(SymLinearVariable * _pubthis);
virtual ~p_SymLinearVariable() { /* std::cout << "~p_SymLinearVariable" << std::endl;*/ };
int dim{};bool names_flushed{};monty::rc_ptr< ::mosek::fusion::SymmetricLinearDomain > dom{};std::shared_ptr< monty::ndarray< int,1 > > nativeidxs{};monty::rc_ptr< ::mosek::fusion::Utils::IntMap > idxmap{};virtual void destroy();
static SymLinearVariable::t _new_SymLinearVariable(monty::rc_ptr< ::mosek::fusion::SymLinearVariable > _2108,monty::rc_ptr< ::mosek::fusion::Model > _2109);
void _initialize(monty::rc_ptr< ::mosek::fusion::SymLinearVariable > _2108,monty::rc_ptr< ::mosek::fusion::Model > _2109);
static SymLinearVariable::t _new_SymLinearVariable(monty::rc_ptr< ::mosek::fusion::Model > _2111,const std::string &  _2112,monty::rc_ptr< ::mosek::fusion::SymmetricLinearDomain > _2113,int _2114,std::shared_ptr< monty::ndarray< int,1 > > _2115,long long _2116);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _2111,const std::string &  _2112,monty::rc_ptr< ::mosek::fusion::SymmetricLinearDomain > _2113,int _2114,std::shared_ptr< monty::ndarray< int,1 > > _2115,long long _2116);
virtual std::string toString();
virtual void flushNames();
virtual void inst(std::shared_ptr< monty::ndarray< long long,1 > > _2126,int _2127,int _2128,long long _2129,long long _2130,std::shared_ptr< monty::ndarray< int,1 > > _2131,std::shared_ptr< monty::ndarray< int,1 > > _2132,std::shared_ptr< monty::ndarray< int,1 > > _2133);
virtual void inst(long long _2139,long long _2140,std::shared_ptr< monty::ndarray< int,1 > > _2141,std::shared_ptr< monty::ndarray< int,1 > > _2142,std::shared_ptr< monty::ndarray< int,1 > > _2143);
virtual void dual_values(long long _2147,std::shared_ptr< monty::ndarray< int,1 > > _2148,std::shared_ptr< monty::ndarray< long long,1 > > _2149,int _2150,std::shared_ptr< monty::ndarray< double,1 > > _2151);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _2163,int _2164,std::shared_ptr< monty::ndarray< double,1 > > _2165);
virtual void set_values(long long _2173,std::shared_ptr< monty::ndarray< int,1 > > _2174,std::shared_ptr< monty::ndarray< long long,1 > > _2175,int _2176,std::shared_ptr< monty::ndarray< double,1 > > _2177,bool _2178);
virtual void set_values(std::shared_ptr< monty::ndarray< long long,1 > > _2192,std::shared_ptr< monty::ndarray< double,1 > > _2193,bool _2194);
virtual void values(long long _2202,std::shared_ptr< monty::ndarray< int,1 > > _2203,std::shared_ptr< monty::ndarray< long long,1 > > _2204,int _2205,std::shared_ptr< monty::ndarray< double,1 > > _2206,bool _2207);
virtual void values(std::shared_ptr< monty::ndarray< long long,1 > > _2217,int _2218,std::shared_ptr< monty::ndarray< double,1 > > _2219,bool _2220);
virtual void make_continuous(std::shared_ptr< monty::ndarray< long long,1 > > _2228);
virtual void make_integer(std::shared_ptr< monty::ndarray< long long,1 > > _2231);
virtual long long tril_idx(long long _2234);
virtual long long tril_lin_idx(long long _2237);
virtual monty::rc_ptr< ::mosek::fusion::ModelVariable > clone(monty::rc_ptr< ::mosek::fusion::Model > _2240);
virtual monty::rc_ptr< ::mosek::fusion::Expression > asExpr() /*override*/
{ return mosek::fusion::p_BaseVariable::asExpr(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(int _2472,int _2473) /*override*/
{ return mosek::fusion::p_ModelVariable::slice(_2472,_2473); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,2 > > _2599) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2599); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2602) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2602); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > antidiag() /*override*/
{ return mosek::fusion::p_BaseVariable::antidiag(); }
virtual void makeContinuous() /*override*/
{ mosek::fusion::p_BaseVariable::makeContinuous(); }
virtual monty::rc_ptr< ::mosek::fusion::Set > shape() /*override*/
{ return mosek::fusion::p_BaseVariable::shape(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2592,std::shared_ptr< monty::ndarray< int,1 > > _2593,std::shared_ptr< monty::ndarray< int,1 > > _2594) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2592,_2593,_2594); }
virtual void elementName(long long _2463,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2464) /*override*/
{ mosek::fusion::p_ModelVariable::elementName(_2463,_2464); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2591) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2591); }
virtual void makeInteger() /*override*/
{ mosek::fusion::p_BaseVariable::makeInteger(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2585,int _2586,int _2587) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2585,_2586,_2587); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > diag() /*override*/
{ return mosek::fusion::p_BaseVariable::diag(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2588,int _2589) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2588,_2589); }
virtual monty::rc_ptr< ::mosek::fusion::Set > getShape() /*override*/
{ return mosek::fusion::p_BaseVariable::getShape(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > transpose() /*override*/
{ return mosek::fusion::p_BaseVariable::transpose(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(std::shared_ptr< monty::ndarray< int,1 > > _2590) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2590); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2596,std::shared_ptr< monty::ndarray< int,1 > > _2597) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2596,_2597); }
virtual std::shared_ptr< monty::ndarray< double,1 > > level() /*override*/
{ return mosek::fusion::p_BaseVariable::level(); }
virtual monty::rc_ptr< ::mosek::fusion::Model > getModel() /*override*/
{ return mosek::fusion::p_BaseVariable::getModel(); }
virtual void setLevel(std::shared_ptr< monty::ndarray< double,1 > > _2576) /*override*/
{ mosek::fusion::p_BaseVariable::setLevel(_2576); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > diag(int _2605) /*override*/
{ return mosek::fusion::p_BaseVariable::diag(_2605); }
virtual std::shared_ptr< monty::ndarray< double,1 > > dual() /*override*/
{ return mosek::fusion::p_BaseVariable::dual(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(std::shared_ptr< monty::ndarray< int,1 > > _2465,std::shared_ptr< monty::ndarray< int,1 > > _2466) /*override*/
{ return mosek::fusion::p_ModelVariable::slice(_2465,_2466); }
virtual monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > elementDesc(long long _2527,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2528) /*override*/
{ return mosek::fusion::p_BaseVariable::elementDesc(_2527,_2528); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > antidiag(int _2604) /*override*/
{ return mosek::fusion::p_BaseVariable::antidiag(_2604); }
virtual long long size() /*override*/
{ return mosek::fusion::p_BaseVariable::size(); }
virtual void values(int _2559,std::shared_ptr< monty::ndarray< double,1 > > _2560,bool _2561) /*override*/
{ mosek::fusion::p_BaseVariable::values(_2559,_2560,_2561); }
}; // struct SymLinearVariable;

struct p_LinearVariable : public ::mosek::fusion::p_ModelVariable
{
LinearVariable * _pubthis;
static mosek::fusion::p_LinearVariable* _get_impl(mosek::fusion::LinearVariable * _inst){ return static_cast< mosek::fusion::p_LinearVariable* >(mosek::fusion::p_ModelVariable::_get_impl(_inst)); }
static mosek::fusion::p_LinearVariable * _get_impl(mosek::fusion::LinearVariable::t _inst) { return _get_impl(_inst.get()); }
p_LinearVariable(LinearVariable * _pubthis);
virtual ~p_LinearVariable() { /* std::cout << "~p_LinearVariable" << std::endl;*/ };
bool names_flushed{};monty::rc_ptr< ::mosek::fusion::LinearDomain > dom{};std::shared_ptr< monty::ndarray< int,1 > > nativeidxs{};monty::rc_ptr< ::mosek::fusion::Utils::IntMap > idxmap{};virtual void destroy();
static LinearVariable::t _new_LinearVariable(monty::rc_ptr< ::mosek::fusion::LinearVariable > _2241,monty::rc_ptr< ::mosek::fusion::Model > _2242);
void _initialize(monty::rc_ptr< ::mosek::fusion::LinearVariable > _2241,monty::rc_ptr< ::mosek::fusion::Model > _2242);
static LinearVariable::t _new_LinearVariable(monty::rc_ptr< ::mosek::fusion::Model > _2244,const std::string &  _2245,monty::rc_ptr< ::mosek::fusion::LinearDomain > _2246,monty::rc_ptr< ::mosek::fusion::Set > _2247,std::shared_ptr< monty::ndarray< int,1 > > _2248,long long _2249);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _2244,const std::string &  _2245,monty::rc_ptr< ::mosek::fusion::LinearDomain > _2246,monty::rc_ptr< ::mosek::fusion::Set > _2247,std::shared_ptr< monty::ndarray< int,1 > > _2248,long long _2249);
virtual monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > elementDesc(long long _2250,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2251);
virtual void elementName(long long _2252,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2253);
virtual void flushNames();
virtual void inst(std::shared_ptr< monty::ndarray< long long,1 > > _2258,int _2259,int _2260,long long _2261,long long _2262,std::shared_ptr< monty::ndarray< int,1 > > _2263,std::shared_ptr< monty::ndarray< int,1 > > _2264,std::shared_ptr< monty::ndarray< int,1 > > _2265);
virtual void inst(long long _2269,long long _2270,std::shared_ptr< monty::ndarray< int,1 > > _2271,std::shared_ptr< monty::ndarray< int,1 > > _2272,std::shared_ptr< monty::ndarray< int,1 > > _2273);
virtual void dual_values(long long _2275,std::shared_ptr< monty::ndarray< int,1 > > _2276,std::shared_ptr< monty::ndarray< long long,1 > > _2277,int _2278,std::shared_ptr< monty::ndarray< double,1 > > _2279);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _2291,int _2292,std::shared_ptr< monty::ndarray< double,1 > > _2293);
virtual void set_values(long long _2301,std::shared_ptr< monty::ndarray< int,1 > > _2302,std::shared_ptr< monty::ndarray< long long,1 > > _2303,int _2304,std::shared_ptr< monty::ndarray< double,1 > > _2305,bool _2306);
virtual void set_values(std::shared_ptr< monty::ndarray< long long,1 > > _2320,std::shared_ptr< monty::ndarray< double,1 > > _2321,bool _2322);
virtual void values(long long _2332,std::shared_ptr< monty::ndarray< int,1 > > _2333,std::shared_ptr< monty::ndarray< long long,1 > > _2334,int _2335,std::shared_ptr< monty::ndarray< double,1 > > _2336,bool _2337);
virtual void make_continuous(std::shared_ptr< monty::ndarray< long long,1 > > _2346);
virtual void make_integer(std::shared_ptr< monty::ndarray< long long,1 > > _2349);
virtual void values(std::shared_ptr< monty::ndarray< long long,1 > > _2352,int _2353,std::shared_ptr< monty::ndarray< double,1 > > _2354,bool _2355);
virtual monty::rc_ptr< ::mosek::fusion::ModelVariable > clone(monty::rc_ptr< ::mosek::fusion::Model > _2360);
}; // struct LinearVariable;

struct p_ConicVariable : public ::mosek::fusion::p_ModelVariable
{
ConicVariable * _pubthis;
static mosek::fusion::p_ConicVariable* _get_impl(mosek::fusion::ConicVariable * _inst){ return static_cast< mosek::fusion::p_ConicVariable* >(mosek::fusion::p_ModelVariable::_get_impl(_inst)); }
static mosek::fusion::p_ConicVariable * _get_impl(mosek::fusion::ConicVariable::t _inst) { return _get_impl(_inst.get()); }
p_ConicVariable(ConicVariable * _pubthis);
virtual ~p_ConicVariable() { /* std::cout << "~p_ConicVariable" << std::endl;*/ };
bool names_flushed{};std::shared_ptr< monty::ndarray< int,1 > > nativeidxs{};monty::rc_ptr< ::mosek::fusion::QConeDomain > dom{};int numcone{};int conesize{};int coneidx{};virtual void destroy();
static ConicVariable::t _new_ConicVariable(monty::rc_ptr< ::mosek::fusion::ConicVariable > _2377,monty::rc_ptr< ::mosek::fusion::Model > _2378);
void _initialize(monty::rc_ptr< ::mosek::fusion::ConicVariable > _2377,monty::rc_ptr< ::mosek::fusion::Model > _2378);
static ConicVariable::t _new_ConicVariable(monty::rc_ptr< ::mosek::fusion::Model > _2380,const std::string &  _2381,monty::rc_ptr< ::mosek::fusion::QConeDomain > _2382,monty::rc_ptr< ::mosek::fusion::Set > _2383,std::shared_ptr< monty::ndarray< int,1 > > _2384,int _2385,int _2386,int _2387,long long _2388);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _2380,const std::string &  _2381,monty::rc_ptr< ::mosek::fusion::QConeDomain > _2382,monty::rc_ptr< ::mosek::fusion::Set > _2383,std::shared_ptr< monty::ndarray< int,1 > > _2384,int _2385,int _2386,int _2387,long long _2388);
virtual std::string toString();
virtual monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > elementDesc(long long _2391,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2392);
virtual void elementName(long long _2393,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2394);
virtual void flushNames();
virtual void inst(std::shared_ptr< monty::ndarray< long long,1 > > _2396,int _2397,int _2398,long long _2399,long long _2400,std::shared_ptr< monty::ndarray< int,1 > > _2401,std::shared_ptr< monty::ndarray< int,1 > > _2402,std::shared_ptr< monty::ndarray< int,1 > > _2403);
virtual void inst(long long _2405,long long _2406,std::shared_ptr< monty::ndarray< int,1 > > _2407,std::shared_ptr< monty::ndarray< int,1 > > _2408,std::shared_ptr< monty::ndarray< int,1 > > _2409);
virtual void set_values(long long _2410,std::shared_ptr< monty::ndarray< int,1 > > _2411,std::shared_ptr< monty::ndarray< long long,1 > > _2412,int _2413,std::shared_ptr< monty::ndarray< double,1 > > _2414,bool _2415);
virtual void set_values(std::shared_ptr< monty::ndarray< long long,1 > > _2425,std::shared_ptr< monty::ndarray< double,1 > > _2426,bool _2427);
virtual void values(long long _2433,std::shared_ptr< monty::ndarray< int,1 > > _2434,std::shared_ptr< monty::ndarray< long long,1 > > _2435,int _2436,std::shared_ptr< monty::ndarray< double,1 > > _2437,bool _2438);
virtual void values(std::shared_ptr< monty::ndarray< long long,1 > > _2445,int _2446,std::shared_ptr< monty::ndarray< double,1 > > _2447,bool _2448);
virtual void make_continuous(std::shared_ptr< monty::ndarray< long long,1 > > _2451);
virtual void make_integer(std::shared_ptr< monty::ndarray< long long,1 > > _2453);
virtual int get_variable_index(int _2455);
virtual monty::rc_ptr< ::mosek::fusion::ModelVariable > clone(monty::rc_ptr< ::mosek::fusion::Model > _2456);
}; // struct ConicVariable;

struct p_NilVariable : public ::mosek::fusion::p_BaseVariable, public /*implements*/ ::mosek::fusion::SymmetricVariable
{
NilVariable * _pubthis;
static mosek::fusion::p_NilVariable* _get_impl(mosek::fusion::NilVariable * _inst){ return static_cast< mosek::fusion::p_NilVariable* >(mosek::fusion::p_BaseVariable::_get_impl(_inst)); }
static mosek::fusion::p_NilVariable * _get_impl(mosek::fusion::NilVariable::t _inst) { return _get_impl(_inst.get()); }
p_NilVariable(NilVariable * _pubthis);
virtual ~p_NilVariable() { /* std::cout << "~p_NilVariable" << std::endl;*/ };
virtual void destroy();
static NilVariable::t _new_NilVariable();
void _initialize();
virtual monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > elementDesc(long long _2476,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2477);
virtual void elementName(long long _2478,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2479);
virtual void inst(std::shared_ptr< monty::ndarray< long long,1 > > _2480,int _2481,int _2482,long long _2483,long long _2484,std::shared_ptr< monty::ndarray< int,1 > > _2485,std::shared_ptr< monty::ndarray< int,1 > > _2486,std::shared_ptr< monty::ndarray< int,1 > > _2487);
virtual void inst(long long _2488,long long _2489,std::shared_ptr< monty::ndarray< int,1 > > _2490,std::shared_ptr< monty::ndarray< int,1 > > _2491,std::shared_ptr< monty::ndarray< int,1 > > _2492);
virtual void set_values(std::shared_ptr< monty::ndarray< long long,1 > > _2493,std::shared_ptr< monty::ndarray< double,1 > > _2494,bool _2495);
virtual void set_values(long long _2496,std::shared_ptr< monty::ndarray< int,1 > > _2497,std::shared_ptr< monty::ndarray< long long,1 > > _2498,int _2499,std::shared_ptr< monty::ndarray< double,1 > > _2500,bool _2501);
virtual void values(std::shared_ptr< monty::ndarray< long long,1 > > _2502,int _2503,std::shared_ptr< monty::ndarray< double,1 > > _2504,bool _2505);
virtual void values(long long _2506,std::shared_ptr< monty::ndarray< int,1 > > _2507,std::shared_ptr< monty::ndarray< long long,1 > > _2508,int _2509,std::shared_ptr< monty::ndarray< double,1 > > _2510,bool _2511);
virtual void make_continuous(std::shared_ptr< monty::ndarray< long long,1 > > _2512);
virtual void make_integer(std::shared_ptr< monty::ndarray< long long,1 > > _2513);
virtual void makeContinuous();
virtual void makeInteger();
virtual std::string toString();
virtual long long size();
virtual std::shared_ptr< monty::ndarray< double,1 > > dual();
virtual std::shared_ptr< monty::ndarray< double,1 > > level();
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(std::shared_ptr< monty::ndarray< int,1 > > _2516);
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2517);
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(std::shared_ptr< monty::ndarray< int,1 > > _2518,std::shared_ptr< monty::ndarray< int,1 > > _2519);
virtual monty::rc_ptr< ::mosek::fusion::Variable > slice(int _2521,int _2522);
virtual monty::rc_ptr< ::mosek::fusion::Expression > asExpr() /*override*/
{ return mosek::fusion::p_BaseVariable::asExpr(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,2 > > _2599) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2599); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2602) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2602); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > antidiag() /*override*/
{ return mosek::fusion::p_BaseVariable::antidiag(); }
virtual monty::rc_ptr< ::mosek::fusion::Set > shape() /*override*/
{ return mosek::fusion::p_BaseVariable::shape(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2592,std::shared_ptr< monty::ndarray< int,1 > > _2593,std::shared_ptr< monty::ndarray< int,1 > > _2594) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2592,_2593,_2594); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2585,int _2586,int _2587) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2585,_2586,_2587); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > diag() /*override*/
{ return mosek::fusion::p_BaseVariable::diag(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > index(int _2588,int _2589) /*override*/
{ return mosek::fusion::p_BaseVariable::index(_2588,_2589); }
virtual monty::rc_ptr< ::mosek::fusion::Set > getShape() /*override*/
{ return mosek::fusion::p_BaseVariable::getShape(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > transpose() /*override*/
{ return mosek::fusion::p_BaseVariable::transpose(); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > pick(std::shared_ptr< monty::ndarray< int,1 > > _2596,std::shared_ptr< monty::ndarray< int,1 > > _2597) /*override*/
{ return mosek::fusion::p_BaseVariable::pick(_2596,_2597); }
virtual monty::rc_ptr< ::mosek::fusion::Model > getModel() /*override*/
{ return mosek::fusion::p_BaseVariable::getModel(); }
virtual void setLevel(std::shared_ptr< monty::ndarray< double,1 > > _2576) /*override*/
{ mosek::fusion::p_BaseVariable::setLevel(_2576); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > diag(int _2605) /*override*/
{ return mosek::fusion::p_BaseVariable::diag(_2605); }
virtual monty::rc_ptr< ::mosek::fusion::Variable > antidiag(int _2604) /*override*/
{ return mosek::fusion::p_BaseVariable::antidiag(_2604); }
virtual void values(int _2559,std::shared_ptr< monty::ndarray< double,1 > > _2560,bool _2561) /*override*/
{ mosek::fusion::p_BaseVariable::values(_2559,_2560,_2561); }
}; // struct NilVariable;

struct p_Var
{
Var * _pubthis;
static mosek::fusion::p_Var* _get_impl(mosek::fusion::Var * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_Var * _get_impl(mosek::fusion::Var::t _inst) { return _get_impl(_inst.get()); }
p_Var(Var * _pubthis);
virtual ~p_Var() { /* std::cout << "~p_Var" << std::endl;*/ };
virtual void destroy();
static  monty::rc_ptr< ::mosek::fusion::Variable > compress(monty::rc_ptr< ::mosek::fusion::Variable > _2688);
static  monty::rc_ptr< ::mosek::fusion::Variable > reshape(monty::rc_ptr< ::mosek::fusion::Variable > _2693,int _2694);
static  monty::rc_ptr< ::mosek::fusion::Variable > reshape(monty::rc_ptr< ::mosek::fusion::Variable > _2695,int _2696,int _2697);
static  monty::rc_ptr< ::mosek::fusion::Variable > flatten(monty::rc_ptr< ::mosek::fusion::Variable > _2698);
static  monty::rc_ptr< ::mosek::fusion::Variable > reshape(monty::rc_ptr< ::mosek::fusion::Variable > _2699,std::shared_ptr< monty::ndarray< int,1 > > _2700);
static  monty::rc_ptr< ::mosek::fusion::Variable > reshape(monty::rc_ptr< ::mosek::fusion::Variable > _2702,monty::rc_ptr< ::mosek::fusion::Set > _2703);
static  monty::rc_ptr< ::mosek::fusion::Variable > reshape_(monty::rc_ptr< ::mosek::fusion::Variable > _2705,monty::rc_ptr< ::mosek::fusion::Set > _2706);
static  monty::rc_ptr< ::mosek::fusion::Variable > index_flip_(monty::rc_ptr< ::mosek::fusion::Variable > _2709,std::shared_ptr< monty::ndarray< int,1 > > _2710);
static  monty::rc_ptr< ::mosek::fusion::Variable > index_permute_(monty::rc_ptr< ::mosek::fusion::Variable > _2717,std::shared_ptr< monty::ndarray< int,1 > > _2718);
static  monty::rc_ptr< ::mosek::fusion::Variable > hrepeat(monty::rc_ptr< ::mosek::fusion::Variable > _2723,int _2724);
static  monty::rc_ptr< ::mosek::fusion::Variable > vrepeat(monty::rc_ptr< ::mosek::fusion::Variable > _2725,int _2726);
static  monty::rc_ptr< ::mosek::fusion::Variable > repeat(monty::rc_ptr< ::mosek::fusion::Variable > _2727,int _2728);
static  monty::rc_ptr< ::mosek::fusion::Variable > repeat(monty::rc_ptr< ::mosek::fusion::Variable > _2729,int _2730,int _2731);
static  monty::rc_ptr< ::mosek::fusion::Variable > drepeat(monty::rc_ptr< ::mosek::fusion::Variable > _2732,int _2733,int _2734);
static  monty::rc_ptr< ::mosek::fusion::Variable > stack(std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > >,1 > > _2739);
static  monty::rc_ptr< ::mosek::fusion::Variable > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _2755,monty::rc_ptr< ::mosek::fusion::Variable > _2756,monty::rc_ptr< ::mosek::fusion::Variable > _2757);
static  monty::rc_ptr< ::mosek::fusion::Variable > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _2758,monty::rc_ptr< ::mosek::fusion::Variable > _2759);
static  monty::rc_ptr< ::mosek::fusion::Variable > vstack(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _2760);
static  monty::rc_ptr< ::mosek::fusion::Variable > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _2761,monty::rc_ptr< ::mosek::fusion::Variable > _2762,monty::rc_ptr< ::mosek::fusion::Variable > _2763);
static  monty::rc_ptr< ::mosek::fusion::Variable > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _2764,monty::rc_ptr< ::mosek::fusion::Variable > _2765);
static  monty::rc_ptr< ::mosek::fusion::Variable > hstack(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _2766);
static  monty::rc_ptr< ::mosek::fusion::Variable > stack(monty::rc_ptr< ::mosek::fusion::Variable > _2767,monty::rc_ptr< ::mosek::fusion::Variable > _2768,monty::rc_ptr< ::mosek::fusion::Variable > _2769,int _2770);
static  monty::rc_ptr< ::mosek::fusion::Variable > stack(monty::rc_ptr< ::mosek::fusion::Variable > _2771,monty::rc_ptr< ::mosek::fusion::Variable > _2772,int _2773);
static  monty::rc_ptr< ::mosek::fusion::Variable > stack(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _2774,int _2775);
static  monty::rc_ptr< ::mosek::fusion::Variable > dstack(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _2776,int _2777);
}; // struct Var;

struct p_ConstraintCache
{
ConstraintCache * _pubthis;
static mosek::fusion::p_ConstraintCache* _get_impl(mosek::fusion::ConstraintCache * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_ConstraintCache * _get_impl(mosek::fusion::ConstraintCache::t _inst) { return _get_impl(_inst.get()); }
p_ConstraintCache(ConstraintCache * _pubthis);
virtual ~p_ConstraintCache() { /* std::cout << "~p_ConstraintCache" << std::endl;*/ };
std::shared_ptr< monty::ndarray< int,1 > > barmatidx{};std::shared_ptr< monty::ndarray< int,1 > > barsubj{};std::shared_ptr< monty::ndarray< int,1 > > barsubi{};long long nbarnz{};long long nunordered{};std::shared_ptr< monty::ndarray< int,1 > > buffer_subi{};std::shared_ptr< monty::ndarray< int,1 > > buffer_subj{};std::shared_ptr< monty::ndarray< double,1 > > buffer_cof{};std::shared_ptr< monty::ndarray< double,1 > > bfix{};std::shared_ptr< monty::ndarray< double,1 > > cof{};std::shared_ptr< monty::ndarray< int,1 > > subi{};std::shared_ptr< monty::ndarray< int,1 > > subj{};long long nnz{};int nrows{};virtual void destroy();
static ConstraintCache::t _new_ConstraintCache(monty::rc_ptr< ::mosek::fusion::ConstraintCache > _3042);
void _initialize(monty::rc_ptr< ::mosek::fusion::ConstraintCache > _3042);
static ConstraintCache::t _new_ConstraintCache(std::shared_ptr< monty::ndarray< long long,1 > > _3043,std::shared_ptr< monty::ndarray< double,1 > > _3044,std::shared_ptr< monty::ndarray< int,1 > > _3045,std::shared_ptr< monty::ndarray< double,1 > > _3046,std::shared_ptr< monty::ndarray< int,1 > > _3047,std::shared_ptr< monty::ndarray< int,1 > > _3048,std::shared_ptr< monty::ndarray< int,1 > > _3049);
void _initialize(std::shared_ptr< monty::ndarray< long long,1 > > _3043,std::shared_ptr< monty::ndarray< double,1 > > _3044,std::shared_ptr< monty::ndarray< int,1 > > _3045,std::shared_ptr< monty::ndarray< double,1 > > _3046,std::shared_ptr< monty::ndarray< int,1 > > _3047,std::shared_ptr< monty::ndarray< int,1 > > _3048,std::shared_ptr< monty::ndarray< int,1 > > _3049);
virtual void unchecked_add_fx(std::shared_ptr< monty::ndarray< double,1 > > _3052);
virtual long long order_barentries();
virtual void add_bar(std::shared_ptr< monty::ndarray< int,1 > > _3062,std::shared_ptr< monty::ndarray< int,1 > > _3063,std::shared_ptr< monty::ndarray< int,1 > > _3064);
virtual void unchecked_add_l(std::shared_ptr< monty::ndarray< long long,1 > > _3070,std::shared_ptr< monty::ndarray< int,1 > > _3071,std::shared_ptr< monty::ndarray< double,1 > > _3072,std::shared_ptr< monty::ndarray< double,1 > > _3073);
virtual void add(std::shared_ptr< monty::ndarray< long long,1 > > _3082,std::shared_ptr< monty::ndarray< int,1 > > _3083,std::shared_ptr< monty::ndarray< double,1 > > _3084,std::shared_ptr< monty::ndarray< double,1 > > _3085);
virtual long long flush(std::shared_ptr< monty::ndarray< int,1 > > _3086,std::shared_ptr< monty::ndarray< int,1 > > _3087,std::shared_ptr< monty::ndarray< double,1 > > _3088,std::shared_ptr< monty::ndarray< double,1 > > _3089);
virtual long long numUnsorted();
virtual monty::rc_ptr< ::mosek::fusion::ConstraintCache > clone();
}; // struct ConstraintCache;

struct p_Constraint
{
Constraint * _pubthis;
static mosek::fusion::p_Constraint* _get_impl(mosek::fusion::Constraint * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_Constraint * _get_impl(mosek::fusion::Constraint::t _inst) { return _get_impl(_inst.get()); }
p_Constraint(Constraint * _pubthis);
virtual ~p_Constraint() { /* std::cout << "~p_Constraint" << std::endl;*/ };
monty::rc_ptr< ::mosek::fusion::Set > shape_p{};monty::rc_ptr< ::mosek::fusion::Model > model{};virtual void destroy();
static Constraint::t _new_Constraint(monty::rc_ptr< ::mosek::fusion::Constraint > _3762,monty::rc_ptr< ::mosek::fusion::Model > _3763);
void _initialize(monty::rc_ptr< ::mosek::fusion::Constraint > _3762,monty::rc_ptr< ::mosek::fusion::Model > _3763);
static Constraint::t _new_Constraint(monty::rc_ptr< ::mosek::fusion::Model > _3764,monty::rc_ptr< ::mosek::fusion::Set > _3765);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _3764,monty::rc_ptr< ::mosek::fusion::Set > _3765);
virtual std::string toString();
virtual void toStringArray(std::shared_ptr< monty::ndarray< long long,1 > > _3771,long long _3772,std::shared_ptr< monty::ndarray< std::string,1 > > _3773) { throw monty::AbstractClassError("Call to abstract method"); }
virtual monty::rc_ptr< ::mosek::fusion::Constraint > add(double _3774);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > add(std::shared_ptr< monty::ndarray< double,1 > > _3779);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > add(monty::rc_ptr< ::mosek::fusion::Variable > _3783);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > add(monty::rc_ptr< ::mosek::fusion::Expression > _3792);
static  void inst(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _3799,std::shared_ptr< monty::ndarray< long long,1 > > _3800,std::shared_ptr< monty::ndarray< int,1 > > _3801,std::shared_ptr< monty::ndarray< int,1 > > _3802,std::shared_ptr< monty::ndarray< int,1 > > _3803);
virtual void add_l(std::shared_ptr< monty::ndarray< long long,1 > > _3811,std::shared_ptr< monty::ndarray< long long,1 > > _3812,std::shared_ptr< monty::ndarray< int,1 > > _3813,std::shared_ptr< monty::ndarray< int,1 > > _3814,std::shared_ptr< monty::ndarray< int,1 > > _3815,std::shared_ptr< monty::ndarray< double,1 > > _3816,std::shared_ptr< monty::ndarray< double,1 > > _3817,long long _3818,int _3819,int _3820) { throw monty::AbstractClassError("Call to abstract method"); }
virtual std::shared_ptr< monty::ndarray< double,1 > > dual(std::shared_ptr< monty::ndarray< int,1 > > _3821,std::shared_ptr< monty::ndarray< int,1 > > _3822);
virtual std::shared_ptr< monty::ndarray< double,1 > > dual(int _3830,int _3831);
virtual std::shared_ptr< monty::ndarray< double,1 > > dual();
virtual void dual_values(int _3835,std::shared_ptr< monty::ndarray< double,1 > > _3836);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _3841,int _3842,std::shared_ptr< monty::ndarray< double,1 > > _3843) { throw monty::AbstractClassError("Call to abstract method"); }
virtual void dual_values(long long _3844,std::shared_ptr< monty::ndarray< int,1 > > _3845,std::shared_ptr< monty::ndarray< long long,1 > > _3846,int _3847,std::shared_ptr< monty::ndarray< double,1 > > _3848) { throw monty::AbstractClassError("Call to abstract method"); }
virtual std::shared_ptr< monty::ndarray< double,1 > > level();
virtual double level(int _3850);
virtual std::shared_ptr< monty::ndarray< double,1 > > level(std::shared_ptr< monty::ndarray< int,1 > > _3853,std::shared_ptr< monty::ndarray< int,1 > > _3854);
virtual std::shared_ptr< monty::ndarray< double,1 > > level(int _3864,int _3865);
virtual void level_values(int _3870,std::shared_ptr< monty::ndarray< double,1 > > _3871);
virtual void level_values(std::shared_ptr< monty::ndarray< long long,1 > > _3876,int _3877,std::shared_ptr< monty::ndarray< double,1 > > _3878) { throw monty::AbstractClassError("Call to abstract method"); }
virtual void level_values(long long _3879,std::shared_ptr< monty::ndarray< int,1 > > _3880,std::shared_ptr< monty::ndarray< long long,1 > > _3881,int _3882,std::shared_ptr< monty::ndarray< double,1 > > _3883) { throw monty::AbstractClassError("Call to abstract method"); }
virtual monty::rc_ptr< ::mosek::fusion::Model > get_model();
virtual int get_nd();
virtual long long size();
static  monty::rc_ptr< ::mosek::fusion::Constraint > stack(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Constraint >,1 > > _3884);
static  monty::rc_ptr< ::mosek::fusion::Constraint > stack(monty::rc_ptr< ::mosek::fusion::Constraint > _3885,monty::rc_ptr< ::mosek::fusion::Constraint > _3886,monty::rc_ptr< ::mosek::fusion::Constraint > _3887);
static  monty::rc_ptr< ::mosek::fusion::Constraint > stack(monty::rc_ptr< ::mosek::fusion::Constraint > _3889,monty::rc_ptr< ::mosek::fusion::Constraint > _3890);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > index(std::shared_ptr< monty::ndarray< int,1 > > _3892);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > index(int _3894);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > slice(std::shared_ptr< monty::ndarray< int,1 > > _3895,std::shared_ptr< monty::ndarray< int,1 > > _3896) { throw monty::AbstractClassError("Call to abstract method"); }
virtual monty::rc_ptr< ::mosek::fusion::Constraint > slice(int _3897,int _3898) { throw monty::AbstractClassError("Call to abstract method"); }
virtual monty::rc_ptr< ::mosek::fusion::Constraint > reduceDims();
virtual monty::rc_ptr< ::mosek::fusion::Set > shape();
}; // struct Constraint;

struct p_CompoundConstraint : public ::mosek::fusion::p_Constraint
{
CompoundConstraint * _pubthis;
static mosek::fusion::p_CompoundConstraint* _get_impl(mosek::fusion::CompoundConstraint * _inst){ return static_cast< mosek::fusion::p_CompoundConstraint* >(mosek::fusion::p_Constraint::_get_impl(_inst)); }
static mosek::fusion::p_CompoundConstraint * _get_impl(mosek::fusion::CompoundConstraint::t _inst) { return _get_impl(_inst.get()); }
p_CompoundConstraint(CompoundConstraint * _pubthis);
virtual ~p_CompoundConstraint() { /* std::cout << "~p_CompoundConstraint" << std::endl;*/ };
int stackdim{};std::shared_ptr< monty::ndarray< int,1 > > consb{};std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Constraint >,1 > > cons{};virtual void destroy();
static CompoundConstraint::t _new_CompoundConstraint(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Constraint >,1 > > _3112);
void _initialize(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Constraint >,1 > > _3112);
virtual void toStringArray(std::shared_ptr< monty::ndarray< long long,1 > > _3119,long long _3120,std::shared_ptr< monty::ndarray< std::string,1 > > _3121);
virtual void add_l(std::shared_ptr< monty::ndarray< long long,1 > > _3122,std::shared_ptr< monty::ndarray< long long,1 > > _3123,std::shared_ptr< monty::ndarray< int,1 > > _3124,std::shared_ptr< monty::ndarray< int,1 > > _3125,std::shared_ptr< monty::ndarray< int,1 > > _3126,std::shared_ptr< monty::ndarray< double,1 > > _3127,std::shared_ptr< monty::ndarray< double,1 > > _3128,long long _3129,int _3130,int _3131);
virtual void dual_values(long long _3156,std::shared_ptr< monty::ndarray< int,1 > > _3157,std::shared_ptr< monty::ndarray< long long,1 > > _3158,int _3159,std::shared_ptr< monty::ndarray< double,1 > > _3160);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _3177,int _3178,std::shared_ptr< monty::ndarray< double,1 > > _3179);
virtual void level_values(long long _3186,std::shared_ptr< monty::ndarray< int,1 > > _3187,std::shared_ptr< monty::ndarray< long long,1 > > _3188,int _3189,std::shared_ptr< monty::ndarray< double,1 > > _3190);
virtual void level_values(std::shared_ptr< monty::ndarray< long long,1 > > _3207,int _3208,std::shared_ptr< monty::ndarray< double,1 > > _3209);
virtual void add(std::shared_ptr< monty::ndarray< long long,1 > > _3216,std::shared_ptr< monty::ndarray< int,1 > > _3217,std::shared_ptr< monty::ndarray< double,1 > > _3218,std::shared_ptr< monty::ndarray< double,1 > > _3219,int _3220,std::shared_ptr< monty::ndarray< int,1 > > _3221,int _3222);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > slice(std::shared_ptr< monty::ndarray< int,1 > > _3223,std::shared_ptr< monty::ndarray< int,1 > > _3224);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > slice(int _3225,int _3226);
static  monty::rc_ptr< ::mosek::fusion::Set > compute_shape(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Constraint >,1 > > _3227,int _3228);
static  int count_numcon(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Constraint >,1 > > _3235);
static  monty::rc_ptr< ::mosek::fusion::Model > model_from_con(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Constraint >,1 > > _3241);
}; // struct CompoundConstraint;

struct p_SliceConstraint : public ::mosek::fusion::p_Constraint
{
SliceConstraint * _pubthis;
static mosek::fusion::p_SliceConstraint* _get_impl(mosek::fusion::SliceConstraint * _inst){ return static_cast< mosek::fusion::p_SliceConstraint* >(mosek::fusion::p_Constraint::_get_impl(_inst)); }
static mosek::fusion::p_SliceConstraint * _get_impl(mosek::fusion::SliceConstraint::t _inst) { return _get_impl(_inst.get()); }
p_SliceConstraint(SliceConstraint * _pubthis);
virtual ~p_SliceConstraint() { /* std::cout << "~p_SliceConstraint" << std::endl;*/ };
std::shared_ptr< monty::ndarray< long long,1 > > strides{};long long first{};monty::rc_ptr< ::mosek::fusion::ModelConstraint > origin{};virtual void destroy();
static SliceConstraint::t _new_SliceConstraint(monty::rc_ptr< ::mosek::fusion::ModelConstraint > _3258,monty::rc_ptr< ::mosek::fusion::Set > _3259,long long _3260,std::shared_ptr< monty::ndarray< long long,1 > > _3261);
void _initialize(monty::rc_ptr< ::mosek::fusion::ModelConstraint > _3258,monty::rc_ptr< ::mosek::fusion::Set > _3259,long long _3260,std::shared_ptr< monty::ndarray< long long,1 > > _3261);
virtual void add_l(std::shared_ptr< monty::ndarray< long long,1 > > _3262,std::shared_ptr< monty::ndarray< long long,1 > > _3263,std::shared_ptr< monty::ndarray< int,1 > > _3264,std::shared_ptr< monty::ndarray< int,1 > > _3265,std::shared_ptr< monty::ndarray< int,1 > > _3266,std::shared_ptr< monty::ndarray< double,1 > > _3267,std::shared_ptr< monty::ndarray< double,1 > > _3268,long long _3269,int _3270,int _3271);
virtual void add_fx(std::shared_ptr< monty::ndarray< long long,1 > > _3282,std::shared_ptr< monty::ndarray< double,1 > > _3283,long long _3284,int _3285,int _3286);
virtual void dual_values(long long _3293,std::shared_ptr< monty::ndarray< int,1 > > _3294,std::shared_ptr< monty::ndarray< long long,1 > > _3295,int _3296,std::shared_ptr< monty::ndarray< double,1 > > _3297);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _3320,int _3321,std::shared_ptr< monty::ndarray< double,1 > > _3322);
virtual void level_values(long long _3328,std::shared_ptr< monty::ndarray< int,1 > > _3329,std::shared_ptr< monty::ndarray< long long,1 > > _3330,int _3331,std::shared_ptr< monty::ndarray< double,1 > > _3332);
virtual void level_values(std::shared_ptr< monty::ndarray< long long,1 > > _3354,int _3355,std::shared_ptr< monty::ndarray< double,1 > > _3356);
virtual void toStringArray(std::shared_ptr< monty::ndarray< long long,1 > > _3362,long long _3363,std::shared_ptr< monty::ndarray< std::string,1 > > _3364);
virtual long long size();
virtual monty::rc_ptr< ::mosek::fusion::Constraint > slice(std::shared_ptr< monty::ndarray< int,1 > > _3370,std::shared_ptr< monty::ndarray< int,1 > > _3371);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > slice(int _3375,int _3376);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > slice_(monty::rc_ptr< ::mosek::fusion::Set > _3378,long long _3379,std::shared_ptr< monty::ndarray< long long,1 > > _3380);
}; // struct SliceConstraint;

struct p_BoundInterfaceConstraint : public ::mosek::fusion::p_SliceConstraint
{
BoundInterfaceConstraint * _pubthis;
static mosek::fusion::p_BoundInterfaceConstraint* _get_impl(mosek::fusion::BoundInterfaceConstraint * _inst){ return static_cast< mosek::fusion::p_BoundInterfaceConstraint* >(mosek::fusion::p_SliceConstraint::_get_impl(_inst)); }
static mosek::fusion::p_BoundInterfaceConstraint * _get_impl(mosek::fusion::BoundInterfaceConstraint::t _inst) { return _get_impl(_inst.get()); }
p_BoundInterfaceConstraint(BoundInterfaceConstraint * _pubthis);
virtual ~p_BoundInterfaceConstraint() { /* std::cout << "~p_BoundInterfaceConstraint" << std::endl;*/ };
monty::rc_ptr< ::mosek::fusion::RangedConstraint > origincon{};bool islower{};virtual void destroy();
static BoundInterfaceConstraint::t _new_BoundInterfaceConstraint(monty::rc_ptr< ::mosek::fusion::RangedConstraint > _3242,monty::rc_ptr< ::mosek::fusion::Set > _3243,long long _3244,std::shared_ptr< monty::ndarray< long long,1 > > _3245,bool _3246);
void _initialize(monty::rc_ptr< ::mosek::fusion::RangedConstraint > _3242,monty::rc_ptr< ::mosek::fusion::Set > _3243,long long _3244,std::shared_ptr< monty::ndarray< long long,1 > > _3245,bool _3246);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > slice_(monty::rc_ptr< ::mosek::fusion::Set > _3247,long long _3248,std::shared_ptr< monty::ndarray< long long,1 > > _3249);
virtual void dual_values(long long _3250,std::shared_ptr< monty::ndarray< int,1 > > _3251,std::shared_ptr< monty::ndarray< long long,1 > > _3252,int _3253,std::shared_ptr< monty::ndarray< double,1 > > _3254);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _3255,int _3256,std::shared_ptr< monty::ndarray< double,1 > > _3257);
}; // struct BoundInterfaceConstraint;

struct p_ModelConstraint : public ::mosek::fusion::p_Constraint
{
ModelConstraint * _pubthis;
static mosek::fusion::p_ModelConstraint* _get_impl(mosek::fusion::ModelConstraint * _inst){ return static_cast< mosek::fusion::p_ModelConstraint* >(mosek::fusion::p_Constraint::_get_impl(_inst)); }
static mosek::fusion::p_ModelConstraint * _get_impl(mosek::fusion::ModelConstraint::t _inst) { return _get_impl(_inst.get()); }
p_ModelConstraint(ModelConstraint * _pubthis);
virtual ~p_ModelConstraint() { /* std::cout << "~p_ModelConstraint" << std::endl;*/ };
bool names_flushed{};std::shared_ptr< monty::ndarray< int,1 > > nativeindexes{};std::string name{};std::shared_ptr< monty::ndarray< double,1 > > cache_bfix{};monty::rc_ptr< ::mosek::fusion::ConstraintCache > cache{};virtual void destroy();
static ModelConstraint::t _new_ModelConstraint(monty::rc_ptr< ::mosek::fusion::ModelConstraint > _3648,monty::rc_ptr< ::mosek::fusion::Model > _3649);
void _initialize(monty::rc_ptr< ::mosek::fusion::ModelConstraint > _3648,monty::rc_ptr< ::mosek::fusion::Model > _3649);
static ModelConstraint::t _new_ModelConstraint(monty::rc_ptr< ::mosek::fusion::Model > _3652,const std::string &  _3653,monty::rc_ptr< ::mosek::fusion::Set > _3654,std::shared_ptr< monty::ndarray< int,1 > > _3655,std::shared_ptr< monty::ndarray< long long,1 > > _3656,std::shared_ptr< monty::ndarray< int,1 > > _3657,std::shared_ptr< monty::ndarray< double,1 > > _3658,std::shared_ptr< monty::ndarray< double,1 > > _3659,std::shared_ptr< monty::ndarray< int,1 > > _3660,std::shared_ptr< monty::ndarray< int,1 > > _3661,std::shared_ptr< monty::ndarray< int,1 > > _3662);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _3652,const std::string &  _3653,monty::rc_ptr< ::mosek::fusion::Set > _3654,std::shared_ptr< monty::ndarray< int,1 > > _3655,std::shared_ptr< monty::ndarray< long long,1 > > _3656,std::shared_ptr< monty::ndarray< int,1 > > _3657,std::shared_ptr< monty::ndarray< double,1 > > _3658,std::shared_ptr< monty::ndarray< double,1 > > _3659,std::shared_ptr< monty::ndarray< int,1 > > _3660,std::shared_ptr< monty::ndarray< int,1 > > _3661,std::shared_ptr< monty::ndarray< int,1 > > _3662);
virtual void flushNames();
virtual std::string toString();
virtual void toStringArray(std::shared_ptr< monty::ndarray< long long,1 > > _3669,long long _3670,std::shared_ptr< monty::ndarray< std::string,1 > > _3671);
virtual void domainToString(long long _3687,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _3688) { throw monty::AbstractClassError("Call to abstract method"); }
virtual void add_l(std::shared_ptr< monty::ndarray< long long,1 > > _3689,std::shared_ptr< monty::ndarray< long long,1 > > _3690,std::shared_ptr< monty::ndarray< int,1 > > _3691,std::shared_ptr< monty::ndarray< int,1 > > _3692,std::shared_ptr< monty::ndarray< int,1 > > _3693,std::shared_ptr< monty::ndarray< double,1 > > _3694,std::shared_ptr< monty::ndarray< double,1 > > _3695,long long _3696,int _3697,int _3698);
virtual void add_fx(std::shared_ptr< monty::ndarray< long long,1 > > _3746,std::shared_ptr< monty::ndarray< double,1 > > _3747,long long _3748,int _3749,int _3750) { throw monty::AbstractClassError("Call to abstract method"); }
virtual monty::rc_ptr< ::mosek::fusion::Constraint > slice(std::shared_ptr< monty::ndarray< int,1 > > _3751,std::shared_ptr< monty::ndarray< int,1 > > _3752);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > slice(int _3758,int _3759);
virtual monty::rc_ptr< ::mosek::fusion::ModelConstraint > clone(monty::rc_ptr< ::mosek::fusion::Model > _3761) { throw monty::AbstractClassError("Call to abstract method"); }
}; // struct ModelConstraint;

struct p_LinearPSDConstraint : public ::mosek::fusion::p_ModelConstraint
{
LinearPSDConstraint * _pubthis;
static mosek::fusion::p_LinearPSDConstraint* _get_impl(mosek::fusion::LinearPSDConstraint * _inst){ return static_cast< mosek::fusion::p_LinearPSDConstraint* >(mosek::fusion::p_ModelConstraint::_get_impl(_inst)); }
static mosek::fusion::p_LinearPSDConstraint * _get_impl(mosek::fusion::LinearPSDConstraint::t _inst) { return _get_impl(_inst.get()); }
p_LinearPSDConstraint(LinearPSDConstraint * _pubthis);
virtual ~p_LinearPSDConstraint() { /* std::cout << "~p_LinearPSDConstraint" << std::endl;*/ };
int psdvardim{};bool names_flushed{};int numcones{};int conesize{};int coneidx{};virtual void destroy();
static LinearPSDConstraint::t _new_LinearPSDConstraint(monty::rc_ptr< ::mosek::fusion::LinearPSDConstraint > _2791,monty::rc_ptr< ::mosek::fusion::Model > _2792);
void _initialize(monty::rc_ptr< ::mosek::fusion::LinearPSDConstraint > _2791,monty::rc_ptr< ::mosek::fusion::Model > _2792);
static LinearPSDConstraint::t _new_LinearPSDConstraint(monty::rc_ptr< ::mosek::fusion::Model > _2793,const std::string &  _2794,monty::rc_ptr< ::mosek::fusion::Set > _2795,std::shared_ptr< monty::ndarray< int,1 > > _2796,int _2797,int _2798,int _2799,std::shared_ptr< monty::ndarray< long long,1 > > _2800,std::shared_ptr< monty::ndarray< int,1 > > _2801,std::shared_ptr< monty::ndarray< double,1 > > _2802,std::shared_ptr< monty::ndarray< double,1 > > _2803,std::shared_ptr< monty::ndarray< int,1 > > _2804,std::shared_ptr< monty::ndarray< int,1 > > _2805,std::shared_ptr< monty::ndarray< int,1 > > _2806);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _2793,const std::string &  _2794,monty::rc_ptr< ::mosek::fusion::Set > _2795,std::shared_ptr< monty::ndarray< int,1 > > _2796,int _2797,int _2798,int _2799,std::shared_ptr< monty::ndarray< long long,1 > > _2800,std::shared_ptr< monty::ndarray< int,1 > > _2801,std::shared_ptr< monty::ndarray< double,1 > > _2802,std::shared_ptr< monty::ndarray< double,1 > > _2803,std::shared_ptr< monty::ndarray< int,1 > > _2804,std::shared_ptr< monty::ndarray< int,1 > > _2805,std::shared_ptr< monty::ndarray< int,1 > > _2806);
virtual void domainToString(long long _2809,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2810);
virtual std::string toString();
virtual void toStringArray(std::shared_ptr< monty::ndarray< long long,1 > > _2820,long long _2821,std::shared_ptr< monty::ndarray< std::string,1 > > _2822);
virtual void toStringArray(std::shared_ptr< monty::ndarray< long long,1 > > _2823,long long _2824,std::shared_ptr< monty::ndarray< std::string,1 > > _2825,bool _2826);
virtual void flushNames();
virtual void add_fx(std::shared_ptr< monty::ndarray< long long,1 > > _2854,std::shared_ptr< monty::ndarray< double,1 > > _2855,long long _2856,int _2857,int _2858);
virtual void dual_values(long long _2863,std::shared_ptr< monty::ndarray< int,1 > > _2864,std::shared_ptr< monty::ndarray< long long,1 > > _2865,int _2866,std::shared_ptr< monty::ndarray< double,1 > > _2867);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _2876,int _2877,std::shared_ptr< monty::ndarray< double,1 > > _2878);
virtual void level_values(long long _2884,std::shared_ptr< monty::ndarray< int,1 > > _2885,std::shared_ptr< monty::ndarray< long long,1 > > _2886,int _2887,std::shared_ptr< monty::ndarray< double,1 > > _2888);
virtual void level_values(std::shared_ptr< monty::ndarray< long long,1 > > _2897,int _2898,std::shared_ptr< monty::ndarray< double,1 > > _2899);
virtual monty::rc_ptr< ::mosek::fusion::ModelConstraint > clone(monty::rc_ptr< ::mosek::fusion::Model > _2905);
}; // struct LinearPSDConstraint;

struct p_PSDConstraint : public ::mosek::fusion::p_ModelConstraint
{
PSDConstraint * _pubthis;
static mosek::fusion::p_PSDConstraint* _get_impl(mosek::fusion::PSDConstraint * _inst){ return static_cast< mosek::fusion::p_PSDConstraint* >(mosek::fusion::p_ModelConstraint::_get_impl(_inst)); }
static mosek::fusion::p_PSDConstraint * _get_impl(mosek::fusion::PSDConstraint::t _inst) { return _get_impl(_inst.get()); }
p_PSDConstraint(PSDConstraint * _pubthis);
virtual ~p_PSDConstraint() { /* std::cout << "~p_PSDConstraint" << std::endl;*/ };
bool names_flushed{};int numcones{};int conesize{};int coneidx{};virtual void destroy();
static PSDConstraint::t _new_PSDConstraint(monty::rc_ptr< ::mosek::fusion::PSDConstraint > _2906,monty::rc_ptr< ::mosek::fusion::Model > _2907);
void _initialize(monty::rc_ptr< ::mosek::fusion::PSDConstraint > _2906,monty::rc_ptr< ::mosek::fusion::Model > _2907);
static PSDConstraint::t _new_PSDConstraint(monty::rc_ptr< ::mosek::fusion::Model > _2908,const std::string &  _2909,monty::rc_ptr< ::mosek::fusion::Set > _2910,std::shared_ptr< monty::ndarray< int,1 > > _2911,int _2912,int _2913,int _2914,std::shared_ptr< monty::ndarray< long long,1 > > _2915,std::shared_ptr< monty::ndarray< int,1 > > _2916,std::shared_ptr< monty::ndarray< double,1 > > _2917,std::shared_ptr< monty::ndarray< double,1 > > _2918,std::shared_ptr< monty::ndarray< int,1 > > _2919,std::shared_ptr< monty::ndarray< int,1 > > _2920,std::shared_ptr< monty::ndarray< int,1 > > _2921);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _2908,const std::string &  _2909,monty::rc_ptr< ::mosek::fusion::Set > _2910,std::shared_ptr< monty::ndarray< int,1 > > _2911,int _2912,int _2913,int _2914,std::shared_ptr< monty::ndarray< long long,1 > > _2915,std::shared_ptr< monty::ndarray< int,1 > > _2916,std::shared_ptr< monty::ndarray< double,1 > > _2917,std::shared_ptr< monty::ndarray< double,1 > > _2918,std::shared_ptr< monty::ndarray< int,1 > > _2919,std::shared_ptr< monty::ndarray< int,1 > > _2920,std::shared_ptr< monty::ndarray< int,1 > > _2921);
virtual void domainToString(long long _2922,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _2923);
virtual std::string toString();
virtual void toStringArray(std::shared_ptr< monty::ndarray< long long,1 > > _2933,long long _2934,std::shared_ptr< monty::ndarray< std::string,1 > > _2935);
virtual void toStringArray(std::shared_ptr< monty::ndarray< long long,1 > > _2936,long long _2937,std::shared_ptr< monty::ndarray< std::string,1 > > _2938,bool _2939);
virtual void flushNames();
virtual void add_fx(std::shared_ptr< monty::ndarray< long long,1 > > _2972,std::shared_ptr< monty::ndarray< double,1 > > _2973,long long _2974,int _2975,int _2976);
virtual void dual_values(long long _2981,std::shared_ptr< monty::ndarray< int,1 > > _2982,std::shared_ptr< monty::ndarray< long long,1 > > _2983,int _2984,std::shared_ptr< monty::ndarray< double,1 > > _2985);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _3001,int _3002,std::shared_ptr< monty::ndarray< double,1 > > _3003);
virtual void level_values(long long _3011,std::shared_ptr< monty::ndarray< int,1 > > _3012,std::shared_ptr< monty::ndarray< long long,1 > > _3013,int _3014,std::shared_ptr< monty::ndarray< double,1 > > _3015);
virtual void level_values(std::shared_ptr< monty::ndarray< long long,1 > > _3031,int _3032,std::shared_ptr< monty::ndarray< double,1 > > _3033);
virtual monty::rc_ptr< ::mosek::fusion::ModelConstraint > clone(monty::rc_ptr< ::mosek::fusion::Model > _3041);
}; // struct PSDConstraint;

struct p_RangedConstraint : public ::mosek::fusion::p_ModelConstraint
{
RangedConstraint * _pubthis;
static mosek::fusion::p_RangedConstraint* _get_impl(mosek::fusion::RangedConstraint * _inst){ return static_cast< mosek::fusion::p_RangedConstraint* >(mosek::fusion::p_ModelConstraint::_get_impl(_inst)); }
static mosek::fusion::p_RangedConstraint * _get_impl(mosek::fusion::RangedConstraint::t _inst) { return _get_impl(_inst.get()); }
p_RangedConstraint(RangedConstraint * _pubthis);
virtual ~p_RangedConstraint() { /* std::cout << "~p_RangedConstraint" << std::endl;*/ };
monty::rc_ptr< ::mosek::fusion::Utils::IntMap > idxmap{};monty::rc_ptr< ::mosek::fusion::RangeDomain > dom{};virtual void destroy();
static RangedConstraint::t _new_RangedConstraint(monty::rc_ptr< ::mosek::fusion::RangedConstraint > _3381,monty::rc_ptr< ::mosek::fusion::Model > _3382);
void _initialize(monty::rc_ptr< ::mosek::fusion::RangedConstraint > _3381,monty::rc_ptr< ::mosek::fusion::Model > _3382);
static RangedConstraint::t _new_RangedConstraint(monty::rc_ptr< ::mosek::fusion::Model > _3383,const std::string &  _3384,monty::rc_ptr< ::mosek::fusion::Set > _3385,monty::rc_ptr< ::mosek::fusion::RangeDomain > _3386,std::shared_ptr< monty::ndarray< int,1 > > _3387,std::shared_ptr< monty::ndarray< long long,1 > > _3388,std::shared_ptr< monty::ndarray< int,1 > > _3389,std::shared_ptr< monty::ndarray< double,1 > > _3390,std::shared_ptr< monty::ndarray< double,1 > > _3391,std::shared_ptr< monty::ndarray< int,1 > > _3392,std::shared_ptr< monty::ndarray< int,1 > > _3393,std::shared_ptr< monty::ndarray< int,1 > > _3394);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _3383,const std::string &  _3384,monty::rc_ptr< ::mosek::fusion::Set > _3385,monty::rc_ptr< ::mosek::fusion::RangeDomain > _3386,std::shared_ptr< monty::ndarray< int,1 > > _3387,std::shared_ptr< monty::ndarray< long long,1 > > _3388,std::shared_ptr< monty::ndarray< int,1 > > _3389,std::shared_ptr< monty::ndarray< double,1 > > _3390,std::shared_ptr< monty::ndarray< double,1 > > _3391,std::shared_ptr< monty::ndarray< int,1 > > _3392,std::shared_ptr< monty::ndarray< int,1 > > _3393,std::shared_ptr< monty::ndarray< int,1 > > _3394);
virtual void add_fx(std::shared_ptr< monty::ndarray< long long,1 > > _3395,std::shared_ptr< monty::ndarray< double,1 > > _3396,long long _3397,int _3398,int _3399);
virtual void dual_u(long long _3405,std::shared_ptr< monty::ndarray< int,1 > > _3406,std::shared_ptr< monty::ndarray< long long,1 > > _3407,int _3408,std::shared_ptr< monty::ndarray< double,1 > > _3409);
virtual void dual_u(std::shared_ptr< monty::ndarray< long long,1 > > _3417,int _3418,std::shared_ptr< monty::ndarray< double,1 > > _3419);
virtual void dual_l(long long _3423,std::shared_ptr< monty::ndarray< int,1 > > _3424,std::shared_ptr< monty::ndarray< long long,1 > > _3425,int _3426,std::shared_ptr< monty::ndarray< double,1 > > _3427);
virtual void dual_l(std::shared_ptr< monty::ndarray< long long,1 > > _3435,int _3436,std::shared_ptr< monty::ndarray< double,1 > > _3437);
virtual void dual_values(long long _3441,std::shared_ptr< monty::ndarray< int,1 > > _3442,std::shared_ptr< monty::ndarray< long long,1 > > _3443,int _3444,std::shared_ptr< monty::ndarray< double,1 > > _3445);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _3453,int _3454,std::shared_ptr< monty::ndarray< double,1 > > _3455);
virtual void level_values(long long _3459,std::shared_ptr< monty::ndarray< int,1 > > _3460,std::shared_ptr< monty::ndarray< long long,1 > > _3461,int _3462,std::shared_ptr< monty::ndarray< double,1 > > _3463);
virtual void level_values(std::shared_ptr< monty::ndarray< long long,1 > > _3471,int _3472,std::shared_ptr< monty::ndarray< double,1 > > _3473);
virtual void add_l(std::shared_ptr< monty::ndarray< long long,1 > > _3477,std::shared_ptr< monty::ndarray< long long,1 > > _3478,std::shared_ptr< monty::ndarray< int,1 > > _3479,std::shared_ptr< monty::ndarray< int,1 > > _3480,std::shared_ptr< monty::ndarray< int,1 > > _3481,std::shared_ptr< monty::ndarray< double,1 > > _3482,std::shared_ptr< monty::ndarray< double,1 > > _3483,int _3484,int _3485,int _3486);
virtual void domainToString(long long _3492,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _3493);
virtual monty::rc_ptr< ::mosek::fusion::Constraint > upperBoundCon();
virtual monty::rc_ptr< ::mosek::fusion::Constraint > lowerBoundCon();
virtual monty::rc_ptr< ::mosek::fusion::ModelConstraint > clone(monty::rc_ptr< ::mosek::fusion::Model > _3498);
}; // struct RangedConstraint;

struct p_ConicConstraint : public ::mosek::fusion::p_ModelConstraint
{
ConicConstraint * _pubthis;
static mosek::fusion::p_ConicConstraint* _get_impl(mosek::fusion::ConicConstraint * _inst){ return static_cast< mosek::fusion::p_ConicConstraint* >(mosek::fusion::p_ModelConstraint::_get_impl(_inst)); }
static mosek::fusion::p_ConicConstraint * _get_impl(mosek::fusion::ConicConstraint::t _inst) { return _get_impl(_inst.get()); }
p_ConicConstraint(ConicConstraint * _pubthis);
virtual ~p_ConicConstraint() { /* std::cout << "~p_ConicConstraint" << std::endl;*/ };
bool names_flushed{};monty::rc_ptr< ::mosek::fusion::QConeDomain > dom{};int conesize{};int last{};int first{};int last_slack{};int first_slack{};int coneidx{};virtual void destroy();
static ConicConstraint::t _new_ConicConstraint(monty::rc_ptr< ::mosek::fusion::ConicConstraint > _3499,monty::rc_ptr< ::mosek::fusion::Model > _3500);
void _initialize(monty::rc_ptr< ::mosek::fusion::ConicConstraint > _3499,monty::rc_ptr< ::mosek::fusion::Model > _3500);
static ConicConstraint::t _new_ConicConstraint(monty::rc_ptr< ::mosek::fusion::Model > _3501,const std::string &  _3502,monty::rc_ptr< ::mosek::fusion::QConeDomain > _3503,monty::rc_ptr< ::mosek::fusion::Set > _3504,std::shared_ptr< monty::ndarray< int,1 > > _3505,int _3506,int _3507,int _3508,int _3509,int _3510,std::shared_ptr< monty::ndarray< long long,1 > > _3511,std::shared_ptr< monty::ndarray< int,1 > > _3512,std::shared_ptr< monty::ndarray< double,1 > > _3513,std::shared_ptr< monty::ndarray< double,1 > > _3514,std::shared_ptr< monty::ndarray< int,1 > > _3515,std::shared_ptr< monty::ndarray< int,1 > > _3516,std::shared_ptr< monty::ndarray< int,1 > > _3517);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _3501,const std::string &  _3502,monty::rc_ptr< ::mosek::fusion::QConeDomain > _3503,monty::rc_ptr< ::mosek::fusion::Set > _3504,std::shared_ptr< monty::ndarray< int,1 > > _3505,int _3506,int _3507,int _3508,int _3509,int _3510,std::shared_ptr< monty::ndarray< long long,1 > > _3511,std::shared_ptr< monty::ndarray< int,1 > > _3512,std::shared_ptr< monty::ndarray< double,1 > > _3513,std::shared_ptr< monty::ndarray< double,1 > > _3514,std::shared_ptr< monty::ndarray< int,1 > > _3515,std::shared_ptr< monty::ndarray< int,1 > > _3516,std::shared_ptr< monty::ndarray< int,1 > > _3517);
virtual void flushNames();
virtual std::string toString();
virtual void dual_values(long long _3525,std::shared_ptr< monty::ndarray< int,1 > > _3526,std::shared_ptr< monty::ndarray< long long,1 > > _3527,int _3528,std::shared_ptr< monty::ndarray< double,1 > > _3529);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _3536,int _3537,std::shared_ptr< monty::ndarray< double,1 > > _3538);
virtual void level_values(long long _3541,std::shared_ptr< monty::ndarray< int,1 > > _3542,std::shared_ptr< monty::ndarray< long long,1 > > _3543,int _3544,std::shared_ptr< monty::ndarray< double,1 > > _3545);
virtual void level_values(std::shared_ptr< monty::ndarray< long long,1 > > _3553,int _3554,std::shared_ptr< monty::ndarray< double,1 > > _3555);
virtual void add_fx(std::shared_ptr< monty::ndarray< long long,1 > > _3559,std::shared_ptr< monty::ndarray< double,1 > > _3560,long long _3561,int _3562,int _3563);
virtual void dual(std::shared_ptr< monty::ndarray< int,1 > > _3568,int _3569,int _3570,int _3571,std::shared_ptr< monty::ndarray< double,1 > > _3572);
virtual void dual_values(std::shared_ptr< monty::ndarray< int,1 > > _3575,std::shared_ptr< monty::ndarray< int,1 > > _3576,int _3577,std::shared_ptr< monty::ndarray< double,1 > > _3578);
virtual void domainToString(long long _3583,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _3584);
virtual monty::rc_ptr< ::mosek::fusion::ModelConstraint > clone(monty::rc_ptr< ::mosek::fusion::Model > _3585);
}; // struct ConicConstraint;

struct p_LinearConstraint : public ::mosek::fusion::p_ModelConstraint
{
LinearConstraint * _pubthis;
static mosek::fusion::p_LinearConstraint* _get_impl(mosek::fusion::LinearConstraint * _inst){ return static_cast< mosek::fusion::p_LinearConstraint* >(mosek::fusion::p_ModelConstraint::_get_impl(_inst)); }
static mosek::fusion::p_LinearConstraint * _get_impl(mosek::fusion::LinearConstraint::t _inst) { return _get_impl(_inst.get()); }
p_LinearConstraint(LinearConstraint * _pubthis);
virtual ~p_LinearConstraint() { /* std::cout << "~p_LinearConstraint" << std::endl;*/ };
monty::rc_ptr< ::mosek::fusion::LinearDomain > dom{};monty::rc_ptr< ::mosek::fusion::Utils::IntMap > idxmap{};virtual void destroy();
static LinearConstraint::t _new_LinearConstraint(monty::rc_ptr< ::mosek::fusion::LinearConstraint > _3586,monty::rc_ptr< ::mosek::fusion::Model > _3587);
void _initialize(monty::rc_ptr< ::mosek::fusion::LinearConstraint > _3586,monty::rc_ptr< ::mosek::fusion::Model > _3587);
static LinearConstraint::t _new_LinearConstraint(monty::rc_ptr< ::mosek::fusion::Model > _3588,const std::string &  _3589,monty::rc_ptr< ::mosek::fusion::LinearDomain > _3590,monty::rc_ptr< ::mosek::fusion::Set > _3591,std::shared_ptr< monty::ndarray< int,1 > > _3592,std::shared_ptr< monty::ndarray< long long,1 > > _3593,std::shared_ptr< monty::ndarray< int,1 > > _3594,std::shared_ptr< monty::ndarray< double,1 > > _3595,std::shared_ptr< monty::ndarray< double,1 > > _3596,std::shared_ptr< monty::ndarray< int,1 > > _3597,std::shared_ptr< monty::ndarray< int,1 > > _3598,std::shared_ptr< monty::ndarray< int,1 > > _3599);
void _initialize(monty::rc_ptr< ::mosek::fusion::Model > _3588,const std::string &  _3589,monty::rc_ptr< ::mosek::fusion::LinearDomain > _3590,monty::rc_ptr< ::mosek::fusion::Set > _3591,std::shared_ptr< monty::ndarray< int,1 > > _3592,std::shared_ptr< monty::ndarray< long long,1 > > _3593,std::shared_ptr< monty::ndarray< int,1 > > _3594,std::shared_ptr< monty::ndarray< double,1 > > _3595,std::shared_ptr< monty::ndarray< double,1 > > _3596,std::shared_ptr< monty::ndarray< int,1 > > _3597,std::shared_ptr< monty::ndarray< int,1 > > _3598,std::shared_ptr< monty::ndarray< int,1 > > _3599);
virtual void add_fx(std::shared_ptr< monty::ndarray< long long,1 > > _3600,std::shared_ptr< monty::ndarray< double,1 > > _3601,long long _3602,int _3603,int _3604);
virtual void dual_values(long long _3609,std::shared_ptr< monty::ndarray< int,1 > > _3610,std::shared_ptr< monty::ndarray< long long,1 > > _3611,int _3612,std::shared_ptr< monty::ndarray< double,1 > > _3613);
virtual void dual_values(std::shared_ptr< monty::ndarray< long long,1 > > _3621,int _3622,std::shared_ptr< monty::ndarray< double,1 > > _3623);
virtual void level_values(long long _3627,std::shared_ptr< monty::ndarray< int,1 > > _3628,std::shared_ptr< monty::ndarray< long long,1 > > _3629,int _3630,std::shared_ptr< monty::ndarray< double,1 > > _3631);
virtual void level_values(std::shared_ptr< monty::ndarray< long long,1 > > _3639,int _3640,std::shared_ptr< monty::ndarray< double,1 > > _3641);
virtual void domainToString(long long _3645,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _3646);
virtual monty::rc_ptr< ::mosek::fusion::ModelConstraint > clone(monty::rc_ptr< ::mosek::fusion::Model > _3647);
}; // struct LinearConstraint;

struct p_Set
{
Set * _pubthis;
static mosek::fusion::p_Set* _get_impl(mosek::fusion::Set * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_Set * _get_impl(mosek::fusion::Set::t _inst) { return _get_impl(_inst.get()); }
p_Set(Set * _pubthis);
virtual ~p_Set() { /* std::cout << "~p_Set" << std::endl;*/ };
long long size{};int nd_p{};int nd{};virtual void destroy();
static Set::t _new_Set(int _3990,long long _3991);
void _initialize(int _3990,long long _3991);
virtual std::string toString();
virtual std::string indexToString(long long _3994) { throw monty::AbstractClassError("Call to abstract method"); }
virtual monty::rc_ptr< ::mosek::fusion::Set > slice(std::shared_ptr< monty::ndarray< int,1 > > _3995,std::shared_ptr< monty::ndarray< int,1 > > _3996) { throw monty::AbstractClassError("Call to abstract method"); }
virtual monty::rc_ptr< ::mosek::fusion::Set > slice(int _3997,int _3998) { throw monty::AbstractClassError("Call to abstract method"); }
virtual long long linearidx(int _3999,int _4000,int _4001);
virtual long long linearidx(int _4002,int _4003);
virtual long long linearidx(std::shared_ptr< monty::ndarray< int,1 > > _4004);
virtual std::shared_ptr< monty::ndarray< int,1 > > idxtokey(long long _4007);
virtual std::string getname(std::shared_ptr< monty::ndarray< int,1 > > _4011) { throw monty::AbstractClassError("Call to abstract method"); }
virtual std::string getname(long long _4012) { throw monty::AbstractClassError("Call to abstract method"); }
virtual long long stride(int _4013) { throw monty::AbstractClassError("Call to abstract method"); }
virtual int dim(int _4014) { throw monty::AbstractClassError("Call to abstract method"); }
static  monty::rc_ptr< ::mosek::fusion::Set > make(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Set >,1 > > _4015);
static  monty::rc_ptr< ::mosek::fusion::Set > make(monty::rc_ptr< ::mosek::fusion::Set > _4016,monty::rc_ptr< ::mosek::fusion::Set > _4017);
static  monty::rc_ptr< ::mosek::fusion::Set > make(std::shared_ptr< monty::ndarray< int,1 > > _4018);
static  monty::rc_ptr< ::mosek::fusion::Set > make(int _4019,int _4020,int _4021);
static  monty::rc_ptr< ::mosek::fusion::Set > make(int _4022,int _4023);
static  monty::rc_ptr< ::mosek::fusion::Set > make(int _4024);
static  monty::rc_ptr< ::mosek::fusion::Set > scalar();
static  monty::rc_ptr< ::mosek::fusion::Set > make(std::shared_ptr< monty::ndarray< std::string,1 > > _4025);
virtual int realnd();
virtual long long getSize();
virtual bool compare(monty::rc_ptr< ::mosek::fusion::Set > _4028);
}; // struct Set;

struct p_BaseSet : public ::mosek::fusion::p_Set
{
BaseSet * _pubthis;
static mosek::fusion::p_BaseSet* _get_impl(mosek::fusion::BaseSet * _inst){ return static_cast< mosek::fusion::p_BaseSet* >(mosek::fusion::p_Set::_get_impl(_inst)); }
static mosek::fusion::p_BaseSet * _get_impl(mosek::fusion::BaseSet::t _inst) { return _get_impl(_inst.get()); }
p_BaseSet(BaseSet * _pubthis);
virtual ~p_BaseSet() { /* std::cout << "~p_BaseSet" << std::endl;*/ };
virtual void destroy();
static BaseSet::t _new_BaseSet(long long _3949);
void _initialize(long long _3949);
virtual int dim(int _3950);
}; // struct BaseSet;

struct p_IntSet : public ::mosek::fusion::p_BaseSet
{
IntSet * _pubthis;
static mosek::fusion::p_IntSet* _get_impl(mosek::fusion::IntSet * _inst){ return static_cast< mosek::fusion::p_IntSet* >(mosek::fusion::p_BaseSet::_get_impl(_inst)); }
static mosek::fusion::p_IntSet * _get_impl(mosek::fusion::IntSet::t _inst) { return _get_impl(_inst.get()); }
p_IntSet(IntSet * _pubthis);
virtual ~p_IntSet() { /* std::cout << "~p_IntSet" << std::endl;*/ };
int last{};int first{};virtual void destroy();
static IntSet::t _new_IntSet(int _3921);
void _initialize(int _3921);
static IntSet::t _new_IntSet(int _3922,int _3923);
void _initialize(int _3922,int _3923);
virtual std::string indexToString(long long _3924);
virtual std::string getname(std::shared_ptr< monty::ndarray< int,1 > > _3925);
virtual std::string getname(long long _3926);
virtual monty::rc_ptr< ::mosek::fusion::Set > slice(std::shared_ptr< monty::ndarray< int,1 > > _3927,std::shared_ptr< monty::ndarray< int,1 > > _3928);
virtual monty::rc_ptr< ::mosek::fusion::Set > slice(int _3929,int _3930);
virtual int getidx(int _3931);
virtual long long stride(int _3932);
}; // struct IntSet;

struct p_StringSet : public ::mosek::fusion::p_BaseSet
{
StringSet * _pubthis;
static mosek::fusion::p_StringSet* _get_impl(mosek::fusion::StringSet * _inst){ return static_cast< mosek::fusion::p_StringSet* >(mosek::fusion::p_BaseSet::_get_impl(_inst)); }
static mosek::fusion::p_StringSet * _get_impl(mosek::fusion::StringSet::t _inst) { return _get_impl(_inst.get()); }
p_StringSet(StringSet * _pubthis);
virtual ~p_StringSet() { /* std::cout << "~p_StringSet" << std::endl;*/ };
std::shared_ptr< monty::ndarray< std::string,1 > > keys{};virtual void destroy();
static StringSet::t _new_StringSet(std::shared_ptr< monty::ndarray< std::string,1 > > _3933);
void _initialize(std::shared_ptr< monty::ndarray< std::string,1 > > _3933);
virtual std::string indexToString(long long _3934);
virtual std::string getname(std::shared_ptr< monty::ndarray< int,1 > > _3935);
virtual std::string getname(long long _3936);
virtual monty::rc_ptr< ::mosek::fusion::BaseSet > slice_(std::shared_ptr< monty::ndarray< int,1 > > _3937,std::shared_ptr< monty::ndarray< int,1 > > _3938);
virtual monty::rc_ptr< ::mosek::fusion::BaseSet > slice_(int _3939,int _3940);
virtual monty::rc_ptr< ::mosek::fusion::Set > slice(std::shared_ptr< monty::ndarray< int,1 > > _3942,std::shared_ptr< monty::ndarray< int,1 > > _3943);
virtual monty::rc_ptr< ::mosek::fusion::Set > slice(int _3944,int _3945);
virtual std::string toString();
virtual long long stride(int _3948);
}; // struct StringSet;

struct p_NDSet : public ::mosek::fusion::p_Set
{
NDSet * _pubthis;
static mosek::fusion::p_NDSet* _get_impl(mosek::fusion::NDSet * _inst){ return static_cast< mosek::fusion::p_NDSet* >(mosek::fusion::p_Set::_get_impl(_inst)); }
static mosek::fusion::p_NDSet * _get_impl(mosek::fusion::NDSet::t _inst) { return _get_impl(_inst.get()); }
p_NDSet(NDSet * _pubthis);
virtual ~p_NDSet() { /* std::cout << "~p_NDSet" << std::endl;*/ };
std::shared_ptr< monty::ndarray< long long,1 > > strides{};std::shared_ptr< monty::ndarray< int,1 > > dimdef{};virtual void destroy();
static NDSet::t _new_NDSet(int _3951,int _3952,int _3953);
void _initialize(int _3951,int _3952,int _3953);
static NDSet::t _new_NDSet(int _3954,int _3955);
void _initialize(int _3954,int _3955);
static NDSet::t _new_NDSet(std::shared_ptr< monty::ndarray< int,1 > > _3956);
void _initialize(std::shared_ptr< monty::ndarray< int,1 > > _3956);
virtual std::string indexToString(long long _3960);
virtual std::string getname(std::shared_ptr< monty::ndarray< int,1 > > _3964);
virtual std::string getname(long long _3968);
virtual int dim(int _3973);
virtual monty::rc_ptr< ::mosek::fusion::Set > slice(std::shared_ptr< monty::ndarray< int,1 > > _3974,std::shared_ptr< monty::ndarray< int,1 > > _3975);
virtual monty::rc_ptr< ::mosek::fusion::Set > slice(int _3979,int _3980);
virtual std::shared_ptr< monty::ndarray< int,1 > > selectidxs(std::shared_ptr< monty::ndarray< std::string,1 > > _3981);
virtual int linear_index_in_dim(int _3982,std::shared_ptr< monty::ndarray< int,1 > > _3983);
virtual int linear_index_in_dim(int _3984,int _3985);
static  long long sumdims(std::shared_ptr< monty::ndarray< int,1 > > _3986);
virtual long long stride(int _3989);
}; // struct NDSet;

struct p_ProductSet : public ::mosek::fusion::p_NDSet
{
ProductSet * _pubthis;
static mosek::fusion::p_ProductSet* _get_impl(mosek::fusion::ProductSet * _inst){ return static_cast< mosek::fusion::p_ProductSet* >(mosek::fusion::p_NDSet::_get_impl(_inst)); }
static mosek::fusion::p_ProductSet * _get_impl(mosek::fusion::ProductSet::t _inst) { return _get_impl(_inst.get()); }
p_ProductSet(ProductSet * _pubthis);
virtual ~p_ProductSet() { /* std::cout << "~p_ProductSet" << std::endl;*/ };
std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Set >,1 > > sets{};virtual void destroy();
static ProductSet::t _new_ProductSet(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Set >,1 > > _3903);
void _initialize(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Set >,1 > > _3903);
virtual std::string indexToString(long long _3905);
static  std::shared_ptr< monty::ndarray< int,1 > > computedims(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Set >,1 > > _3915);
}; // struct ProductSet;

struct p_QConeDomain
{
QConeDomain * _pubthis;
static mosek::fusion::p_QConeDomain* _get_impl(mosek::fusion::QConeDomain * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_QConeDomain * _get_impl(mosek::fusion::QConeDomain::t _inst) { return _get_impl(_inst.get()); }
p_QConeDomain(QConeDomain * _pubthis);
virtual ~p_QConeDomain() { /* std::cout << "~p_QConeDomain" << std::endl;*/ };
monty::rc_ptr< ::mosek::fusion::Set > shape{};bool int_flag{};int axisidx{};mosek::fusion::QConeKey key{};virtual void destroy();
static QConeDomain::t _new_QConeDomain(mosek::fusion::QConeKey _4031,std::shared_ptr< monty::ndarray< int,1 > > _4032,int _4033);
void _initialize(mosek::fusion::QConeKey _4031,std::shared_ptr< monty::ndarray< int,1 > > _4032,int _4033);
virtual std::string domainToString(long long _4034,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _4035);
virtual bool match_shape(monty::rc_ptr< ::mosek::fusion::Set > _4042);
virtual monty::rc_ptr< ::mosek::fusion::QConeDomain > integral();
virtual int getAxis();
virtual monty::rc_ptr< ::mosek::fusion::QConeDomain > axis(int _4043);
}; // struct QConeDomain;

struct p_LinPSDDomain
{
LinPSDDomain * _pubthis;
static mosek::fusion::p_LinPSDDomain* _get_impl(mosek::fusion::LinPSDDomain * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_LinPSDDomain * _get_impl(mosek::fusion::LinPSDDomain::t _inst) { return _get_impl(_inst.get()); }
p_LinPSDDomain(LinPSDDomain * _pubthis);
virtual ~p_LinPSDDomain() { /* std::cout << "~p_LinPSDDomain" << std::endl;*/ };
monty::rc_ptr< ::mosek::fusion::Set > shape{};virtual void destroy();
static LinPSDDomain::t _new_LinPSDDomain(monty::rc_ptr< ::mosek::fusion::Set > _4044);
void _initialize(monty::rc_ptr< ::mosek::fusion::Set > _4044);
}; // struct LinPSDDomain;

struct p_PSDDomain
{
PSDDomain * _pubthis;
static mosek::fusion::p_PSDDomain* _get_impl(mosek::fusion::PSDDomain * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_PSDDomain * _get_impl(mosek::fusion::PSDDomain::t _inst) { return _get_impl(_inst.get()); }
p_PSDDomain(PSDDomain * _pubthis);
virtual ~p_PSDDomain() { /* std::cout << "~p_PSDDomain" << std::endl;*/ };
mosek::fusion::PSDKey key{};monty::rc_ptr< ::mosek::fusion::Set > shape{};virtual void destroy();
static PSDDomain::t _new_PSDDomain(mosek::fusion::PSDKey _4045,monty::rc_ptr< ::mosek::fusion::Set > _4046);
void _initialize(mosek::fusion::PSDKey _4045,monty::rc_ptr< ::mosek::fusion::Set > _4046);
virtual std::string domainToString(long long _4047,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _4048);
}; // struct PSDDomain;

struct p_RangeDomain
{
RangeDomain * _pubthis;
static mosek::fusion::p_RangeDomain* _get_impl(mosek::fusion::RangeDomain * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_RangeDomain * _get_impl(mosek::fusion::RangeDomain::t _inst) { return _get_impl(_inst.get()); }
p_RangeDomain(RangeDomain * _pubthis);
virtual ~p_RangeDomain() { /* std::cout << "~p_RangeDomain" << std::endl;*/ };
bool sparse_flag{};bool cardinal_flag{};std::shared_ptr< monty::ndarray< double,1 > > ub{};std::shared_ptr< monty::ndarray< double,1 > > lb{};monty::rc_ptr< ::mosek::fusion::Utils::IntMap > idxmap{};monty::rc_ptr< ::mosek::fusion::Set > shape{};virtual void destroy();
static RangeDomain::t _new_RangeDomain(std::shared_ptr< monty::ndarray< double,1 > > _4053,std::shared_ptr< monty::ndarray< double,1 > > _4054,std::shared_ptr< monty::ndarray< int,1 > > _4055,std::shared_ptr< monty::ndarray< long long,1 > > _4056);
void _initialize(std::shared_ptr< monty::ndarray< double,1 > > _4053,std::shared_ptr< monty::ndarray< double,1 > > _4054,std::shared_ptr< monty::ndarray< int,1 > > _4055,std::shared_ptr< monty::ndarray< long long,1 > > _4056);
static RangeDomain::t _new_RangeDomain(monty::rc_ptr< ::mosek::fusion::RangeDomain > _4058);
void _initialize(monty::rc_ptr< ::mosek::fusion::RangeDomain > _4058);
virtual monty::rc_ptr< ::mosek::fusion::SymmetricRangeDomain > symmetric();
virtual monty::rc_ptr< ::mosek::fusion::RangeDomain > sparse();
virtual monty::rc_ptr< ::mosek::fusion::RangeDomain > integral();
virtual std::string domainToString(long long _4059,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _4060);
virtual bool match_shape(monty::rc_ptr< ::mosek::fusion::Set > _4061);
virtual double get_ub_item(long long _4062);
virtual double get_lb_item(long long _4063);
}; // struct RangeDomain;

struct p_SymmetricRangeDomain : public ::mosek::fusion::p_RangeDomain
{
SymmetricRangeDomain * _pubthis;
static mosek::fusion::p_SymmetricRangeDomain* _get_impl(mosek::fusion::SymmetricRangeDomain * _inst){ return static_cast< mosek::fusion::p_SymmetricRangeDomain* >(mosek::fusion::p_RangeDomain::_get_impl(_inst)); }
static mosek::fusion::p_SymmetricRangeDomain * _get_impl(mosek::fusion::SymmetricRangeDomain::t _inst) { return _get_impl(_inst.get()); }
p_SymmetricRangeDomain(SymmetricRangeDomain * _pubthis);
virtual ~p_SymmetricRangeDomain() { /* std::cout << "~p_SymmetricRangeDomain" << std::endl;*/ };
int dim{};virtual void destroy();
static SymmetricRangeDomain::t _new_SymmetricRangeDomain(monty::rc_ptr< ::mosek::fusion::RangeDomain > _4052);
void _initialize(monty::rc_ptr< ::mosek::fusion::RangeDomain > _4052);
}; // struct SymmetricRangeDomain;

struct p_SymmetricLinearDomain
{
SymmetricLinearDomain * _pubthis;
static mosek::fusion::p_SymmetricLinearDomain* _get_impl(mosek::fusion::SymmetricLinearDomain * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_SymmetricLinearDomain * _get_impl(mosek::fusion::SymmetricLinearDomain::t _inst) { return _get_impl(_inst.get()); }
p_SymmetricLinearDomain(SymmetricLinearDomain * _pubthis);
virtual ~p_SymmetricLinearDomain() { /* std::cout << "~p_SymmetricLinearDomain" << std::endl;*/ };
bool sparse_flag{};bool cardinal_flag{};mosek::fusion::RelationKey key{};monty::rc_ptr< ::mosek::fusion::Set > shape{};monty::rc_ptr< ::mosek::fusion::LinearDomain > dom{};int dim{};virtual void destroy();
static SymmetricLinearDomain::t _new_SymmetricLinearDomain(monty::rc_ptr< ::mosek::fusion::LinearDomain > _4064);
void _initialize(monty::rc_ptr< ::mosek::fusion::LinearDomain > _4064);
virtual monty::rc_ptr< ::mosek::fusion::SymmetricLinearDomain > sparse();
virtual monty::rc_ptr< ::mosek::fusion::SymmetricLinearDomain > integral();
virtual std::string domainToString(long long _4065,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _4066);
virtual bool match_shape(monty::rc_ptr< ::mosek::fusion::Set > _4067);
virtual double get_rhs_item(long long _4068);
}; // struct SymmetricLinearDomain;

struct p_LinearDomain
{
LinearDomain * _pubthis;
static mosek::fusion::p_LinearDomain* _get_impl(mosek::fusion::LinearDomain * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_LinearDomain * _get_impl(mosek::fusion::LinearDomain::t _inst) { return _get_impl(_inst.get()); }
p_LinearDomain(LinearDomain * _pubthis);
virtual ~p_LinearDomain() { /* std::cout << "~p_LinearDomain" << std::endl;*/ };
bool sparse_flag{};bool cardinal_flag{};mosek::fusion::RelationKey key{};std::shared_ptr< monty::ndarray< double,1 > > bnd{};monty::rc_ptr< ::mosek::fusion::Utils::IntMap > inst{};monty::rc_ptr< ::mosek::fusion::Set > shape{};virtual void destroy();
static LinearDomain::t _new_LinearDomain(mosek::fusion::RelationKey _4069,std::shared_ptr< monty::ndarray< double,1 > > _4070,std::shared_ptr< monty::ndarray< long long,1 > > _4071,std::shared_ptr< monty::ndarray< int,1 > > _4072);
void _initialize(mosek::fusion::RelationKey _4069,std::shared_ptr< monty::ndarray< double,1 > > _4070,std::shared_ptr< monty::ndarray< long long,1 > > _4071,std::shared_ptr< monty::ndarray< int,1 > > _4072);
static LinearDomain::t _new_LinearDomain(monty::rc_ptr< ::mosek::fusion::LinearDomain > _4074);
void _initialize(monty::rc_ptr< ::mosek::fusion::LinearDomain > _4074);
virtual monty::rc_ptr< ::mosek::fusion::SymmetricLinearDomain > symmetric();
virtual monty::rc_ptr< ::mosek::fusion::LinearDomain > sparse();
virtual monty::rc_ptr< ::mosek::fusion::LinearDomain > integral();
virtual std::string domainToString(long long _4075,monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _4076);
virtual bool match_shape(monty::rc_ptr< ::mosek::fusion::Set > _4077);
virtual double get_rhs_item(long long _4078);
virtual bool scalable();
}; // struct LinearDomain;

struct p_Domain
{
Domain * _pubthis;
static mosek::fusion::p_Domain* _get_impl(mosek::fusion::Domain * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_Domain * _get_impl(mosek::fusion::Domain::t _inst) { return _get_impl(_inst.get()); }
p_Domain(Domain * _pubthis);
virtual ~p_Domain() { /* std::cout << "~p_Domain" << std::endl;*/ };
virtual void destroy();
static  long long dimsize(std::shared_ptr< monty::ndarray< int,1 > > _4079);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > mkLinearDomain(mosek::fusion::RelationKey _4082,monty::rc_ptr< ::mosek::fusion::Matrix > _4083);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > mkLinearDomain(mosek::fusion::RelationKey _4089,std::shared_ptr< monty::ndarray< double,2 > > _4090);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > mkLinearDomain(mosek::fusion::RelationKey _4093,std::shared_ptr< monty::ndarray< double,1 > > _4094,std::shared_ptr< monty::ndarray< int,1 > > _4095);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > mkLinearDomain(mosek::fusion::RelationKey _4098,std::shared_ptr< monty::ndarray< double,1 > > _4099);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > mkLinearDomain(mosek::fusion::RelationKey _4101,double _4102,std::shared_ptr< monty::ndarray< int,1 > > _4103);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > mkLinearDomain(mosek::fusion::RelationKey _4106,double _4107);
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > inRange(std::shared_ptr< monty::ndarray< double,1 > > _4108,std::shared_ptr< monty::ndarray< double,1 > > _4109,std::shared_ptr< monty::ndarray< int,1 > > _4110);
static  monty::rc_ptr< ::mosek::fusion::SymmetricRangeDomain > symmetric(monty::rc_ptr< ::mosek::fusion::RangeDomain > _4112);
static  monty::rc_ptr< ::mosek::fusion::SymmetricLinearDomain > symmetric(monty::rc_ptr< ::mosek::fusion::LinearDomain > _4113);
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > sparse(monty::rc_ptr< ::mosek::fusion::RangeDomain > _4114);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > sparse(monty::rc_ptr< ::mosek::fusion::LinearDomain > _4115);
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > integral(monty::rc_ptr< ::mosek::fusion::RangeDomain > _4116);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > integral(monty::rc_ptr< ::mosek::fusion::LinearDomain > _4117);
static  monty::rc_ptr< ::mosek::fusion::QConeDomain > integral(monty::rc_ptr< ::mosek::fusion::QConeDomain > _4118);
static  monty::rc_ptr< ::mosek::fusion::QConeDomain > axis(monty::rc_ptr< ::mosek::fusion::QConeDomain > _4119,int _4120);
static  monty::rc_ptr< ::mosek::fusion::QConeDomain > inRotatedQCone(std::shared_ptr< monty::ndarray< int,1 > > _4121);
static  monty::rc_ptr< ::mosek::fusion::QConeDomain > inRotatedQCone(int _4123,int _4124);
static  monty::rc_ptr< ::mosek::fusion::QConeDomain > inRotatedQCone(int _4125);
static  monty::rc_ptr< ::mosek::fusion::QConeDomain > inRotatedQCone();
static  monty::rc_ptr< ::mosek::fusion::QConeDomain > inQCone(std::shared_ptr< monty::ndarray< int,1 > > _4126);
static  monty::rc_ptr< ::mosek::fusion::QConeDomain > inQCone(int _4128,int _4129);
static  monty::rc_ptr< ::mosek::fusion::QConeDomain > inQCone(int _4130);
static  monty::rc_ptr< ::mosek::fusion::QConeDomain > inQCone();
static  monty::rc_ptr< ::mosek::fusion::LinPSDDomain > isLinPSD(int _4131,int _4132);
static  monty::rc_ptr< ::mosek::fusion::LinPSDDomain > isLinPSD(int _4133);
static  monty::rc_ptr< ::mosek::fusion::LinPSDDomain > isLinPSD();
static  monty::rc_ptr< ::mosek::fusion::PSDDomain > isTrilPSD(int _4134,int _4135);
static  monty::rc_ptr< ::mosek::fusion::PSDDomain > isTrilPSD(int _4136);
static  monty::rc_ptr< ::mosek::fusion::PSDDomain > isTrilPSD();
static  monty::rc_ptr< ::mosek::fusion::PSDDomain > inPSDCone(int _4137,int _4138);
static  monty::rc_ptr< ::mosek::fusion::PSDDomain > inPSDCone(int _4139);
static  monty::rc_ptr< ::mosek::fusion::PSDDomain > inPSDCone();
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > binary();
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > binary(std::shared_ptr< monty::ndarray< int,1 > > _4140);
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > binary(int _4143,int _4144);
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > binary(int _4147);
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > inRange(monty::rc_ptr< ::mosek::fusion::Matrix > _4150,monty::rc_ptr< ::mosek::fusion::Matrix > _4151);
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > inRange(monty::rc_ptr< ::mosek::fusion::Matrix > _4152,double _4153);
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > inRange(double _4155,monty::rc_ptr< ::mosek::fusion::Matrix > _4156);
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > inRange(std::shared_ptr< monty::ndarray< double,1 > > _4158,std::shared_ptr< monty::ndarray< double,1 > > _4159);
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > inRange(std::shared_ptr< monty::ndarray< double,1 > > _4160,double _4161);
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > inRange(double _4163,std::shared_ptr< monty::ndarray< double,1 > > _4164);
static  monty::rc_ptr< ::mosek::fusion::RangeDomain > inRange(double _4166,double _4167);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > greaterThan(monty::rc_ptr< ::mosek::fusion::Matrix > _4168);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > greaterThan(std::shared_ptr< monty::ndarray< double,1 > > _4169,std::shared_ptr< monty::ndarray< int,1 > > _4170);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > greaterThan(std::shared_ptr< monty::ndarray< double,2 > > _4171);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > greaterThan(std::shared_ptr< monty::ndarray< double,1 > > _4172);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > greaterThan(double _4173,std::shared_ptr< monty::ndarray< int,1 > > _4174);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > greaterThan(double _4175,int _4176,int _4177);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > greaterThan(double _4178,int _4179);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > greaterThan(double _4180);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > lessThan(monty::rc_ptr< ::mosek::fusion::Matrix > _4181);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > lessThan(std::shared_ptr< monty::ndarray< double,1 > > _4182,std::shared_ptr< monty::ndarray< int,1 > > _4183);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > lessThan(std::shared_ptr< monty::ndarray< double,2 > > _4184);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > lessThan(std::shared_ptr< monty::ndarray< double,1 > > _4185);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > lessThan(double _4186,std::shared_ptr< monty::ndarray< int,1 > > _4187);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > lessThan(double _4188,int _4189,int _4190);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > lessThan(double _4191,int _4192);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > lessThan(double _4193);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > equalsTo(monty::rc_ptr< ::mosek::fusion::Matrix > _4194);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > equalsTo(std::shared_ptr< monty::ndarray< double,1 > > _4195,std::shared_ptr< monty::ndarray< int,1 > > _4196);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > equalsTo(std::shared_ptr< monty::ndarray< double,2 > > _4197);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > equalsTo(std::shared_ptr< monty::ndarray< double,1 > > _4198);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > equalsTo(double _4199,std::shared_ptr< monty::ndarray< int,1 > > _4200);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > equalsTo(double _4201,int _4202,int _4203);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > equalsTo(double _4204,int _4205);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > equalsTo(double _4206);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > unbounded(std::shared_ptr< monty::ndarray< int,1 > > _4207);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > unbounded(int _4209,int _4210);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > unbounded(int _4211);
static  monty::rc_ptr< ::mosek::fusion::LinearDomain > unbounded();
}; // struct Domain;

struct p_SymmetricExpr
{
SymmetricExpr * _pubthis;
static mosek::fusion::p_SymmetricExpr* _get_impl(mosek::fusion::SymmetricExpr * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_SymmetricExpr * _get_impl(mosek::fusion::SymmetricExpr::t _inst) { return _get_impl(_inst.get()); }
p_SymmetricExpr(SymmetricExpr * _pubthis);
virtual ~p_SymmetricExpr() { /* std::cout << "~p_SymmetricExpr" << std::endl;*/ };
std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > xs{};monty::rc_ptr< ::mosek::fusion::SymmetricMatrix > b{};std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::SymmetricMatrix >,1 > > Ms{};int n{};virtual void destroy();
static SymmetricExpr::t _new_SymmetricExpr(int _4220,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::SymmetricMatrix >,1 > > _4221,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _4222,monty::rc_ptr< ::mosek::fusion::SymmetricMatrix > _4223);
void _initialize(int _4220,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::SymmetricMatrix >,1 > > _4221,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _4222,monty::rc_ptr< ::mosek::fusion::SymmetricMatrix > _4223);
static  monty::rc_ptr< ::mosek::fusion::SymmetricExpr > add(monty::rc_ptr< ::mosek::fusion::SymmetricExpr > _4224,monty::rc_ptr< ::mosek::fusion::SymmetricMatrix > _4225);
static  monty::rc_ptr< ::mosek::fusion::SymmetricExpr > mul(monty::rc_ptr< ::mosek::fusion::SymmetricExpr > _4226,double _4227);
static  monty::rc_ptr< ::mosek::fusion::SymmetricExpr > add(monty::rc_ptr< ::mosek::fusion::SymmetricExpr > _4229,monty::rc_ptr< ::mosek::fusion::SymmetricExpr > _4230);
virtual std::string toString();
}; // struct SymmetricExpr;

struct p_Expr : public /*implements*/ ::mosek::fusion::Expression
{
Expr * _pubthis;
static mosek::fusion::p_Expr* _get_impl(mosek::fusion::Expr * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_Expr * _get_impl(mosek::fusion::Expr::t _inst) { return _get_impl(_inst.get()); }
p_Expr(Expr * _pubthis);
virtual ~p_Expr() { /* std::cout << "~p_Expr" << std::endl;*/ };
std::shared_ptr< monty::ndarray< long long,1 > > varsb{};std::shared_ptr< monty::ndarray< long long,1 > > inst{};std::shared_ptr< monty::ndarray< double,1 > > cof_v{};std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > x{};std::shared_ptr< monty::ndarray< long long,1 > > subj{};std::shared_ptr< monty::ndarray< long long,1 > > ptrb{};std::shared_ptr< monty::ndarray< double,1 > > bfix{};monty::rc_ptr< ::mosek::fusion::Set > shape_p{};monty::rc_ptr< ::mosek::fusion::Model > model{};virtual void destroy();
static Expr::t _new_Expr(std::shared_ptr< monty::ndarray< long long,1 > > _4241,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _4242,std::shared_ptr< monty::ndarray< long long,1 > > _4243,std::shared_ptr< monty::ndarray< double,1 > > _4244,std::shared_ptr< monty::ndarray< double,1 > > _4245,monty::rc_ptr< ::mosek::fusion::Set > _4246,std::shared_ptr< monty::ndarray< long long,1 > > _4247);
void _initialize(std::shared_ptr< monty::ndarray< long long,1 > > _4241,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _4242,std::shared_ptr< monty::ndarray< long long,1 > > _4243,std::shared_ptr< monty::ndarray< double,1 > > _4244,std::shared_ptr< monty::ndarray< double,1 > > _4245,monty::rc_ptr< ::mosek::fusion::Set > _4246,std::shared_ptr< monty::ndarray< long long,1 > > _4247);
static Expr::t _new_Expr(std::shared_ptr< monty::ndarray< long long,1 > > _4251,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _4252,std::shared_ptr< monty::ndarray< long long,1 > > _4253,std::shared_ptr< monty::ndarray< double,1 > > _4254,std::shared_ptr< monty::ndarray< double,1 > > _4255,monty::rc_ptr< ::mosek::fusion::Set > _4256,std::shared_ptr< monty::ndarray< long long,1 > > _4257,int _4258);
void _initialize(std::shared_ptr< monty::ndarray< long long,1 > > _4251,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _4252,std::shared_ptr< monty::ndarray< long long,1 > > _4253,std::shared_ptr< monty::ndarray< double,1 > > _4254,std::shared_ptr< monty::ndarray< double,1 > > _4255,monty::rc_ptr< ::mosek::fusion::Set > _4256,std::shared_ptr< monty::ndarray< long long,1 > > _4257,int _4258);
static Expr::t _new_Expr(monty::rc_ptr< ::mosek::fusion::Expression > _4260);
void _initialize(monty::rc_ptr< ::mosek::fusion::Expression > _4260);
virtual std::string toString();
virtual void tostr(monty::rc_ptr< ::mosek::fusion::Utils::StringBuffer > _4269,int _4270);
static  std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > varstack(std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > >,1 > > _4276);
static  std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > varstack(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _4279,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _4280);
static  monty::rc_ptr< ::mosek::fusion::Expression > flatten(monty::rc_ptr< ::mosek::fusion::Expression > _4284);
static  monty::rc_ptr< ::mosek::fusion::Expression > reshape(monty::rc_ptr< ::mosek::fusion::Expression > _4285,int _4286,int _4287);
static  monty::rc_ptr< ::mosek::fusion::Expression > reshape(monty::rc_ptr< ::mosek::fusion::Expression > _4288,int _4289);
static  monty::rc_ptr< ::mosek::fusion::Expression > reshape(monty::rc_ptr< ::mosek::fusion::Expression > _4290,monty::rc_ptr< ::mosek::fusion::Set > _4291);
virtual long long size();
virtual monty::rc_ptr< ::mosek::fusion::FlatExpr > eval();
static  monty::rc_ptr< ::mosek::fusion::Expression > zeros(int _4297);
static  monty::rc_ptr< ::mosek::fusion::Expression > ones(int _4303);
static  monty::rc_ptr< ::mosek::fusion::Expression > constTerm(monty::rc_ptr< ::mosek::fusion::NDSparseArray > _4308);
static  monty::rc_ptr< ::mosek::fusion::Expression > constTerm(monty::rc_ptr< ::mosek::fusion::Matrix > _4316);
static  monty::rc_ptr< ::mosek::fusion::Expression > constTerm(double _4328);
static  monty::rc_ptr< ::mosek::fusion::Expression > constTerm(monty::rc_ptr< ::mosek::fusion::Set > _4334,double _4335);
static  monty::rc_ptr< ::mosek::fusion::Expression > constTerm(int _4341,double _4342);
static  monty::rc_ptr< ::mosek::fusion::Expression > constTerm(std::shared_ptr< monty::ndarray< double,2 > > _4348);
static  monty::rc_ptr< ::mosek::fusion::Expression > constTerm(std::shared_ptr< monty::ndarray< double,1 > > _4358);
virtual long long numNonzeros();
static  monty::rc_ptr< ::mosek::fusion::Expression > sum_expr(monty::rc_ptr< ::mosek::fusion::Expression > _4364,int _4365,int _4366);
static  monty::rc_ptr< ::mosek::fusion::Expression > sum_var(monty::rc_ptr< ::mosek::fusion::Variable > _4412,int _4413,int _4414);
static  monty::rc_ptr< ::mosek::fusion::Expression > sum(monty::rc_ptr< ::mosek::fusion::Expression > _4444,int _4445,int _4446);
static  monty::rc_ptr< ::mosek::fusion::Expression > sum(monty::rc_ptr< ::mosek::fusion::Expression > _4447,int _4448);
static  monty::rc_ptr< ::mosek::fusion::Expression > sum(monty::rc_ptr< ::mosek::fusion::Variable > _4449,int _4450,int _4451);
static  monty::rc_ptr< ::mosek::fusion::Expression > sum(monty::rc_ptr< ::mosek::fusion::Variable > _4452,int _4453);
static  monty::rc_ptr< ::mosek::fusion::Expression > sum(monty::rc_ptr< ::mosek::fusion::Variable > _4454);
static  monty::rc_ptr< ::mosek::fusion::Expression > sum(monty::rc_ptr< ::mosek::fusion::Expression > _4455);
static  monty::rc_ptr< ::mosek::fusion::Expression > neg(monty::rc_ptr< ::mosek::fusion::Variable > _4463);
static  monty::rc_ptr< ::mosek::fusion::Expression > neg(monty::rc_ptr< ::mosek::fusion::Expression > _4468);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul__(monty::rc_ptr< ::mosek::fusion::Matrix > _4474,monty::rc_ptr< ::mosek::fusion::Expression > _4475);
static  void sparseMatrixVector(std::shared_ptr< monty::ndarray< long long,1 > > _4587,std::shared_ptr< monty::ndarray< int,1 > > _4588,std::shared_ptr< monty::ndarray< double,1 > > _4589,std::shared_ptr< monty::ndarray< double,1 > > _4590,std::shared_ptr< monty::ndarray< double,1 > > _4591,int _4592);
static  void sparseMatmul(std::shared_ptr< monty::ndarray< long long,1 > > _4597,std::shared_ptr< monty::ndarray< int,1 > > _4598,std::shared_ptr< monty::ndarray< double,1 > > _4599,std::shared_ptr< monty::ndarray< long long,1 > > _4600,std::shared_ptr< monty::ndarray< int,1 > > _4601,std::shared_ptr< monty::ndarray< double,1 > > _4602,std::shared_ptr< monty::ndarray< long long,1 > > _4603,std::shared_ptr< monty::ndarray< int,1 > > _4604,std::shared_ptr< monty::ndarray< double,1 > > _4605,int _4606,int _4607,std::shared_ptr< monty::ndarray< int,1 > > _4608);
static  long long computeNz(std::shared_ptr< monty::ndarray< long long,1 > > _4620,std::shared_ptr< monty::ndarray< int,1 > > _4621,std::shared_ptr< monty::ndarray< long long,1 > > _4622,std::shared_ptr< monty::ndarray< int,1 > > _4623,int _4624,int _4625,std::shared_ptr< monty::ndarray< int,1 > > _4626,std::shared_ptr< monty::ndarray< long long,1 > > _4627);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulDiag(monty::rc_ptr< ::mosek::fusion::Variable > _4636,monty::rc_ptr< ::mosek::fusion::Matrix > _4637);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulDiag(monty::rc_ptr< ::mosek::fusion::Matrix > _4662,monty::rc_ptr< ::mosek::fusion::Variable > _4663);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulDiag(monty::rc_ptr< ::mosek::fusion::Expression > _4682,monty::rc_ptr< ::mosek::fusion::Matrix > _4683);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulDiag(monty::rc_ptr< ::mosek::fusion::Matrix > _4766,monty::rc_ptr< ::mosek::fusion::Expression > _4767);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulDiag(monty::rc_ptr< ::mosek::fusion::Variable > _4832,std::shared_ptr< monty::ndarray< double,2 > > _4833);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulDiag(std::shared_ptr< monty::ndarray< double,2 > > _4834,monty::rc_ptr< ::mosek::fusion::Variable > _4835);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulDiag(monty::rc_ptr< ::mosek::fusion::Expression > _4836,std::shared_ptr< monty::ndarray< double,2 > > _4837);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulDiag(std::shared_ptr< monty::ndarray< double,2 > > _4838,monty::rc_ptr< ::mosek::fusion::Expression > _4839);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm_(monty::rc_ptr< ::mosek::fusion::Matrix > _4840,monty::rc_ptr< ::mosek::fusion::Expression > _4841);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm_(monty::rc_ptr< ::mosek::fusion::Matrix > _4849,monty::rc_ptr< ::mosek::fusion::Variable > _4850);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm_(std::shared_ptr< monty::ndarray< double,1 > > _4857,monty::rc_ptr< ::mosek::fusion::Variable > _4858);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm_(std::shared_ptr< monty::ndarray< double,1 > > _4859,monty::rc_ptr< ::mosek::fusion::Expression > _4860);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm_(monty::rc_ptr< ::mosek::fusion::NDSparseArray > _4862,monty::rc_ptr< ::mosek::fusion::Expression > _4863);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm_(monty::rc_ptr< ::mosek::fusion::NDSparseArray > _4868,monty::rc_ptr< ::mosek::fusion::Variable > _4869);
static  monty::rc_ptr< ::mosek::fusion::Expression > dotmul_(std::shared_ptr< monty::ndarray< long long,1 > > _4871,std::shared_ptr< monty::ndarray< double,1 > > _4872,monty::rc_ptr< ::mosek::fusion::Variable > _4873,monty::rc_ptr< ::mosek::fusion::Set > _4874);
static  monty::rc_ptr< ::mosek::fusion::Expression > dotmul_(std::shared_ptr< monty::ndarray< long long,1 > > _4880,std::shared_ptr< monty::ndarray< double,1 > > _4881,std::shared_ptr< monty::ndarray< long long,1 > > _4882,std::shared_ptr< monty::ndarray< long long,1 > > _4883,std::shared_ptr< monty::ndarray< double,1 > > _4884,std::shared_ptr< monty::ndarray< double,1 > > _4885,std::shared_ptr< monty::ndarray< long long,1 > > _4886,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _4887,monty::rc_ptr< ::mosek::fusion::Set > _4888);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(monty::rc_ptr< ::mosek::fusion::Matrix > _4909,monty::rc_ptr< ::mosek::fusion::Expression > _4910);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(monty::rc_ptr< ::mosek::fusion::Expression > _4923,monty::rc_ptr< ::mosek::fusion::Matrix > _4924);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(monty::rc_ptr< ::mosek::fusion::Expression > _4937,std::shared_ptr< monty::ndarray< double,1 > > _4938);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(std::shared_ptr< monty::ndarray< double,1 > > _4946,monty::rc_ptr< ::mosek::fusion::Expression > _4947);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(double _4955,monty::rc_ptr< ::mosek::fusion::Expression > _4956);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(monty::rc_ptr< ::mosek::fusion::Expression > _4960,double _4961);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul_SMatrix_2DSExpr(std::shared_ptr< monty::ndarray< long long,1 > > _4962,std::shared_ptr< monty::ndarray< long long,1 > > _4963,std::shared_ptr< monty::ndarray< double,1 > > _4964,std::shared_ptr< monty::ndarray< double,1 > > _4965,std::shared_ptr< monty::ndarray< long long,1 > > _4966,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _4967,monty::rc_ptr< ::mosek::fusion::Set > _4968,int _4969,int _4970,std::shared_ptr< monty::ndarray< int,1 > > _4971,std::shared_ptr< monty::ndarray< int,1 > > _4972,std::shared_ptr< monty::ndarray< double,1 > > _4973,int _4974,int _4975);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul_2DSExpr_SMatrix(std::shared_ptr< monty::ndarray< long long,1 > > _5027,std::shared_ptr< monty::ndarray< long long,1 > > _5028,std::shared_ptr< monty::ndarray< double,1 > > _5029,std::shared_ptr< monty::ndarray< double,1 > > _5030,std::shared_ptr< monty::ndarray< long long,1 > > _5031,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _5032,monty::rc_ptr< ::mosek::fusion::Set > _5033,int _5034,int _5035,std::shared_ptr< monty::ndarray< int,1 > > _5036,std::shared_ptr< monty::ndarray< int,1 > > _5037,std::shared_ptr< monty::ndarray< double,1 > > _5038,int _5039,int _5040);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul_DMatrix_2DDExpr(std::shared_ptr< monty::ndarray< long long,1 > > _5088,std::shared_ptr< monty::ndarray< long long,1 > > _5089,std::shared_ptr< monty::ndarray< double,1 > > _5090,std::shared_ptr< monty::ndarray< double,1 > > _5091,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _5092,monty::rc_ptr< ::mosek::fusion::Set > _5093,int _5094,int _5095,std::shared_ptr< monty::ndarray< double,1 > > _5096,int _5097,int _5098);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul_2DDExpr_DMatrix(std::shared_ptr< monty::ndarray< long long,1 > > _5118,std::shared_ptr< monty::ndarray< long long,1 > > _5119,std::shared_ptr< monty::ndarray< double,1 > > _5120,std::shared_ptr< monty::ndarray< double,1 > > _5121,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _5122,monty::rc_ptr< ::mosek::fusion::Set > _5123,int _5124,int _5125,std::shared_ptr< monty::ndarray< double,1 > > _5126,int _5127,int _5128);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul_0DExpr_Matrix(std::shared_ptr< monty::ndarray< long long,1 > > _5150,std::shared_ptr< monty::ndarray< double,1 > > _5151,std::shared_ptr< monty::ndarray< double,1 > > _5152,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _5153,monty::rc_ptr< ::mosek::fusion::Matrix > _5154);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(monty::rc_ptr< ::mosek::fusion::Variable > _5172,std::shared_ptr< monty::ndarray< double,2 > > _5173);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(std::shared_ptr< monty::ndarray< double,2 > > _5177,monty::rc_ptr< ::mosek::fusion::Variable > _5178);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(monty::rc_ptr< ::mosek::fusion::Variable > _5179,double _5180);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(double _5181,monty::rc_ptr< ::mosek::fusion::Variable > _5182);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(std::shared_ptr< monty::ndarray< double,1 > > _5189,monty::rc_ptr< ::mosek::fusion::Variable > _5190);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(monty::rc_ptr< ::mosek::fusion::Variable > _5202,std::shared_ptr< monty::ndarray< double,1 > > _5203);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(monty::rc_ptr< ::mosek::fusion::Variable > _5221,monty::rc_ptr< ::mosek::fusion::Matrix > _5222);
static  monty::rc_ptr< ::mosek::fusion::Expression > mul(monty::rc_ptr< ::mosek::fusion::Matrix > _5265,monty::rc_ptr< ::mosek::fusion::Variable > _5266);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot_(monty::rc_ptr< ::mosek::fusion::Matrix > _5312,monty::rc_ptr< ::mosek::fusion::Expression > _5313);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot_(monty::rc_ptr< ::mosek::fusion::Matrix > _5321,monty::rc_ptr< ::mosek::fusion::Variable > _5322);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot_(std::shared_ptr< monty::ndarray< double,1 > > _5329,monty::rc_ptr< ::mosek::fusion::Variable > _5330);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot_(std::shared_ptr< monty::ndarray< double,1 > > _5331,monty::rc_ptr< ::mosek::fusion::Expression > _5332);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot_(monty::rc_ptr< ::mosek::fusion::NDSparseArray > _5334,monty::rc_ptr< ::mosek::fusion::Expression > _5335);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot_(monty::rc_ptr< ::mosek::fusion::NDSparseArray > _5340,monty::rc_ptr< ::mosek::fusion::Variable > _5341);
static  monty::rc_ptr< ::mosek::fusion::Expression > inner_(std::shared_ptr< monty::ndarray< long long,1 > > _5343,std::shared_ptr< monty::ndarray< double,1 > > _5344,monty::rc_ptr< ::mosek::fusion::Variable > _5345);
static  monty::rc_ptr< ::mosek::fusion::Expression > inner_(std::shared_ptr< monty::ndarray< long long,1 > > _5351,std::shared_ptr< monty::ndarray< double,1 > > _5352,std::shared_ptr< monty::ndarray< long long,1 > > _5353,std::shared_ptr< monty::ndarray< long long,1 > > _5354,std::shared_ptr< monty::ndarray< double,1 > > _5355,std::shared_ptr< monty::ndarray< double,1 > > _5356,std::shared_ptr< monty::ndarray< long long,1 > > _5357,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _5358);
static  monty::rc_ptr< ::mosek::fusion::Expression > outer(std::shared_ptr< monty::ndarray< double,1 > > _5375,monty::rc_ptr< ::mosek::fusion::Expression > _5376);
static  monty::rc_ptr< ::mosek::fusion::Expression > outer(monty::rc_ptr< ::mosek::fusion::Expression > _5400,std::shared_ptr< monty::ndarray< double,1 > > _5401);
static  monty::rc_ptr< ::mosek::fusion::Expression > outer(std::shared_ptr< monty::ndarray< double,1 > > _5425,monty::rc_ptr< ::mosek::fusion::Variable > _5426);
static  monty::rc_ptr< ::mosek::fusion::Expression > outer(monty::rc_ptr< ::mosek::fusion::Variable > _5436,std::shared_ptr< monty::ndarray< double,1 > > _5437);
virtual monty::rc_ptr< ::mosek::fusion::Expression > pick(std::shared_ptr< monty::ndarray< int,2 > > _5447);
virtual monty::rc_ptr< ::mosek::fusion::Expression > pick(std::shared_ptr< monty::ndarray< int,1 > > _5453);
virtual monty::rc_ptr< ::mosek::fusion::Expression > pick_(std::shared_ptr< monty::ndarray< long long,1 > > _5456);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Expression >,1 > >,1 > > _5494);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(double _5500,double _5501,double _5502);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(double _5503,double _5504,monty::rc_ptr< ::mosek::fusion::Variable > _5505);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(double _5506,double _5507,monty::rc_ptr< ::mosek::fusion::Expression > _5508);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(double _5509,monty::rc_ptr< ::mosek::fusion::Variable > _5510,double _5511);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(double _5512,monty::rc_ptr< ::mosek::fusion::Variable > _5513,monty::rc_ptr< ::mosek::fusion::Variable > _5514);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(double _5515,monty::rc_ptr< ::mosek::fusion::Variable > _5516,monty::rc_ptr< ::mosek::fusion::Expression > _5517);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(double _5518,monty::rc_ptr< ::mosek::fusion::Expression > _5519,double _5520);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(double _5521,monty::rc_ptr< ::mosek::fusion::Expression > _5522,monty::rc_ptr< ::mosek::fusion::Variable > _5523);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(double _5524,monty::rc_ptr< ::mosek::fusion::Expression > _5525,monty::rc_ptr< ::mosek::fusion::Expression > _5526);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _5527,double _5528,double _5529);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _5530,double _5531,monty::rc_ptr< ::mosek::fusion::Variable > _5532);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _5533,double _5534,monty::rc_ptr< ::mosek::fusion::Expression > _5535);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _5536,monty::rc_ptr< ::mosek::fusion::Variable > _5537,double _5538);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _5539,monty::rc_ptr< ::mosek::fusion::Variable > _5540,monty::rc_ptr< ::mosek::fusion::Variable > _5541);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _5542,monty::rc_ptr< ::mosek::fusion::Variable > _5543,monty::rc_ptr< ::mosek::fusion::Expression > _5544);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _5545,monty::rc_ptr< ::mosek::fusion::Expression > _5546,double _5547);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _5548,monty::rc_ptr< ::mosek::fusion::Expression > _5549,monty::rc_ptr< ::mosek::fusion::Variable > _5550);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _5551,monty::rc_ptr< ::mosek::fusion::Expression > _5552,monty::rc_ptr< ::mosek::fusion::Expression > _5553);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Expression > _5554,double _5555,double _5556);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Expression > _5557,double _5558,monty::rc_ptr< ::mosek::fusion::Variable > _5559);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Expression > _5560,double _5561,monty::rc_ptr< ::mosek::fusion::Expression > _5562);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Expression > _5563,monty::rc_ptr< ::mosek::fusion::Variable > _5564,double _5565);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Expression > _5566,monty::rc_ptr< ::mosek::fusion::Variable > _5567,monty::rc_ptr< ::mosek::fusion::Variable > _5568);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Expression > _5569,monty::rc_ptr< ::mosek::fusion::Variable > _5570,monty::rc_ptr< ::mosek::fusion::Expression > _5571);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Expression > _5572,monty::rc_ptr< ::mosek::fusion::Expression > _5573,double _5574);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Expression > _5575,monty::rc_ptr< ::mosek::fusion::Expression > _5576,monty::rc_ptr< ::mosek::fusion::Variable > _5577);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Expression > _5578,monty::rc_ptr< ::mosek::fusion::Expression > _5579,monty::rc_ptr< ::mosek::fusion::Expression > _5580);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(double _5581,monty::rc_ptr< ::mosek::fusion::Variable > _5582);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(double _5583,monty::rc_ptr< ::mosek::fusion::Expression > _5584);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _5585,double _5586);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _5587,monty::rc_ptr< ::mosek::fusion::Variable > _5588);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Variable > _5589,monty::rc_ptr< ::mosek::fusion::Expression > _5590);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Expression > _5591,double _5592);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Expression > _5593,monty::rc_ptr< ::mosek::fusion::Variable > _5594);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(monty::rc_ptr< ::mosek::fusion::Expression > _5595,monty::rc_ptr< ::mosek::fusion::Expression > _5596);
static  monty::rc_ptr< ::mosek::fusion::Expression > vstack(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Expression >,1 > > _5597);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Expression > _5599,monty::rc_ptr< ::mosek::fusion::Expression > _5600,monty::rc_ptr< ::mosek::fusion::Expression > _5601);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Expression > _5602,monty::rc_ptr< ::mosek::fusion::Expression > _5603,monty::rc_ptr< ::mosek::fusion::Variable > _5604);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Expression > _5605,monty::rc_ptr< ::mosek::fusion::Expression > _5606,double _5607);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Expression > _5608,monty::rc_ptr< ::mosek::fusion::Variable > _5609,monty::rc_ptr< ::mosek::fusion::Expression > _5610);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Expression > _5611,monty::rc_ptr< ::mosek::fusion::Variable > _5612,monty::rc_ptr< ::mosek::fusion::Variable > _5613);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Expression > _5614,monty::rc_ptr< ::mosek::fusion::Variable > _5615,double _5616);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Expression > _5617,double _5618,monty::rc_ptr< ::mosek::fusion::Expression > _5619);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Expression > _5620,double _5621,monty::rc_ptr< ::mosek::fusion::Variable > _5622);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Expression > _5623,double _5624,double _5625);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _5626,monty::rc_ptr< ::mosek::fusion::Expression > _5627,monty::rc_ptr< ::mosek::fusion::Expression > _5628);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _5629,monty::rc_ptr< ::mosek::fusion::Expression > _5630,monty::rc_ptr< ::mosek::fusion::Variable > _5631);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _5632,monty::rc_ptr< ::mosek::fusion::Expression > _5633,double _5634);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _5635,monty::rc_ptr< ::mosek::fusion::Variable > _5636,monty::rc_ptr< ::mosek::fusion::Expression > _5637);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _5638,monty::rc_ptr< ::mosek::fusion::Variable > _5639,monty::rc_ptr< ::mosek::fusion::Variable > _5640);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _5641,monty::rc_ptr< ::mosek::fusion::Variable > _5642,double _5643);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _5644,double _5645,monty::rc_ptr< ::mosek::fusion::Expression > _5646);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _5647,double _5648,monty::rc_ptr< ::mosek::fusion::Variable > _5649);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _5650,double _5651,double _5652);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(double _5653,monty::rc_ptr< ::mosek::fusion::Expression > _5654,monty::rc_ptr< ::mosek::fusion::Expression > _5655);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(double _5656,monty::rc_ptr< ::mosek::fusion::Expression > _5657,monty::rc_ptr< ::mosek::fusion::Variable > _5658);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(double _5659,monty::rc_ptr< ::mosek::fusion::Expression > _5660,double _5661);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(double _5662,monty::rc_ptr< ::mosek::fusion::Variable > _5663,monty::rc_ptr< ::mosek::fusion::Expression > _5664);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(double _5665,monty::rc_ptr< ::mosek::fusion::Variable > _5666,monty::rc_ptr< ::mosek::fusion::Variable > _5667);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(double _5668,monty::rc_ptr< ::mosek::fusion::Variable > _5669,double _5670);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(double _5671,double _5672,monty::rc_ptr< ::mosek::fusion::Expression > _5673);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(double _5674,double _5675,monty::rc_ptr< ::mosek::fusion::Variable > _5676);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _5677,monty::rc_ptr< ::mosek::fusion::Expression > _5678);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _5679,monty::rc_ptr< ::mosek::fusion::Variable > _5680);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Variable > _5681,double _5682);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(double _5683,monty::rc_ptr< ::mosek::fusion::Expression > _5684);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(double _5685,monty::rc_ptr< ::mosek::fusion::Variable > _5686);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Expression > _5687,monty::rc_ptr< ::mosek::fusion::Variable > _5688);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Expression > _5689,double _5690);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(monty::rc_ptr< ::mosek::fusion::Expression > _5691,monty::rc_ptr< ::mosek::fusion::Expression > _5692);
static  monty::rc_ptr< ::mosek::fusion::Expression > hstack(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Expression >,1 > > _5693);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5695,monty::rc_ptr< ::mosek::fusion::Expression > _5696,monty::rc_ptr< ::mosek::fusion::Expression > _5697,monty::rc_ptr< ::mosek::fusion::Expression > _5698);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5699,monty::rc_ptr< ::mosek::fusion::Expression > _5700,monty::rc_ptr< ::mosek::fusion::Expression > _5701,monty::rc_ptr< ::mosek::fusion::Variable > _5702);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5703,monty::rc_ptr< ::mosek::fusion::Expression > _5704,monty::rc_ptr< ::mosek::fusion::Expression > _5705,double _5706);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5707,monty::rc_ptr< ::mosek::fusion::Expression > _5708,monty::rc_ptr< ::mosek::fusion::Variable > _5709,monty::rc_ptr< ::mosek::fusion::Expression > _5710);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5711,monty::rc_ptr< ::mosek::fusion::Expression > _5712,monty::rc_ptr< ::mosek::fusion::Variable > _5713,monty::rc_ptr< ::mosek::fusion::Variable > _5714);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5715,monty::rc_ptr< ::mosek::fusion::Expression > _5716,monty::rc_ptr< ::mosek::fusion::Variable > _5717,double _5718);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5719,monty::rc_ptr< ::mosek::fusion::Expression > _5720,double _5721,monty::rc_ptr< ::mosek::fusion::Expression > _5722);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5723,monty::rc_ptr< ::mosek::fusion::Expression > _5724,double _5725,monty::rc_ptr< ::mosek::fusion::Variable > _5726);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5727,monty::rc_ptr< ::mosek::fusion::Expression > _5728,double _5729,double _5730);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5731,monty::rc_ptr< ::mosek::fusion::Variable > _5732,monty::rc_ptr< ::mosek::fusion::Expression > _5733,monty::rc_ptr< ::mosek::fusion::Expression > _5734);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5735,monty::rc_ptr< ::mosek::fusion::Variable > _5736,monty::rc_ptr< ::mosek::fusion::Expression > _5737,monty::rc_ptr< ::mosek::fusion::Variable > _5738);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5739,monty::rc_ptr< ::mosek::fusion::Variable > _5740,monty::rc_ptr< ::mosek::fusion::Expression > _5741,double _5742);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5743,monty::rc_ptr< ::mosek::fusion::Variable > _5744,monty::rc_ptr< ::mosek::fusion::Variable > _5745,monty::rc_ptr< ::mosek::fusion::Expression > _5746);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5747,monty::rc_ptr< ::mosek::fusion::Variable > _5748,monty::rc_ptr< ::mosek::fusion::Variable > _5749,monty::rc_ptr< ::mosek::fusion::Variable > _5750);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5751,monty::rc_ptr< ::mosek::fusion::Variable > _5752,monty::rc_ptr< ::mosek::fusion::Variable > _5753,double _5754);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5755,monty::rc_ptr< ::mosek::fusion::Variable > _5756,double _5757,monty::rc_ptr< ::mosek::fusion::Expression > _5758);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5759,monty::rc_ptr< ::mosek::fusion::Variable > _5760,double _5761,monty::rc_ptr< ::mosek::fusion::Variable > _5762);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5763,monty::rc_ptr< ::mosek::fusion::Variable > _5764,double _5765,double _5766);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5767,double _5768,monty::rc_ptr< ::mosek::fusion::Expression > _5769,monty::rc_ptr< ::mosek::fusion::Expression > _5770);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5771,double _5772,monty::rc_ptr< ::mosek::fusion::Expression > _5773,monty::rc_ptr< ::mosek::fusion::Variable > _5774);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5775,double _5776,monty::rc_ptr< ::mosek::fusion::Expression > _5777,double _5778);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5779,double _5780,monty::rc_ptr< ::mosek::fusion::Variable > _5781,monty::rc_ptr< ::mosek::fusion::Expression > _5782);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5783,double _5784,monty::rc_ptr< ::mosek::fusion::Variable > _5785,monty::rc_ptr< ::mosek::fusion::Variable > _5786);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5787,double _5788,monty::rc_ptr< ::mosek::fusion::Variable > _5789,double _5790);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5791,double _5792,double _5793,monty::rc_ptr< ::mosek::fusion::Expression > _5794);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5795,double _5796,double _5797,monty::rc_ptr< ::mosek::fusion::Variable > _5798);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5799,monty::rc_ptr< ::mosek::fusion::Variable > _5800,monty::rc_ptr< ::mosek::fusion::Expression > _5801);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5802,monty::rc_ptr< ::mosek::fusion::Variable > _5803,monty::rc_ptr< ::mosek::fusion::Variable > _5804);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5805,monty::rc_ptr< ::mosek::fusion::Variable > _5806,double _5807);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5808,double _5809,monty::rc_ptr< ::mosek::fusion::Expression > _5810);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5811,double _5812,monty::rc_ptr< ::mosek::fusion::Variable > _5813);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5814,monty::rc_ptr< ::mosek::fusion::Expression > _5815,monty::rc_ptr< ::mosek::fusion::Variable > _5816);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5817,monty::rc_ptr< ::mosek::fusion::Expression > _5818,double _5819);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5820,monty::rc_ptr< ::mosek::fusion::Expression > _5821,monty::rc_ptr< ::mosek::fusion::Expression > _5822);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack(int _5823,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Expression >,1 > > _5824);
static  monty::rc_ptr< ::mosek::fusion::Expression > stack_(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Expression >,1 > > _5825,int _5826);
static  monty::rc_ptr< ::mosek::fusion::Expression > repeat(monty::rc_ptr< ::mosek::fusion::Expression > _5900,int _5901,int _5902);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Expression >,1 > > _5904);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _5974);
static  monty::rc_ptr< ::mosek::fusion::Expression > add_(monty::rc_ptr< ::mosek::fusion::Expression > _5987,double _5988,monty::rc_ptr< ::mosek::fusion::Expression > _5989,double _5990);
virtual monty::rc_ptr< ::mosek::fusion::Expression > transpose();
virtual monty::rc_ptr< ::mosek::fusion::Expression > slice(std::shared_ptr< monty::ndarray< int,1 > > _6090,std::shared_ptr< monty::ndarray< int,1 > > _6091);
virtual monty::rc_ptr< ::mosek::fusion::Expression > index(std::shared_ptr< monty::ndarray< int,1 > > _6132);
virtual monty::rc_ptr< ::mosek::fusion::Expression > index(int _6135);
virtual monty::rc_ptr< ::mosek::fusion::Expression > slice(int _6136,int _6137);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(monty::rc_ptr< ::mosek::fusion::Matrix > _6138,monty::rc_ptr< ::mosek::fusion::Expression > _6139);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(monty::rc_ptr< ::mosek::fusion::Matrix > _6140,monty::rc_ptr< ::mosek::fusion::Variable > _6141);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6142,monty::rc_ptr< ::mosek::fusion::Variable > _6143);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6144,monty::rc_ptr< ::mosek::fusion::Expression > _6145);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(std::shared_ptr< monty::ndarray< double,2 > > _6146,monty::rc_ptr< ::mosek::fusion::Variable > _6147);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(std::shared_ptr< monty::ndarray< double,2 > > _6148,monty::rc_ptr< ::mosek::fusion::Expression > _6149);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(std::shared_ptr< monty::ndarray< double,1 > > _6150,monty::rc_ptr< ::mosek::fusion::Variable > _6151);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(std::shared_ptr< monty::ndarray< double,1 > > _6152,monty::rc_ptr< ::mosek::fusion::Expression > _6153);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(monty::rc_ptr< ::mosek::fusion::Expression > _6154,monty::rc_ptr< ::mosek::fusion::Matrix > _6155);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(monty::rc_ptr< ::mosek::fusion::Expression > _6156,std::shared_ptr< monty::ndarray< double,2 > > _6157);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(monty::rc_ptr< ::mosek::fusion::Expression > _6158,std::shared_ptr< monty::ndarray< double,1 > > _6159);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(monty::rc_ptr< ::mosek::fusion::Expression > _6160,monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6161);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(monty::rc_ptr< ::mosek::fusion::Variable > _6162,monty::rc_ptr< ::mosek::fusion::Matrix > _6163);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(monty::rc_ptr< ::mosek::fusion::Variable > _6164,monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6165);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(monty::rc_ptr< ::mosek::fusion::Variable > _6166,std::shared_ptr< monty::ndarray< double,2 > > _6167);
static  monty::rc_ptr< ::mosek::fusion::Expression > mulElm(monty::rc_ptr< ::mosek::fusion::Variable > _6168,std::shared_ptr< monty::ndarray< double,1 > > _6169);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(monty::rc_ptr< ::mosek::fusion::Matrix > _6170,monty::rc_ptr< ::mosek::fusion::Expression > _6171);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(monty::rc_ptr< ::mosek::fusion::Matrix > _6172,monty::rc_ptr< ::mosek::fusion::Variable > _6173);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6174,monty::rc_ptr< ::mosek::fusion::Variable > _6175);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6176,monty::rc_ptr< ::mosek::fusion::Expression > _6177);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(std::shared_ptr< monty::ndarray< double,2 > > _6178,monty::rc_ptr< ::mosek::fusion::Variable > _6179);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(std::shared_ptr< monty::ndarray< double,2 > > _6180,monty::rc_ptr< ::mosek::fusion::Expression > _6181);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(std::shared_ptr< monty::ndarray< double,1 > > _6182,monty::rc_ptr< ::mosek::fusion::Variable > _6183);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(std::shared_ptr< monty::ndarray< double,1 > > _6184,monty::rc_ptr< ::mosek::fusion::Expression > _6185);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(monty::rc_ptr< ::mosek::fusion::Expression > _6186,monty::rc_ptr< ::mosek::fusion::Matrix > _6187);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(monty::rc_ptr< ::mosek::fusion::Expression > _6188,std::shared_ptr< monty::ndarray< double,2 > > _6189);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(monty::rc_ptr< ::mosek::fusion::Expression > _6190,std::shared_ptr< monty::ndarray< double,1 > > _6191);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(monty::rc_ptr< ::mosek::fusion::Expression > _6192,monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6193);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(monty::rc_ptr< ::mosek::fusion::Variable > _6194,monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6195);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(monty::rc_ptr< ::mosek::fusion::Variable > _6196,monty::rc_ptr< ::mosek::fusion::Matrix > _6197);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(monty::rc_ptr< ::mosek::fusion::Variable > _6198,std::shared_ptr< monty::ndarray< double,2 > > _6199);
static  monty::rc_ptr< ::mosek::fusion::Expression > dot(monty::rc_ptr< ::mosek::fusion::Variable > _6200,std::shared_ptr< monty::ndarray< double,1 > > _6201);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6202,monty::rc_ptr< ::mosek::fusion::Variable > _6203);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Variable > _6204,monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6205);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Matrix > _6206,monty::rc_ptr< ::mosek::fusion::Variable > _6207);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Variable > _6208,monty::rc_ptr< ::mosek::fusion::Matrix > _6209);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(double _6210,monty::rc_ptr< ::mosek::fusion::Variable > _6211);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Variable > _6212,double _6213);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(std::shared_ptr< monty::ndarray< double,2 > > _6214,monty::rc_ptr< ::mosek::fusion::Variable > _6215);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(std::shared_ptr< monty::ndarray< double,1 > > _6216,monty::rc_ptr< ::mosek::fusion::Variable > _6217);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Variable > _6218,std::shared_ptr< monty::ndarray< double,2 > > _6219);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Variable > _6220,std::shared_ptr< monty::ndarray< double,1 > > _6221);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Variable > _6222,monty::rc_ptr< ::mosek::fusion::Variable > _6223);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6224,monty::rc_ptr< ::mosek::fusion::Expression > _6225);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Expression > _6226,monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6227);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Matrix > _6228,monty::rc_ptr< ::mosek::fusion::Expression > _6229);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Expression > _6230,monty::rc_ptr< ::mosek::fusion::Matrix > _6231);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(double _6232,monty::rc_ptr< ::mosek::fusion::Expression > _6233);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Expression > _6234,double _6235);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(std::shared_ptr< monty::ndarray< double,2 > > _6236,monty::rc_ptr< ::mosek::fusion::Expression > _6237);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(std::shared_ptr< monty::ndarray< double,1 > > _6238,monty::rc_ptr< ::mosek::fusion::Expression > _6239);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Expression > _6240,std::shared_ptr< monty::ndarray< double,2 > > _6241);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Expression > _6242,std::shared_ptr< monty::ndarray< double,1 > > _6243);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Variable > _6244,monty::rc_ptr< ::mosek::fusion::Expression > _6245);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Expression > _6246,monty::rc_ptr< ::mosek::fusion::Variable > _6247);
static  monty::rc_ptr< ::mosek::fusion::Expression > sub(monty::rc_ptr< ::mosek::fusion::Expression > _6248,monty::rc_ptr< ::mosek::fusion::Expression > _6249);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6250,monty::rc_ptr< ::mosek::fusion::Variable > _6251);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Variable > _6252,monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6253);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Matrix > _6254,monty::rc_ptr< ::mosek::fusion::Variable > _6255);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Variable > _6256,monty::rc_ptr< ::mosek::fusion::Matrix > _6257);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(double _6258,monty::rc_ptr< ::mosek::fusion::Variable > _6259);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Variable > _6260,double _6261);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(std::shared_ptr< monty::ndarray< double,2 > > _6262,monty::rc_ptr< ::mosek::fusion::Variable > _6263);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(std::shared_ptr< monty::ndarray< double,1 > > _6264,monty::rc_ptr< ::mosek::fusion::Variable > _6265);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Variable > _6266,std::shared_ptr< monty::ndarray< double,2 > > _6267);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Variable > _6268,std::shared_ptr< monty::ndarray< double,1 > > _6269);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Variable > _6270,monty::rc_ptr< ::mosek::fusion::Variable > _6271);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6272,monty::rc_ptr< ::mosek::fusion::Expression > _6273);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Expression > _6274,monty::rc_ptr< ::mosek::fusion::NDSparseArray > _6275);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Matrix > _6276,monty::rc_ptr< ::mosek::fusion::Expression > _6277);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Expression > _6278,monty::rc_ptr< ::mosek::fusion::Matrix > _6279);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(double _6280,monty::rc_ptr< ::mosek::fusion::Expression > _6281);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Expression > _6282,double _6283);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(std::shared_ptr< monty::ndarray< double,2 > > _6284,monty::rc_ptr< ::mosek::fusion::Expression > _6285);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(std::shared_ptr< monty::ndarray< double,1 > > _6286,monty::rc_ptr< ::mosek::fusion::Expression > _6287);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Expression > _6288,std::shared_ptr< monty::ndarray< double,2 > > _6289);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Expression > _6290,std::shared_ptr< monty::ndarray< double,1 > > _6291);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Variable > _6292,monty::rc_ptr< ::mosek::fusion::Expression > _6293);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Expression > _6294,monty::rc_ptr< ::mosek::fusion::Variable > _6295);
static  monty::rc_ptr< ::mosek::fusion::Expression > add(monty::rc_ptr< ::mosek::fusion::Expression > _6296,monty::rc_ptr< ::mosek::fusion::Expression > _6297);
virtual monty::rc_ptr< ::mosek::fusion::Set > shape();
virtual monty::rc_ptr< ::mosek::fusion::Set > getShape();
virtual monty::rc_ptr< ::mosek::fusion::Model > getModel();
static  void validateData(std::shared_ptr< monty::ndarray< long long,1 > > _6298,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _6299,std::shared_ptr< monty::ndarray< long long,1 > > _6300,std::shared_ptr< monty::ndarray< double,1 > > _6301,std::shared_ptr< monty::ndarray< double,1 > > _6302,monty::rc_ptr< ::mosek::fusion::Set > _6303,std::shared_ptr< monty::ndarray< long long,1 > > _6304);
static  monty::rc_ptr< ::mosek::fusion::Model > extractModel(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _6319);
}; // struct Expr;

struct p_FlatExpr
{
FlatExpr * _pubthis;
static mosek::fusion::p_FlatExpr* _get_impl(mosek::fusion::FlatExpr * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_FlatExpr * _get_impl(mosek::fusion::FlatExpr::t _inst) { return _get_impl(_inst.get()); }
p_FlatExpr(FlatExpr * _pubthis);
virtual ~p_FlatExpr() { /* std::cout << "~p_FlatExpr" << std::endl;*/ };
std::shared_ptr< monty::ndarray< long long,1 > > inst{};monty::rc_ptr< ::mosek::fusion::Set > shape{};long long nnz{};std::shared_ptr< monty::ndarray< double,1 > > cof{};std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > x{};std::shared_ptr< monty::ndarray< long long,1 > > subj{};std::shared_ptr< monty::ndarray< long long,1 > > ptrb{};std::shared_ptr< monty::ndarray< double,1 > > bfix{};virtual void destroy();
static FlatExpr::t _new_FlatExpr(monty::rc_ptr< ::mosek::fusion::FlatExpr > _6320);
void _initialize(monty::rc_ptr< ::mosek::fusion::FlatExpr > _6320);
static FlatExpr::t _new_FlatExpr(std::shared_ptr< monty::ndarray< double,1 > > _6321,std::shared_ptr< monty::ndarray< long long,1 > > _6322,std::shared_ptr< monty::ndarray< long long,1 > > _6323,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _6324,std::shared_ptr< monty::ndarray< double,1 > > _6325,monty::rc_ptr< ::mosek::fusion::Set > _6326,std::shared_ptr< monty::ndarray< long long,1 > > _6327);
void _initialize(std::shared_ptr< monty::ndarray< double,1 > > _6321,std::shared_ptr< monty::ndarray< long long,1 > > _6322,std::shared_ptr< monty::ndarray< long long,1 > > _6323,std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Variable >,1 > > _6324,std::shared_ptr< monty::ndarray< double,1 > > _6325,monty::rc_ptr< ::mosek::fusion::Set > _6326,std::shared_ptr< monty::ndarray< long long,1 > > _6327);
virtual std::string toString();
virtual int size();
}; // struct FlatExpr;

struct p_SymmetricMatrix
{
SymmetricMatrix * _pubthis;
static mosek::fusion::p_SymmetricMatrix* _get_impl(mosek::fusion::SymmetricMatrix * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_SymmetricMatrix * _get_impl(mosek::fusion::SymmetricMatrix::t _inst) { return _get_impl(_inst.get()); }
p_SymmetricMatrix(SymmetricMatrix * _pubthis);
virtual ~p_SymmetricMatrix() { /* std::cout << "~p_SymmetricMatrix" << std::endl;*/ };
int nnz{};double scale{};std::shared_ptr< monty::ndarray< double,1 > > vval{};std::shared_ptr< monty::ndarray< int,1 > > vsubj{};std::shared_ptr< monty::ndarray< int,1 > > vsubi{};std::shared_ptr< monty::ndarray< double,1 > > uval{};std::shared_ptr< monty::ndarray< int,1 > > usubj{};std::shared_ptr< monty::ndarray< int,1 > > usubi{};int d1{};int d0{};virtual void destroy();
static SymmetricMatrix::t _new_SymmetricMatrix(int _6329,int _6330,std::shared_ptr< monty::ndarray< int,1 > > _6331,std::shared_ptr< monty::ndarray< int,1 > > _6332,std::shared_ptr< monty::ndarray< double,1 > > _6333,std::shared_ptr< monty::ndarray< int,1 > > _6334,std::shared_ptr< monty::ndarray< int,1 > > _6335,std::shared_ptr< monty::ndarray< double,1 > > _6336,double _6337);
void _initialize(int _6329,int _6330,std::shared_ptr< monty::ndarray< int,1 > > _6331,std::shared_ptr< monty::ndarray< int,1 > > _6332,std::shared_ptr< monty::ndarray< double,1 > > _6333,std::shared_ptr< monty::ndarray< int,1 > > _6334,std::shared_ptr< monty::ndarray< int,1 > > _6335,std::shared_ptr< monty::ndarray< double,1 > > _6336,double _6337);
static  monty::rc_ptr< ::mosek::fusion::SymmetricMatrix > rankOne(int _6338,std::shared_ptr< monty::ndarray< int,1 > > _6339,std::shared_ptr< monty::ndarray< double,1 > > _6340);
static  monty::rc_ptr< ::mosek::fusion::SymmetricMatrix > rankOne(std::shared_ptr< monty::ndarray< double,1 > > _6348);
static  monty::rc_ptr< ::mosek::fusion::SymmetricMatrix > antiDiag(std::shared_ptr< monty::ndarray< double,1 > > _6356);
static  monty::rc_ptr< ::mosek::fusion::SymmetricMatrix > diag(std::shared_ptr< monty::ndarray< double,1 > > _6363);
virtual monty::rc_ptr< ::mosek::fusion::SymmetricMatrix > add(monty::rc_ptr< ::mosek::fusion::SymmetricMatrix > _6369);
virtual monty::rc_ptr< ::mosek::fusion::SymmetricMatrix > sub(monty::rc_ptr< ::mosek::fusion::SymmetricMatrix > _6389);
virtual monty::rc_ptr< ::mosek::fusion::SymmetricMatrix > mul(double _6390);
virtual int getdim();
}; // struct SymmetricMatrix;

struct p_NDSparseArray
{
NDSparseArray * _pubthis;
static mosek::fusion::p_NDSparseArray* _get_impl(mosek::fusion::NDSparseArray * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_NDSparseArray * _get_impl(mosek::fusion::NDSparseArray::t _inst) { return _get_impl(_inst.get()); }
p_NDSparseArray(NDSparseArray * _pubthis);
virtual ~p_NDSparseArray() { /* std::cout << "~p_NDSparseArray" << std::endl;*/ };
std::shared_ptr< monty::ndarray< double,1 > > cof{};std::shared_ptr< monty::ndarray< long long,1 > > inst{};std::shared_ptr< monty::ndarray< int,1 > > dims{};long long size{};virtual void destroy();
static NDSparseArray::t _new_NDSparseArray(std::shared_ptr< monty::ndarray< int,1 > > _6391,std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< int,1 > >,1 > > _6392,std::shared_ptr< monty::ndarray< double,1 > > _6393);
void _initialize(std::shared_ptr< monty::ndarray< int,1 > > _6391,std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< int,1 > >,1 > > _6392,std::shared_ptr< monty::ndarray< double,1 > > _6393);
static NDSparseArray::t _new_NDSparseArray(std::shared_ptr< monty::ndarray< int,1 > > _6413,std::shared_ptr< monty::ndarray< long long,1 > > _6414,std::shared_ptr< monty::ndarray< double,1 > > _6415);
void _initialize(std::shared_ptr< monty::ndarray< int,1 > > _6413,std::shared_ptr< monty::ndarray< long long,1 > > _6414,std::shared_ptr< monty::ndarray< double,1 > > _6415);
static NDSparseArray::t _new_NDSparseArray(monty::rc_ptr< ::mosek::fusion::Matrix > _6429);
void _initialize(monty::rc_ptr< ::mosek::fusion::Matrix > _6429);
static  monty::rc_ptr< ::mosek::fusion::NDSparseArray > make(monty::rc_ptr< ::mosek::fusion::Matrix > _6437);
static  monty::rc_ptr< ::mosek::fusion::NDSparseArray > make(std::shared_ptr< monty::ndarray< int,1 > > _6438,std::shared_ptr< monty::ndarray< long long,1 > > _6439,std::shared_ptr< monty::ndarray< double,1 > > _6440);
static  monty::rc_ptr< ::mosek::fusion::NDSparseArray > make(std::shared_ptr< monty::ndarray< int,1 > > _6441,std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< int,1 > >,1 > > _6442,std::shared_ptr< monty::ndarray< double,1 > > _6443);
}; // struct NDSparseArray;

struct p_Matrix
{
Matrix * _pubthis;
static mosek::fusion::p_Matrix* _get_impl(mosek::fusion::Matrix * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_Matrix * _get_impl(mosek::fusion::Matrix::t _inst) { return _get_impl(_inst.get()); }
p_Matrix(Matrix * _pubthis);
virtual ~p_Matrix() { /* std::cout << "~p_Matrix" << std::endl;*/ };
int dimj{};int dimi{};virtual void destroy();
static Matrix::t _new_Matrix(int _6512,int _6513);
void _initialize(int _6512,int _6513);
virtual std::string toString();
virtual void switchDims();
static  monty::rc_ptr< ::mosek::fusion::Matrix > diag(int _6515,monty::rc_ptr< ::mosek::fusion::Matrix > _6516);
static  monty::rc_ptr< ::mosek::fusion::Matrix > diag(std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Matrix >,1 > > _6518);
static  monty::rc_ptr< ::mosek::fusion::Matrix > antidiag(int _6536,double _6537,int _6538);
static  monty::rc_ptr< ::mosek::fusion::Matrix > antidiag(int _6539,double _6540);
static  monty::rc_ptr< ::mosek::fusion::Matrix > diag(int _6541,double _6542,int _6543);
static  monty::rc_ptr< ::mosek::fusion::Matrix > diag(int _6544,double _6545);
static  monty::rc_ptr< ::mosek::fusion::Matrix > antidiag(std::shared_ptr< monty::ndarray< double,1 > > _6546,int _6547);
static  monty::rc_ptr< ::mosek::fusion::Matrix > antidiag(std::shared_ptr< monty::ndarray< double,1 > > _6557);
static  monty::rc_ptr< ::mosek::fusion::Matrix > diag(std::shared_ptr< monty::ndarray< double,1 > > _6558,int _6559);
static  monty::rc_ptr< ::mosek::fusion::Matrix > diag(std::shared_ptr< monty::ndarray< double,1 > > _6567);
static  monty::rc_ptr< ::mosek::fusion::Matrix > ones(int _6568,int _6569);
static  monty::rc_ptr< ::mosek::fusion::Matrix > eye(int _6570);
static  monty::rc_ptr< ::mosek::fusion::Matrix > dense(monty::rc_ptr< ::mosek::fusion::Matrix > _6572);
static  monty::rc_ptr< ::mosek::fusion::Matrix > dense(int _6573,int _6574,double _6575);
static  monty::rc_ptr< ::mosek::fusion::Matrix > dense(int _6576,int _6577,std::shared_ptr< monty::ndarray< double,1 > > _6578);
static  monty::rc_ptr< ::mosek::fusion::Matrix > dense(std::shared_ptr< monty::ndarray< double,2 > > _6579);
static  monty::rc_ptr< ::mosek::fusion::Matrix > sparse(monty::rc_ptr< ::mosek::fusion::Matrix > _6580);
static  monty::rc_ptr< ::mosek::fusion::Matrix > sparse(std::shared_ptr< monty::ndarray< std::shared_ptr< monty::ndarray< monty::rc_ptr< ::mosek::fusion::Matrix >,1 > >,1 > > _6584);
static  monty::rc_ptr< ::mosek::fusion::Matrix > sparse(std::shared_ptr< monty::ndarray< double,2 > > _6615);
static  monty::rc_ptr< ::mosek::fusion::Matrix > sparse(int _6628,int _6629);
static  monty::rc_ptr< ::mosek::fusion::Matrix > sparse(int _6630,int _6631,std::shared_ptr< monty::ndarray< int,1 > > _6632,std::shared_ptr< monty::ndarray< int,1 > > _6633,double _6634);
static  monty::rc_ptr< ::mosek::fusion::Matrix > sparse(std::shared_ptr< monty::ndarray< int,1 > > _6636,std::shared_ptr< monty::ndarray< int,1 > > _6637,double _6638);
static  monty::rc_ptr< ::mosek::fusion::Matrix > sparse(std::shared_ptr< monty::ndarray< int,1 > > _6643,std::shared_ptr< monty::ndarray< int,1 > > _6644,std::shared_ptr< monty::ndarray< double,1 > > _6645);
static  monty::rc_ptr< ::mosek::fusion::Matrix > sparse(int _6650,int _6651,std::shared_ptr< monty::ndarray< int,1 > > _6652,std::shared_ptr< monty::ndarray< int,1 > > _6653,std::shared_ptr< monty::ndarray< double,1 > > _6654);
virtual double get(int _6659,int _6660) { throw monty::AbstractClassError("Call to abstract method"); }
virtual bool isSparse() { throw monty::AbstractClassError("Call to abstract method"); }
virtual std::shared_ptr< monty::ndarray< double,1 > > getDataAsArray() { throw monty::AbstractClassError("Call to abstract method"); }
virtual void getDataAsTriplets(std::shared_ptr< monty::ndarray< int,1 > > _6661,std::shared_ptr< monty::ndarray< int,1 > > _6662,std::shared_ptr< monty::ndarray< double,1 > > _6663) { throw monty::AbstractClassError("Call to abstract method"); }
virtual monty::rc_ptr< ::mosek::fusion::Matrix > transpose() { throw monty::AbstractClassError("Call to abstract method"); }
virtual long long numNonzeros() { throw monty::AbstractClassError("Call to abstract method"); }
virtual int numColumns();
virtual int numRows();
}; // struct Matrix;

struct p_DenseMatrix : public ::mosek::fusion::p_Matrix
{
DenseMatrix * _pubthis;
static mosek::fusion::p_DenseMatrix* _get_impl(mosek::fusion::DenseMatrix * _inst){ return static_cast< mosek::fusion::p_DenseMatrix* >(mosek::fusion::p_Matrix::_get_impl(_inst)); }
static mosek::fusion::p_DenseMatrix * _get_impl(mosek::fusion::DenseMatrix::t _inst) { return _get_impl(_inst.get()); }
p_DenseMatrix(DenseMatrix * _pubthis);
virtual ~p_DenseMatrix() { /* std::cout << "~p_DenseMatrix" << std::endl;*/ };
long long nnz{};std::shared_ptr< monty::ndarray< double,1 > > data{};virtual void destroy();
static DenseMatrix::t _new_DenseMatrix(int _6444,int _6445,std::shared_ptr< monty::ndarray< double,1 > > _6446);
void _initialize(int _6444,int _6445,std::shared_ptr< monty::ndarray< double,1 > > _6446);
static DenseMatrix::t _new_DenseMatrix(monty::rc_ptr< ::mosek::fusion::Matrix > _6447);
void _initialize(monty::rc_ptr< ::mosek::fusion::Matrix > _6447);
static DenseMatrix::t _new_DenseMatrix(std::shared_ptr< monty::ndarray< double,2 > > _6452);
void _initialize(std::shared_ptr< monty::ndarray< double,2 > > _6452);
static DenseMatrix::t _new_DenseMatrix(int _6455,int _6456,double _6457);
void _initialize(int _6455,int _6456,double _6457);
virtual std::string toString();
virtual monty::rc_ptr< ::mosek::fusion::Matrix > transpose();
virtual bool isSparse();
virtual std::shared_ptr< monty::ndarray< double,1 > > getDataAsArray();
virtual void getDataAsTriplets(std::shared_ptr< monty::ndarray< int,1 > > _6470,std::shared_ptr< monty::ndarray< int,1 > > _6471,std::shared_ptr< monty::ndarray< double,1 > > _6472);
virtual double get(int _6476,int _6477);
virtual long long numNonzeros();
}; // struct DenseMatrix;

struct p_SparseMatrix : public ::mosek::fusion::p_Matrix
{
SparseMatrix * _pubthis;
static mosek::fusion::p_SparseMatrix* _get_impl(mosek::fusion::SparseMatrix * _inst){ return static_cast< mosek::fusion::p_SparseMatrix* >(mosek::fusion::p_Matrix::_get_impl(_inst)); }
static mosek::fusion::p_SparseMatrix * _get_impl(mosek::fusion::SparseMatrix::t _inst) { return _get_impl(_inst.get()); }
p_SparseMatrix(SparseMatrix * _pubthis);
virtual ~p_SparseMatrix() { /* std::cout << "~p_SparseMatrix" << std::endl;*/ };
long long nnz{};std::shared_ptr< monty::ndarray< double,1 > > val{};std::shared_ptr< monty::ndarray< int,1 > > subj{};std::shared_ptr< monty::ndarray< int,1 > > subi{};virtual void destroy();
static SparseMatrix::t _new_SparseMatrix(int _6478,int _6479,std::shared_ptr< monty::ndarray< int,1 > > _6480,std::shared_ptr< monty::ndarray< int,1 > > _6481,std::shared_ptr< monty::ndarray< double,1 > > _6482,long long _6483);
void _initialize(int _6478,int _6479,std::shared_ptr< monty::ndarray< int,1 > > _6480,std::shared_ptr< monty::ndarray< int,1 > > _6481,std::shared_ptr< monty::ndarray< double,1 > > _6482,long long _6483);
static SparseMatrix::t _new_SparseMatrix(int _6488,int _6489,std::shared_ptr< monty::ndarray< int,1 > > _6490,std::shared_ptr< monty::ndarray< int,1 > > _6491,std::shared_ptr< monty::ndarray< double,1 > > _6492);
void _initialize(int _6488,int _6489,std::shared_ptr< monty::ndarray< int,1 > > _6490,std::shared_ptr< monty::ndarray< int,1 > > _6491,std::shared_ptr< monty::ndarray< double,1 > > _6492);
virtual std::shared_ptr< monty::ndarray< long long,1 > > formPtrb();
virtual std::string toString();
virtual long long numNonzeros();
virtual monty::rc_ptr< ::mosek::fusion::Matrix > transpose();
virtual bool isSparse();
virtual std::shared_ptr< monty::ndarray< double,1 > > getDataAsArray();
virtual void getDataAsTriplets(std::shared_ptr< monty::ndarray< int,1 > > _6504,std::shared_ptr< monty::ndarray< int,1 > > _6505,std::shared_ptr< monty::ndarray< double,1 > > _6506);
virtual double get(int _6507,int _6508);
}; // struct SparseMatrix;

struct p_Parameters
{
Parameters * _pubthis;
static mosek::fusion::p_Parameters* _get_impl(mosek::fusion::Parameters * _inst){ assert(_inst); assert(_inst->_impl); return _inst->_impl; }
static mosek::fusion::p_Parameters * _get_impl(mosek::fusion::Parameters::t _inst) { return _get_impl(_inst.get()); }
p_Parameters(Parameters * _pubthis);
virtual ~p_Parameters() { /* std::cout << "~p_Parameters" << std::endl;*/ };
virtual void destroy();
static  void setParameter(monty::rc_ptr< ::mosek::fusion::Model > _6687,const std::string &  _6688,double _6689);
static  void setParameter(monty::rc_ptr< ::mosek::fusion::Model > _6799,const std::string &  _6800,int _6801);
static  void setParameter(monty::rc_ptr< ::mosek::fusion::Model > _6911,const std::string &  _6912,const std::string &  _6913);
static  int string_to_miocontsoltype_value(const std::string &  _7182);
static  int string_to_internal_dinf_value(const std::string &  _7183);
static  int string_to_presolvemode_value(const std::string &  _7184);
static  int string_to_optimizertype_value(const std::string &  _7185);
static  int string_to_xmlwriteroutputtype_value(const std::string &  _7186);
static  int string_to_iinfitem_value(const std::string &  _7187);
static  int string_to_simreform_value(const std::string &  _7188);
static  int string_to_stakey_value(const std::string &  _7189);
static  int string_to_value_value(const std::string &  _7190);
static  int string_to_scalingmethod_value(const std::string &  _7191);
static  int string_to_soltype_value(const std::string &  _7192);
static  int string_to_startpointtype_value(const std::string &  _7193);
static  int string_to_language_value(const std::string &  _7194);
static  int string_to_checkconvexitytype_value(const std::string &  _7195);
static  int string_to_variabletype_value(const std::string &  _7196);
static  int string_to_mpsformat_value(const std::string &  _7197);
static  int string_to_nametype_value(const std::string &  _7198);
static  int string_to_compresstype_value(const std::string &  _7199);
static  int string_to_simdupvec_value(const std::string &  _7200);
static  int string_to_dparam_value(const std::string &  _7201);
static  int string_to_inftype_value(const std::string &  _7202);
static  int string_to_problemtype_value(const std::string &  _7203);
static  int string_to_orderingtype_value(const std::string &  _7204);
static  int string_to_dataformat_value(const std::string &  _7205);
static  int string_to_simdegen_value(const std::string &  _7206);
static  int string_to_onoffkey_value(const std::string &  _7207);
static  int string_to_transpose_value(const std::string &  _7208);
static  int string_to_mionodeseltype_value(const std::string &  _7209);
static  int string_to_rescode_value(const std::string &  _7210);
static  int string_to_scalingtype_value(const std::string &  _7211);
static  int string_to_prosta_value(const std::string &  _7212);
static  int string_to_rescodetype_value(const std::string &  _7213);
static  int string_to_parametertype_value(const std::string &  _7214);
static  int string_to_dinfitem_value(const std::string &  _7215);
static  int string_to_miomode_value(const std::string &  _7216);
static  int string_to_msgkey_value(const std::string &  _7217);
static  int string_to_simseltype_value(const std::string &  _7218);
static  int string_to_internal_liinf_value(const std::string &  _7219);
static  int string_to_iomode_value(const std::string &  _7220);
static  int string_to_streamtype_value(const std::string &  _7221);
static  int string_to_conetype_value(const std::string &  _7222);
static  int string_to_mark_value(const std::string &  _7223);
static  int string_to_feature_value(const std::string &  _7224);
static  int string_to_symmattype_value(const std::string &  _7225);
static  int string_to_callbackcode_value(const std::string &  _7226);
static  int string_to_simhotstart_value(const std::string &  _7227);
static  int string_to_liinfitem_value(const std::string &  _7228);
static  int string_to_branchdir_value(const std::string &  _7229);
static  int string_to_basindtype_value(const std::string &  _7230);
static  int string_to_internal_iinf_value(const std::string &  _7231);
static  int string_to_boundkey_value(const std::string &  _7232);
static  int string_to_solitem_value(const std::string &  _7233);
static  int string_to_objsense_value(const std::string &  _7234);
static  int string_to_solsta_value(const std::string &  _7235);
static  int string_to_iparam_value(const std::string &  _7236);
static  int string_to_sparam_value(const std::string &  _7237);
static  int string_to_intpnthotstart_value(const std::string &  _7238);
static  int string_to_uplo_value(const std::string &  _7239);
static  int string_to_sensitivitytype_value(const std::string &  _7240);
static  int string_to_accmode_value(const std::string &  _7241);
static  int string_to_problemitem_value(const std::string &  _7242);
static  int string_to_solveform_value(const std::string &  _7243);
}; // struct Parameters;

}
}
namespace mosek
{
namespace fusion
{
namespace Utils
{
// mosek.fusion.Utils.IntMap from file 'src/fusion/cxx/IntMap_p.h'
struct p_IntMap 
{
  IntMap * _pubself;

  static p_IntMap * _get_impl(IntMap * _inst) { return _inst->_impl.get(); }

  p_IntMap(IntMap * _pubself) : _pubself(_pubself) {}

  static IntMap::t _new_IntMap() { return new IntMap(); }

  ::std::unordered_map<long long,int> m;

  bool hasItem (long long key) { return m.find(key) != m.end(); }
  int  getItem (long long key) { return m.find(key)->second; } // will probably throw something or crash of no such key
  void setItem (long long key, int val) { m[key] = val; }

  std::shared_ptr<monty::ndarray<long long,1>> keys()
  { 
    size_t size = m.size();
    auto res = std::shared_ptr<monty::ndarray<long long,1>>(new monty::ndarray<long long,1>(monty::shape((int)size)));

    ptrdiff_t i = 0;
    for (auto it = m.begin(); it != m.end(); ++it)
      (*res)[i++] = it->first;

    return res;    
  }

  std::shared_ptr<monty::ndarray<int,1>> values()
  {
    size_t size = m.size();
    auto res = std::shared_ptr<monty::ndarray<int,1>>(new monty::ndarray<int,1>(monty::shape((int)size)));

    ptrdiff_t i = 0;
    for (auto it = m.begin(); it != m.end(); ++it)
      (*res)[i++] = it->second;

    return res;
  }

  IntMap::t clone();
};



struct p_StringIntMap
{
  StringIntMap * _pubself;

  static p_StringIntMap * _get_impl(StringIntMap * _inst) { return _inst->_impl.get(); }

  p_StringIntMap(StringIntMap * _pubself) : _pubself(_pubself) {}

  static StringIntMap::t _new_StringIntMap() { return new StringIntMap(); }

  ::std::unordered_map<std::string,int> m;

  bool hasItem (const std::string & key) { return m.find(key) != m.end(); }
  int  getItem (const std::string & key) { return m.find(key)->second; } // will probably throw something or crash of no such key
  void setItem (const std::string & key, int val) { m[key] = val; }

  std::shared_ptr<monty::ndarray<std::string,1>> keys()
  {
    size_t size = m.size();
    auto res = std::shared_ptr<monty::ndarray<std::string,1>>(new monty::ndarray<std::string,1>(monty::shape((int)size)));

    ptrdiff_t i = 0;
    for (auto it = m.begin(); it != m.end(); ++it)
      (*res)[i++] = it->first;

    return res;
  }

  std::shared_ptr<monty::ndarray<int,1>> values()
  {
    size_t size = m.size();
    auto res = std::shared_ptr<monty::ndarray<int,1>>(new monty::ndarray<int,1>(monty::shape((int)size)));

    ptrdiff_t i = 0;
    for (auto it = m.begin(); it != m.end(); ++it)
      (*res)[i++] = it->second;

    return res;
  }

  StringIntMap::t clone();
};
// End of file 'src/fusion/cxx/IntMap_p.h'
// mosek.fusion.Utils.StringBuffer from file 'src/fusion/cxx/StringBuffer_p.h'
// namespace mosek::fusion::Utils
struct p_StringBuffer
{
  StringBuffer * _pubthis; 
  std::stringstream ss;

  p_StringBuffer(StringBuffer * _pubthis) : _pubthis(_pubthis) {}

  static p_StringBuffer * _get_impl(StringBuffer::t ptr) { return ptr->_impl.get(); }
  static p_StringBuffer * _get_impl(StringBuffer * ptr) { return ptr->_impl.get(); }

  static StringBuffer::t _new_StringBuffer() { return new StringBuffer(); }

  StringBuffer::t clear ();

  template<typename T> StringBuffer::t a (const monty::ndarray<T,1> & val);

  template<typename T> StringBuffer::t a (const T & val);

  StringBuffer::t lf ();
  std::string toString () const;
};

template<typename T> 
StringBuffer::t p_StringBuffer::a(const monty::ndarray<T,1> & val)
{
  if (val.size() > 0)
  {
    ss << val[0];
    for (int i = 1; i < val.size(); ++i)
      ss << "," << val[i];
  }
  return StringBuffer::t(_pubthis);
}
  
template<typename T>
StringBuffer::t p_StringBuffer::a (const T & val)
{
  ss << val;
  return _pubthis;
}


// End of file 'src/fusion/cxx/StringBuffer_p.h'
}
}
}
#endif
